#<profile4xi profile = "33" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "forkingpickler4xi.py">
#<abstract4xi abstract = "provides class ForkingPickler4Xi"/>
#<refer4xi reference = "forking.py; pickle.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = ["ForkingPickler4Xi", "PickleError", "PicklingError", "UnpicklingError", "dump", "dumps", "load", "loads"]
#"
#/>           

# -------------------------------------------------------------------------------------------------
# Standard Imports                           
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import time         
except:
    pass
try:                 # rare ones
    import codecs 
    import _compat_pickle
    from copyreg import dispatch_table 
    from copyreg import _extension_registry, _inverted_registry, _extension_cache
    import io
    import marshal
    import re
    import struct 
    from types import FunctionType, BuiltinFunctionType         
except:
    pass 
try:
    from org.python.core import PyStringMap
except ImportError:
    PyStringMap = None
    
try:                 # import as     
    import binascii as _binascii
except:
    pass 

# dragonized by embedding all classes and functions from /lib/pickle.py into this module forkingpickler4xi.py 
# therefore no imports for those modules is being done here
#"
#/>                 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
from multiprocessing.connection import Connection
from multiprocessing import forking
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from print4xi import print4xi
from time4xi import time4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
#<define4xi definition = "

# Shortcut for use in isinstance testing

bytes_types = (bytes, bytearray)

# These are purely informational; no code uses these.
format_version = "3.0"                  # File format version we write
compatible_formats = ["1.0",            # Original protocol 0
                      "1.1",            # Protocol 0 with INST added
                      "1.2",            # Original protocol 1
                      "1.3",            # Protocol 1 with BINFLOAT added
                      "2.0",            # Protocol 2
                      "3.0",            # Protocol 3
                      ]                 # Old format versions we can read
# These are purely informational; no code uses these.
format_version = "3.0"                  # File format version we write
compatible_formats = ["1.0",            # Original protocol 0
                      "1.1",            # Protocol 0 with INST added
                      "1.2",            # Original protocol 1
                      "1.3",            # Protocol 1 with BINFLOAT added
                      "2.0",            # Protocol 2
                      "3.0",            # Protocol 3
                      ]                 # Old format versions we can read
                      
# This is the highest protocol number we know how to read.
HIGHEST_PROTOCOL = 3

# The protocol we write by default.  May be less than HIGHEST_PROTOCOL.
# We intentionally write a protocol that Python 2.x cannot read;
# there are too many issues with that.
DEFAULT_PROTOCOL = 3

# Why use struct.pack() for pickling but marshal.loads() for
# unpickling?  struct.pack() is 40% faster than marshal.dumps(), but
# marshal.loads() is twice as fast as struct.unpack()!
mloads = marshal.loads
#"
#/>

# -------------------------------------------------------------------------------------------------
#<class4xi class = "PickleError" base = "Exception" child = "None"
#   synopsis = "A common base class for the other pickling exceptions"
#   usage = "embedded in forkingpickler4xi.py"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
class PickleError(Exception):
    """A common base class for the other pickling exceptions."""
    
    pass
#</class4xi>

# -------------------------------------------------------------------------------------------------
#<class4xi class = "PicklingError" base = "PickleError" child = "None"
#   synopsis = "Exception to be raised when an unpicklable object is passed to the dump method"
#   usage = "embedded in forkingpickler4xi.py"
#> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
class PicklingError(PickleError):
    """This exception is raised when an unpicklable object is passed to the
    dump() method.

    """
    pass
#</class4xi>
    
# -------------------------------------------------------------------------------------------------
#<class4xi class = "UnpicklingError" base = "PickleError" child = "None"
#   synopsis = "Exception to be raised when there is a problem unpickling an object"
#   usage = "embedded in forkingpickler4xi.py"
#> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
class UnpicklingError(PickleError):
    """This exception is raised when there is a problem unpickling an object,
    such as a security violation.

    Note that other exceptions may also be raised during unpickling, including
    (but not necessarily limited to) AttributeError, EOFError, ImportError,
    and IndexError.

    """
    pass
#</class4xi>
    
# -------------------------------------------------------------------------------------------------
#<class4xi class = "_Stop" base = "Exception" child = "None"
#   synopsis = "An instance of this class is raised by _Unpickler.load_stop"
#   usage = "embedded in forkingpickler4xi.py"
#> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# -------------------------------------------------------------------------------------------------
class _Stop(Exception):

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Stop" usage = "initializes _Stop">
    # ---------------------------------------------------------------------------------------------    
    def __init__(self, value):
    
        self.value = value
    #</methodize4xi>
          
#</class4xi>
                


# -------------------------------------------------------------------------------------------------
#<define4xi definition = "
# Pickle opcodes.  See pickletools.py for extensive docs.  The listing
# here is in kind-of alphabetical order of 1-character pickle code.
# pickletools groups them by purpose.

MARK           = b'('   # push special markobject on stack
STOP           = b'.'   # every pickle ends with STOP
POP            = b'0'   # discard topmost stack item
POP_MARK       = b'1'   # discard stack top through topmost markobject
DUP            = b'2'   # duplicate top stack item
FLOAT          = b'F'   # push float object; decimal string argument
INT            = b'I'   # push integer or bool; decimal string argument
BININT         = b'J'   # push four-byte signed int
BININT1        = b'K'   # push 1-byte unsigned int
LONG           = b'L'   # push long; decimal string argument
BININT2        = b'M'   # push 2-byte unsigned int
NONE           = b'N'   # push None
PERSID         = b'P'   # push persistent object; id is taken from string arg
BINPERSID      = b'Q'   #  "       "         "  ;  "  "   "     "  stack
REDUCE         = b'R'   # apply callable to argtuple, both on stack
STRING         = b'S'   # push string; NL-terminated string argument
BINSTRING      = b'T'   # push string; counted binary string argument
SHORT_BINSTRING= b'U'   #  "     "   ;    "      "       "      " < 256 bytes
UNICODE        = b'V'   # push Unicode string; raw-unicode-escaped'd argument
BINUNICODE     = b'X'   #   "     "       "  ; counted UTF-8 string argument
APPEND         = b'a'   # append stack top to list below it
BUILD          = b'b'   # call __setstate__ or __dict__.update()
GLOBAL         = b'c'   # push self.find_class(modname, name); 2 string args
DICT           = b'd'   # build a dict from stack items
EMPTY_DICT     = b'}'   # push empty dict
APPENDS        = b'e'   # extend list on stack by topmost stack slice
GET            = b'g'   # push item from memo on stack; index is string arg
BINGET         = b'h'   #   "    "    "    "   "   "  ;   "    " 1-byte arg
INST           = b'i'   # build & push class instance
LONG_BINGET    = b'j'   # push item from memo on stack; index is 4-byte arg
LIST           = b'l'   # build list from topmost stack items
EMPTY_LIST     = b']'   # push empty list
OBJ            = b'o'   # build & push class instance
PUT            = b'p'   # store stack top in memo; index is string arg
BINPUT         = b'q'   #   "     "    "   "   " ;   "    " 1-byte arg
LONG_BINPUT    = b'r'   #   "     "    "   "   " ;   "    " 4-byte arg
SETITEM        = b's'   # add key+value pair to dict
TUPLE          = b't'   # build tuple from topmost stack items
EMPTY_TUPLE    = b')'   # push empty tuple
SETITEMS       = b'u'   # modify dict by adding topmost key+value pairs
BINFLOAT       = b'G'   # push float; arg is 8-byte float encoding

TRUE           = b'I01\n'  # not an opcode; see INT docs in pickletools.py
FALSE          = b'I00\n'  # not an opcode; see INT docs in pickletools.py

# Protocol 2

PROTO          = b'\x80'  # identify pickle protocol
NEWOBJ         = b'\x81'  # build object by applying cls.__new__ to argtuple
EXT1           = b'\x82'  # push object from extension registry; 1-byte index
EXT2           = b'\x83'  # ditto, but 2-byte index
EXT4           = b'\x84'  # ditto, but 4-byte index
TUPLE1         = b'\x85'  # build 1-tuple from stack top
TUPLE2         = b'\x86'  # build 2-tuple from two topmost stack items
TUPLE3         = b'\x87'  # build 3-tuple from three topmost stack items
NEWTRUE        = b'\x88'  # push True
NEWFALSE       = b'\x89'  # push False
LONG1          = b'\x8a'  # push long from < 256 bytes
LONG4          = b'\x8b'  # push really big long

_tuplesize2code = [EMPTY_TUPLE, TUPLE1, TUPLE2, TUPLE3]

#"
#/>

# -------------------------------------------------------------------------------------------------
#<define4xi definition = "
# Protocol 3 (Python 3.x)

BINBYTES       = b'B'   # push bytes; counted binary string argument
SHORT_BINBYTES = b'C'   #  "     "   ;    "      "       "      " < 256 bytes

__all__.extend([x for x in dir() if re.match("[A-Z][A-Z0-9_]+$",x)])
#"
#/>





# =================================================================================================
#<class4xi class = "ForkingPickler4Xi" base = "Pickler" child = "None"
#  synopsis = "Takes a binary file for writing a pickle data stream"
#  usage = "
#     connection.py:
#         ForkingPickler(buf, pickle.HIGHEST_PROTOCOL).dump(obj)
#         ForkingPickler.register(socket.socket, reduction.reduce_socket)
#         ForkingPickler.register(Connection, reduction.reduce_connection)
#         ForkingPickler.register(PipeConnection, reduction.reduce_pipe_connection)
#         ForkingPickler.register(socket.socket, reduction.reduce_socket)
#         
#     forking.py
#         ForkingPickler.register(type(_C().f), _reduce_method ace:)
#         ForkingPickler.register(type(list.append), _reduce_method_descriptor)
#         ForkingPickler.register(type(int.__add__), _reduce_method_descriptor)
#
#     managers.py:
#         ForkingPickler.register(array.array, reduce_array)
#         ForkingPickler.register(view_type, rebuild_as_list)
#
#     reduction.py:
#         ForkingPickler.register(Connection, reduce_connection)
#         ForkingPickler.register(Connection, reduce_connection)
#         ForkingPickler.register(socket.socket, reduce_socket)
#         ForkingPickler.register(PipeConnection, reduce_pipe_connection)
#
#     sharedctypes.py:
#         ForkingPickler.register(type_, reduce_ctype)
#         
#     connection4xi.py: <-- _connectionbase4xi.py has some code
#         ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
#         ForkingPickler4Xi.register(Connection4Xi, reduction.reduce_connection)
#         ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
#
#     pickle4xi.py:
#         ForkingPickler4Xi.register(view_type2xi, rebuild_as_list4xi)
#
#     pipeconnection4xi.py:
#         ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
#         ForkingPickler4Xi.register(PipeConnection4Xi, reduction.reduce_connection)
#         ForkingPickler4Xi.register(socket.socket, reduction.reduce_socket)
#         ForkingPickler4Xi.register(PipeConnection4Xi, reduction.reduce_connection)
#
#     popen4xi.py:
#         ForkingPickler4Xi(file, protocol).dump(obj)
#     dump4xi.py:
#         ForkingPickler4Xi(file, protocol).dump(obj)
#     process4xi.py
#         _bootstrap(self):
#             ForkingPickler4Xi.register(type(_C4Xi().f), _reduce_method4xi) 
#             ForkingPickler4Xi.register(type(list.append), _reduce_method_descriptor4xi)
#             ForkingPickler4Xi.register(type(int.__add__), _reduce_method_descriptor4xi)
#             ForkingPickler4Xi.register(partial, _reduce_partial4xi)
#             
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
#<note4xi note = "Tailors both _Pickler and _Unpickler class of module /lib/pickle4xi.py  and therefore for example includes __init__"/>
# =================================================================================================
class ForkingPickler4Xi(object):                          # dragonized to inherit from object
    ''' 
        Takes a binary file for writing a pickle data stream.
        All protocols now read and write bytes.
        The default protocol is 3; a backward-incompatible protocol designed for Python 3.0.
        The file argument must have a write() method that accepts a single bytes argument. 
        It can thus be a file object opened for binary writing, a io.BytesIO instance, or any other custom object that meets this interface.
    '''
   
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('save_global','save_dict','save_list','save_tuple','save_str','save_bytes','save_float','save_long',
                 'save_bool','save_none','save_reduce','save_pers','persistent_id','save','get','put',
                 'clear_memo','register','dump','sense', 'reason', 'ace') 
      
    #"
    #/>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch = {}               #dragonized dispatch = Pickler.dispatch.copy() 
    #"
    #/>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-ForkingPickler4Xi" usage = "initializes ForkingPickler4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, file, protocol=None, fix_imports=False):
    
        # -----------------------------------------------------------------------------------------
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------        
        
        class2print = "ForkingPickler4Xi"
        method2print = "init"
        
        message2print = "{(Step 1/7a) started" + "; init-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 20:46:00 02/10/10 FLE Standard Time
        
        message2print = "{(Step 1/7b) passed" + "; file-passed-to-init-of-ForkingPickler4Xi: " + str(file) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # _io.BufferedWriter name=4
        
        message2print = "{(Step 1/7c) passed" + "; protocol-passed-to-init-of-ForkingPickler4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        # 3
        
        message2print = "{(Step 1/7d) borning" + "; forkingpickler2xi-of-ForkingPickler4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # finalizer4xi.Finalizer4Xi object at 0x02FDE070      

        message2print = "{(Step 1/7e) inheriting" + "; object-inheriting-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class '_pickle.Pickler'>
        
        message2print = "{(Step 1/7e) sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(ForkingPickler4Xi.dispatch)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                       
        #HW:
        # {<class 'bytes'>: <function save_bytes at 0x02B50930>, 
        #  <class 'float'>: <function save_float at 0x02B508E8>,
        #  <class 'bool'>: <function save_bool at 0x02B50858>, 
        #  <class 'str'>: <function save_str at 0x02B50978>, 
        #  <class 'builtin_function_or_method'>: # <function save_global at 0x02B50B28>, 
        #  <class 'list'>: <function save_list at 0x02B50A08>, 
        #  <class 'tuple'>: <function save_tuple at 0x02B509C0>, 
        #  <class 'int'>: <function save_long at 0x02B508A0>, 
        #  <class 'dict'>: <function save_dict at 0x02B50A98>,
        #  <class 'NoneType'>: <function save_none at 0x02B50810>, 
        #  <class 'type'>: <function save_global at 0x02B50B28>, 
        #  <class 'function'>: <function save_global at 0x02B50B28>
                                    
        # -----------------------------------------------------------------------------------------
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------        
        if protocol is None:
            protocol = DEFAULT_PROTOCOL
            
            message2print = "{(Step 2/7a) set" + "; protocol-set-by-init-of-ForkingPickler4Xi to: " + str(protocol)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                            
        if protocol < 0:
            protocol = HIGHEST_PROTOCOL
            
            message2print = "{(Step 2/7b) set" + "; protocol-set-by-init-of-ForkingPickler4Xi to: " + str(protocol)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
               
        elif not 0 <= protocol <= HIGHEST_PROTOCOL:
            message2print = "{(Step 2/7c) yielded" + "; protocol-yielded-by-init-of-ForkingPickler4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                        

        try:
            self.write = file.write
            
            message2print = "{(Step 2/7d) set" + "; write-of-forkingpickler2xi-set-by-init-of-ForkingPickler4Xi to: " + str(self.write)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #HW: <built-in method write of _io.BufferedWriter object at 0x02C0F0B0>
            
        except AttributeError:
            message2print = "{(Step 2/7e) yielded" + "; TypeError-yielded-by-init-of-ForkingPickler4Xi: " + " *** file with no write attribut: " + str(file) + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)


        
    
        # -----------------------------------------------------------------------------------------
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------        
                
        self.memo = {}
 
        message2print = "{(Step 3/7) set" + "; memo-set-by-init-of-ForkingPickler4Xi to: " + str(self.memo)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: {} 
    
        # -----------------------------------------------------------------------------------------
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------        
            
        self.proto = int(protocol)

        message2print = "{(Step 4/7) set" + "; proto-set-by-init-of-ForkingPickler4Xi to: " + str(self.proto)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        #HW: 3 
    
        # -----------------------------------------------------------------------------------------
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------        
            
        self.bin = protocol >= 1

        message2print = "{(Step 5/7) set" + "; binary-protocol-logical-value-set-by-init-of-ForkingPickler4Xi to: " + str(self.bin)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        #HW: True 
    
        # -----------------------------------------------------------------------------------------
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------        
            
        self.fast = 0    

        message2print = "{(Step 6/7) set" + "; fast-set-by-init-of-ForkingPickler4Xi to: " + str(self.fast)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        #HW: 0 
    
        # -----------------------------------------------------------------------------------------
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 7/7a) inherited" + "; object-inherited-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #HW: <class '_pickle.Pickler'> before upgrade
                            
        message2print = "{(Step 7/7b) activated" + "; forkingpickler2xi-of-ForkingPickler4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW: forkingpickler4xi.ForkingPickler4Xi object at 0x02CECFD0
        
        message2print = "{(Step 7/7c) shared" + "; dispatch-shared-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(ForkingPickler4Xi.dispatch)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        #HW: <class 'bytes'>: <function save_bytes at 0x02B50930>, <class 'float'>: <function save_float at 0x02B508E8>, <class 'bool'>: <function save_bool at 0x02B50858>, <class 'str'>: <function save_str at 0x02B50978>, <class 'builtin_function_or_method'>: <function save_global at 0x02B50B28>, <class 'list'>: <function save_list at 0x02B50A08>, <class 'tuple'>: <function save_tuple at 0x02B509C0>, <class 'int'>: <function save_long at 0x02B508A0>, <class 'dict'>: <function save_dict at 0x02B50A98>, <class 'NoneType'>: <function save_none at 0x02B50810>, <class 'type'>: <function save_global at 0x02B50B28>, <class 'function'>: <function save_global at 0x02B50B28>
        
 
             
        message2print = "{(Step 7/7d) returning" + "; init-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #HW: 00:16:54 03/16/10 FLE Standard Time
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "register-of-ForkingPickler4Xi" usage = "registers with (cls, type, reduce)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    @classmethod
    def register(cls, type, reduce):
        '''
        ForkingPickler4Xi method register: registers with (cls, type, reduce)
        '''   
        # ---------------------------------------------------------------------------------------------
        # 
        # ---------------------------------------------------------------------------------------------
        class2print = "ForkingPickler4Xi"
        method2print = "register"
        
        message2print = "{started" + "; register-of-ForkingPickler4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        
        message2print = "{passed" + "; type-passed-to-register-of-ForkingPickler4Xi: " + str(type) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # class 'functools.partial' 
        
        message2print = "{passed" + "; reduce-passed-to-register-of-ForkingPickler4Xi: " + str(reduce) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # function _reduce_method_descriptor4xi at 0x02C61270 
                  
        # ---------------------------------------------------------------------------------------------
        #<function4xi function = "dispatcher-of-ForkingPickler4Xi" usage = "inside register method#"> 
        #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
        # ---------------------------------------------------------------------------------------------
        def dispatcher(self, obj):
        
            # -----------------------------------------------------------------------------------------
            # (Step 1/4) 
            # -----------------------------------------------------------------------------------------          
            
            class2print = "ForkingPickler4Xi"
            method2print = "dispatcher"
            
            message2print = "{(Step 1/4a) started" + "; dispatcher-of-ForkingPickler4Xi-started: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
            
           
            message2print = "{(Step 1/4b) passed" + "; object-passed-to-dispatcher-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/4) 
            # -----------------------------------------------------------------------------------------          
                 
            rv2xi = reduce(obj)
            
            message2print = "{(Step 2/4a) set" + "; rv2xi-set-by-dispatcher-of-ForkingPickler4Xi to: " + str(rv2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
       
            message2print = "{(Step 2/4b) set" + "; second-set-by-dispatcher-of-ForkingPickler4Xi to: " + str(second)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
        
            # -----------------------------------------------------------------------------------------
            # (Step 3/4) 
            # -----------------------------------------------------------------------------------------          
                                        
            if isinstance(rv2xi, str):
                self.save_global(obj, rv2xi)
                 
                message2print = "{(Step 3/4a) called" + "; save-global-called-by-dispatcher-of-ForkingPickler4Xi: " + str(self.save_global) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
            else:
                self.save_reduce(obj=obj, *rv2xi)
                    
                message2print = "{(Step 3/4b) called" + "; save-reduce-called-by-dispatcher-of-ForkingPickler4Xi: " + str(self.save_reduce) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                              
        
            # -----------------------------------------------------------------------------------------
            # (Step 4/4) 
            # -----------------------------------------------------------------------------------------          
     
            message2print = "{(Step 4/4) returning" + "; dispatcher-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
            # 11:47:35 02/11/10 FLE Standard Time    
                
        #</function4xi>         
        
        # ---------------------------------------------------------------------------------------------
        # 
        # ---------------------------------------------------------------------------------------------                
        message2print = "{(Step 5/a) setting" + "; class-dispacth-set-by-dispatcher-of-ForkingPickler4Xi to: " + str(dispatcher)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # function dispatcher at 0x02AE6CD8
        
        cls.dispatch[type] = dispatcher 

        message2print = "{(Step 5/b) set" + "; class-dispatch-type-set-by-dispatcher-of-ForkingPickler4Xi to: " + str(cls.dispatch[type])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # function dispatcher at 0x02AE6C90
                
        # -----------------------------------------------------------------------------------------    
        # 
        # -----------------------------------------------------------------------------------------   
        message2print = "{returning" + "; dispatcher-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        # 11:47:35 02/11/10 FLE Standard Time 
        
    #</methodize4xi>
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "dump-of-ForkingPickler4Xi" usage = "Write a pickled representation of obj to the open file">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------        
    def dump(self, obj):
        """Write a pickled representation of obj to the open file."""
        
        # -------------------------------------------------------------------------------------    
        #
        # -------------------------------------------------------------------------------------         
 
        class2print = "ForkingPickler4Xi"
        method2print = "dump" 
        
        message2print = "{started" + "; dump-of-ForkingPickler4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
             
        message2print = "{passed" + "; object-passed-to-dump-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)               
         
        #HW: {'authkey': b'\xc4\x17\xd42\xef\x1a\xd1\xed \x1dr~h\x8e\xfd\x9a\xfaO\x81S\xae\xdfBF\x07\x84\x1b\x16[\xc0\xa1Q',
        #    'sys_path': ['E:\\Sw4Xi\\Python32\\Lib\\multiprocessing4xi', 'd:\\Python32', 'D:\\Windows\\system32\\python32.zip', 'd:\\Python32\\DLLs', 'd:\\Python32\\lib', 'd:\\Python32\\lib\\site-packages'], 
        #        'name': 'Process4Xi-1', 'orig_dir': 'E:\\Sw4Xi\\Python32\\Lib\\multiprocessing4xi', 
        #    'sys_argv': ['hello_world4xi.py'], 'main_path': 'E:\\Sw4Xi\\Python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 'log_to_stderr': True}; time: 01:24:51 02/18/11 Pacific Standard Time}

        
        
        # -------------------------------------------------------------------------------------    
        # OK 
        # -------------------------------------------------------------------------------------         
        message2print = "{examined" + "; proto-examined-by-dump-of-ForkingPickler4Xi: " + str(self.proto) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        #HW 1. and 2. : 3 
        
        # -------------------------------------------------------------------------------------    
        # OK
        # -------------------------------------------------------------------------------------         
        
        if self.proto >= 2:
            message2print = "{calling" + "; write-calling-by-dump-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HW 1. and 2. :  built-in method write of _io.BufferedWriter object at 0x02BA2CB0 
            
            self.write(PROTO + bytes([self.proto]))
            
            message2print = "{performed" + "; write-performed-by-dump-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #HW 1. and 2. :  built-in method write of _io.BufferedWriter object at 0x02BA2CB0
                                   
        # -------------------------------------------------------------------------------------    
        # OK
        # -------------------------------------------------------------------------------------         
        message2print = "{calling" + "; save-calling-by-dump-of-ForkingPickler4Xi: " + str(self.save) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW 1. and 2. : <bound method ForkingPickler4Xi.save of <forkingpickler4xi.ForkingPickler4Xi object at 0x02B13B70>>  
        
        self.save(obj) 
        
        message2print = "{performed" + "; save-performed-by-dump-of-ForkingPickler4Xi: " + str(self.save) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        #HW 1. : bound method ForkingPickler4Xi.save of forkingpickler4xi.ForkingPickler4Xi object at 0x02BCE250 
        #HW 2. :  
                                
        # -------------------------------------------------------------------------------------    
        # OK
        # -------------------------------------------------------------------------------------         
        message2print = "{passing" + "; STOP-passing-to-write-by-dump-of-ForkingPickler4Xi: " + str(STOP) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # b'.'     
       
        # -------------------------------------------------------------------------------------    
        # OK 
        # -------------------------------------------------------------------------------------         
        message2print = "{calling" + "; write-calling-by-dump-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # built-in method write of _io.BufferedWriter object at 0x02BA2CB0 
        
        self.write(STOP)
        
        message2print = "{performed" + "; write-performed-by-dump-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # built-in method write of _io.BufferedWriter object at 0x02BA2CB0                        
                        
        # -------------------------------------------------------------------------------------    
        # OK
        # ------------------------------------------------------------------------------------- 
                
        message2print = "{returning" + "; dump-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #  11:47:35 02/11/10 FLE Standard Time 
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "memoize-of-ForkingPickler4Xi" usage = "To store an object in the memo">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------  
    def memoize(self, obj):
        """
        Stores an object in the memo.
        """
        
        # -------------------------------------------------------------------------------------    
        #
        # -------------------------------------------------------------------------------------         
 
        class2print = "ForkingPickler4Xi"
        method2print = "memoize" 
             
        # The Pickler memo is a dictionary, which maps object ids to 2-tuples.
        # 2-tuple contains the Unpickler memo key and the object being memoized.
        # The memo key is written to the pickle and will become the key in the Unpickler's memo.
        # The object is stored in the Pickler memo so that transient objects are kept alive during pickling.

        # The use of the Unpickler memo length as the memo key is just a convention.
        # The only requirement is that the memo values be unique.
        # There appears no advantage to any other scheme.
        # This selected scheme allows the Unpickler memo to be implemented as a plain (but growable) array, indexed by memo key.
        
        message2print = "{started" + "; memoize-of-ForkingPickler4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: 18:07:07 04/07/10 FLE Daylight Time                 
                                
        
        if self.fast:                                                        # set in __init__ to 0
            pass
        else:                
            # -------------------------------------------------------------------------------------    
            # 
            # ------------------------------------------------------------------------------------- 
                    
            if id(obj) in self.memo:                #initialized in __init__ and updated by methods
                message2print = "{yielded" + "; id-obj-yielded-by-memoize-of-ForkingPickler4Xi: " + str(id(obj))  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
                            
            # -------------------------------------------------------------------------------------    
            # 
            # ------------------------------------------------------------------------------------- 
                
            memo_len = len(self.memo)
            
            message2print = "{set" + "; memo-length-set-by-memoize-of-ForkingPickler4Xi to: " + str(memo_len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #HW: 0
                            
            # -------------------------------------------------------------------------------------    
            # 
            # ------------------------------------------------------------------------------------- 
            ack2memowrite = False
            nack2memowrite = True
            
            try:    
                self.write(self.put(memo_len))
                
                nack2memowrite = False
                ack2memowrite = True
                
                message2print = "{acknowledged" + "; ack2memowrite-acknowledged-by-memoize-of-ForkingPickler4Xi: " + str(ack2memowrite) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                          
                #HW: <built-in method write of _io.BufferedWriter object at 0x02B8E830>
                
            except:
                message2print = "{yielded" + "; nack2memowrite-yielded-by-memoize-of-ForkingPickler4Xi: " + str(nack2memowrite)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                                            
            # -------------------------------------------------------------------------------------    
            # 
            # ------------------------------------------------------------------------------------- 
            ack2memo = False
            nack2memo = True
            
            try:                   
                
            
                self.memo[id(obj)] = memo_len, obj
            
                message2print = "{added" + "; memo-of-forkingpickler2xi-added-by-memoize-of-ForkingPickler4Xi: " + str(self.memo[id(obj)]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                #HW: (0, {'authkey': b'\x9d\x9cq\xb8(#!S\x15O\xf7\xf7\x9e\xe77\xd8\x8aG\xf6/\x13\xcd\xf6\xee\xff\xeb\x84\x96 \xbcJ\xd8', 
                #        'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'],
                #            'name': 'Process4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 
                #        'sys_argv': ['hello_world4xi.py'], 
                #       'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
                #   'log_to_stderr': True}); time: 18:07:07 04/07/10 FLE Daylight Time}

                #    (5, <function f4xi at 0x02C7F618>)
                
                #PX: (0, {'authkey': b'\xbf\x8d\x03\t{\x01\xe4\xdb\xa7v7\\\xb1\x8e\xc5\xf7\xd1\xea\x17\xac\x1f\xd7<\x96\xc5\xff\xdf\xd03\xf4\x92\xce', 
                #        'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'], 
                #            'name': 'SyncManager4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 
                #        'sys_argv': ['test_proxy_mp_xi4.py'], 'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py',
                #   'log_to_stderr': True}); time: 18:26:27 04/07/10 FLE Daylight Time}
                #    (1, 'authkey')
                #    (2, b'\xbf\x8d\x03\t{\x01\xe4\xdb\xa7v7\\\xb1\x8e\xc5\xf7\xd1\xea\x17\xac\x1f\xd7<\x96\xc5\xff\xdf\xd03\xf4\x92\xce')
                #    (3, 'sys_path')
                #    (4, ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages']); time: 18:26:27 04/07/10 FLE Daylight Time}
                #    (5, 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi')
                #    (6, 'c:\\sw\\python32')...(12, 'name'), (13, 'SyncManager4Xi-1') ...(16, 'sys_argv'), (17, ['test_proxy_mp_xi4.py']), (18, 'test_proxy_mp_xi4.py'), (19, 'main_path')
                #    (20, 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py'), (21, 'log_to_stderr') 
                 
                #PX: (0, <Process4Xi(SyncManager4Xi-1, initial)>)
                #    (1, {'_daemonic': False, '_target': <bound method type._run_server of <class 'syncmanager4xi.SyncManager4Xi'>>,
                #             '_args': ({'AsyncResult': (None, None, None, <function AutoProxy4Xi at 0x02EC5ED0>), 
                #                       'Condition4Xi': (<function Condition at 0x02987738>, ('wait', 'notify', 'notify_all', 'sense', 'reason', 'ace'), None, <class 'conditionproxy4xi.ConditionProxy4Xi'>), 
                #                           'Event4Xi': (<function Event at 0x02987E40>, ('is_set', 'set', 'clear', 'wait', 'sense', 'reason', 'ace'), None, <class 'eventproxy4xi.EventProxy4Xi'>), 
                #                           'Iterator': (None, ('send', 'throw', 'close', 'sense', 'reason', 'ace'), None, <class 'iteratorproxy4xi.IteratorProxy4Xi'>), 
                #                'BoundedSemaphore4Xi': (<function BoundedSemaphore at 0x02987CD8>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                           'Array4Xi': (<function Array4Xi at 0x02EC58A0>, ('__len__', '__getitem__', '__setitem__'), None, <class 'arrayproxy4xi.ArrayProxy4Xi'>), 
                #                               'list': (<class 'list'>, ('sense', 'reason', 'ace'), None, <class 'listproxy4xi.ListProxy4Xi'>), 
                #                           'Queue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x02EC5ED0>), 
                #                            'Pool4Xi': (<class 'pool4xi.Pool4Xi'>, ('apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join', 'map', 'map_async', 'terminate', 'sense', 'reason', 'ace'), {'apply_async': 'AsyncResult', 'imap_unordered': 'Iterator', 'imap': 'Iterator', 'map_async': 'AsyncResult'}, <class 'poolproxy4xi.PoolProxy4Xi'>), 
                #                            'Lock4Xi': (<built-in function allocate_lock>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                       'Semaphore4Xi': (<function Semaphore at 0x029879C0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                           'RLock4Xi': (<function RLock at 0x029876F0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                               'dict': (<class 'dict'>, ('__contains__', '__delitem__', '__getitem__', '__len__', '__setitem__', 'clear', 'copy', 'get', 'has_key', 'items', 'keys', 'pop', 'popitem', 'setdefault', 'update', 'values', 'sense', 'reason', 'ace'), None, <class 'dictproxy4xi.DictProxy4Xi'>), 
                #                   'JoinableQueue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x02EC5ED0>), 
                #                           'Value4Xi': (<class 'value4xi.Value4Xi'>, ('get', 'set', 'sense', 'reason', 'ace'), None, <class 'valueproxy4xi.ValueProxy4Xi'>), 
                #                       'Namespace4Xi': (<class 'namespace4xi.Namespace4Xi'>, ('__getattribute__', '__setattr__', '__delattr__', 'sense', 'reason', 'ace'), None, <class 'namespaceproxy4xi.NamespaceProxy4Xi'>)
                #                       },
                #         '\\\\.\\pipe\\pyc-1676-0-h6xe6x',
                #         b'\x8d\x19\x87\xc7\xdf\x85B\x1f\xe5\xbeG\xf0\xcd"\x8b}BWs>\x92\xf3\xeb+\\O\x9a\x7f\x0b\xb7\xa2%', 'pickle',
                #         <write-only PipeConnection, handle 420>),
                #         '_tempdir': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-kylvgw', 
                #         '_name': 'SyncManager4Xi-1', 
                #         '_authkey': b'\xbf\x8d\x03\t{\x01\xe4\xdb\xa7v7\\\xb1\x8e\xc5\xf7\xd1\xea\x17\xac\x1f\xd7<\x96\xc5\xff\xdf\xd03\xf4\x92\xce', 
                #         '_parent_pid': 1676,
                #         '_kwargs': {},
                #         '_identity': (1,), 
                #         '_popen': None}
                #    )
            
                nack2memo = False
                ack2memo = True
                
                message2print = "{acknowledged" + "; ack2memo-acknowledged-by-memoize-of-ForkingPickler4Xi: " + str(ack2memo) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
            except:
                message2print = "{yielded" + "; nack2memo-yielded-by-memoize-of-ForkingPickler4Xi: " + str(nack2memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                  
        # -------------------------------------------------------------------------------------    
        #
        # -------------------------------------------------------------------------------------         
                 
        message2print = "{returning" + "; memoize-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HW: 18:07:07 04/07/10 FLE Daylight Time        
        
                
    #</methodize4xi>
    
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "clear-memo-of-ForkingPickler4Xi" usage = "(self)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def clear_memo(self):
        """Clears the pickler's "memo".

        The memo is the data structure that remembers which objects the  pickler has already seen, so that shared or recursive objects are
        pickled by reference and not by value.  This method is useful when re-using picklers.
        """
        
        # -------------------------------------------------------------------------------------    
        # 
        # -------------------------------------------------------------------------------------          
        
        class2print = "ForkingPickler4Xi"
        method2print = "clear-memo"  
        
        message2print = "{started" + "; clear-memo-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        self.memo.clear()   
        
            
        message2print = "{performed" + "; memo-clear-performed-by-clear-memo-of-ForkingPickler4Xi: " + str(self.memo.clear) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                               
        message2print = "{returning" + "; clear-memo-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        
        # -------------------------------------------------------------------------------------    
        # 
        # -------------------------------------------------------------------------------------          
                            
                
                                
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "put-of-ForkingPickler4Xi" usage = "(self, i, pack=struct.pack)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------                
    def put(self, i, pack=struct.pack):
        
        class2print = "ForkingPickler4Xi"
        method2print = "put"      
        
        # -------------------------------------------------------------------------------------    
        # 
        # -------------------------------------------------------------------------------------          
            
        message2print = "{started" + "; put-of-ForkingPickler4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        # forkingpickler4xi.ForkingPickler4Xi object at 0x02AF3E30
             
        message2print = "{passed" + "; index-passed-to-put-of-ForkingPickler4Xi: " + str(i) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 0-21 
        
        message2print = "{passed" + "; pack-passed-to-put-of-ForkingPickler4Xi: " + str(pack) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # built-in function pack 
        
        # -------------------------------------------------------------------------------------    
        # 
        # -------------------------------------------------------------------------------------          
            
        if self.bin:                                              #binary-protocol-logical-value
            if i < 256:
      
                message2print = "{(Step 1/3) returning" + "; BINPUT-returning-by-put-of-ForkingPickler4Xi: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                # 11:47:35 02/11/10 FLE Standard Time
                
                
                return BINPUT + bytes([i])
            else:
                message2print = "{(Step 2/3) returning" + "; LONG-BINPUT-returning-by-put-of-ForkingPickler4Xi: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                        
                
                return LONG_BINPUT + pack("<i", i)

        
        
        # -------------------------------------------------------------------------------------    
        # 
        # -------------------------------------------------------------------------------------          
           
        message2print = "{(Step 3/3) returning" + "; PUT-returning-by-put-of-ForkingPickler4Xi: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)
        #  11:47:35 02/11/10 FLE Standard Time   
    
        return PUT + repr(i).encode("ascii") + b'\n'
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-of-ForkingPickler4Xi" usage = "(self, i, pack=struct.pack)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------        
    def get(self, i, pack=struct.pack):
        
        # -------------------------------------------------------------------------------------    
        # 
        # -------------------------------------------------------------------------------------          
          
        class2print = "ForkingPickler4Xi"
        method2print = "get"      
      
        message2print = "{started" + "; get-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
             
        message2print = "{passed" + "; i-passed-to-get-of-ForkingPickler4Xi: " + str(i) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        #      
        message2print = "{passed" + "; pack-passed-to-get-of-ForkingPickler4Xi: " + str(pack) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #           

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
      
        if self.bin:                                                 #binary-protocol-logical-value
            if i < 256:
           
                message2print = "{(Step 1/3) returning" + "; BINGET-returning-by-get-of-ForkingPickler4Xi: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)    
                #                  
                
                return BINGET + bytes([i])
            else:
                message2print = "{(Step 2/3) returning" + "; LONG-BINGET-returning-by-get-of-ForkingPickler4Xi: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
                #      
                           
                return LONG_BINGET + pack("<i", i)
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 3/3) returning" + "; GET-returning-by-get-of-ForkingPickler4Xi: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        #             
        
        return GET + repr(i).encode("ascii") + b'\n'        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-of-ForkingPickler4Xi" usage = "(self, obj, save_persistent_id=True)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------  
    def save(self, obj, save_persistent_id=True):
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------      
        class2print = "ForkingPickler4Xi"
        method2print = "save"             
        
         
        message2print = "{(Step 1/9a) started" + "; save-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # forkingpickler4xi.ForkingPickler4Xi object at 0x02AF3E30
             
        message2print = "{(Step 1/9b) passed" + "; object-passed-to-save-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        #HW: 1.
        #  {'authkey': "b'\\xfd\\xc5+\\xb8\\xea\\xba:\\xb4\\x14W\\xe1\\xa1a\\xbb-i\\xced-w\\x98d\\xe0\\x82\\xd8J\\x9d\\xb0B4\\xeb\\x02'",
        #      'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'],
        #          'name': 'Process4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 
        #      'sys_argv': ['hello_world4xi.py'], 
        #     'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
        # 'log_to_stderr': True}

        #HW: 2.
        #<Process4Xi(Process4Xi-1, initial)>
        
   
        message2print = "{(Step 1/9c) passed" + "; save-persistent-id-passed-to-save-of-ForkingPickler4Xi: " + str(save_persistent_id) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        #HW: 1. and 2. : True
               
        message2print = "{(Step 1/9d) calling" + "; persistent-id-calling-by-save-of-ForkingPickler4Xi: " + str(self.persistent_id) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 1. and 2. : bound method ForkingPickler4Xi.persistent_id of forkingpickler4xi.ForkingPickler4Xi object at 0x02B11E50
        
        pid = self.persistent_id(obj)
        
        message2print = "{(Step 1/9e) got" + "; pid-got-by-save-of-ForkingPickler4Xi: " + str(pid)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW: 1. and 2. : None             
        
        if pid is not None and save_persistent_id:
            # -----------------------------------------------------------------------------------------    
            # 
            # -----------------------------------------------------------------------------------------               
            message2print = "{(Step 1/9f) calling" + "; save-pers-calling-by-save-of-ForkingPickler4Xi: " + str(self.save_pers) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{(Step 1/9d) passing" + "; pid-passing-to-save-pers-by-save-of-ForkingPickler4Xi: " + str(pid) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
                              
            self.save_pers(pid)
            
            message2print = "{(Step 1/9g) performed" + "; save-pers-performed-by-save-of-ForkingPickler4Xi: " + str(self.save_pers) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            message2print = "{(Step 1/9h) returning" + "; save-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
          
            return
        
        # -----------------------------------------------------------------------------------------    
        # Step 2/9: Check the memo OK
        # -----------------------------------------------------------------------------------------                 
           
        message2print = "{(Step 2/9a1) calling" + "; memo-get-calling-by-save-of-ForkingPickler4Xi: " + str(self.memo.get) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 1. and 2. : built-in method get of dict object at 0x02D7CDB0
                        
        message2print = "{(Step 2/9a2) passing" + "; id-object-passing-to-memo-get-by-save-of-ForkingPickler4Xi: " + str(id(obj)) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 1. : 45121872, 2. : 45114992              

        
        # -----------------------------------------------------------------------------------------    
        # OK
        # -----------------------------------------------------------------------------------------                 
       
        x = self.memo.get(id(obj))
        
 
        message2print = "{(Step 2/9b) got" + "; x-got-by-save-of-ForkingPickler4Xi: " + str(x)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HWHW: 1. and 2. : None
        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        
        if x:
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                   
            message2print = "{(Step 2/9c) got" + "; x-got-by-save-of-ForkingPickler4Xi: " + str(x)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            # { ForkingPickler4Xi : save ; Step 2/9c: starting to write time: 11:22:39 07/03/09 FLE Daylight Time}
            # 25 { ForkingPickler4Xi : save ; Step 2/9c: starting to write time: 11:22:39 07/03/09 FLE Daylight Time}
            # 27
            message2print = "{(Step 2/9d) calling" + "; write-calling-by-save-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{(Step 2/9e) passing" + "; x0-passing-to-write-by-save-of-ForkingPickler4Xi: " + str(self.get(x[0])) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
                                                        
            self.write(self.get(x[0]))
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------               
            message2print = "{(Step 2/9f) performed" + "; write-performed-by-save-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                  
            message2print = "{(Step 2/9g) returning" + "; save-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                        
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------         
                                    
            return
        
        # -----------------------------------------------------------------------------------------    
        # Step 3/9: Check the type dispatch table OK
        # -----------------------------------------------------------------------------------------                 
        # message2print = "{(Step 3/9a) checking; " + "type: dispatch table" + "; time: " + time4xi() + "}"
        # print4xi(class2print, method2print, message2print)    

        
        # -----------------------------------------------------------------------------------------    
        # Step 3/9a: OK 
        # -----------------------------------------------------------------------------------------                 
       
        message2print = "{(Step 3/9a2) passing" + "; object-passing-to-type-by-save-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 1. {'authkey': "b'Bg\\xb5\\xf8\\xfc\\xe5\\x9d=\\x00\\xfe|\\xabK_\\xfa\\x9a\\xf7\\xc5\\xa3I\\xc7\\xd9\\xe2c\\x17t\\x91\\x92\\x143u\\xc3'",
        #    'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'], 
        #        'name': 'Process4Xi-1', 
        #    'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 
        #    'sys_argv': ['hello_world4xi.py'], 'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
        # 'log_to_stderr': True}</rdfs:comment>
        # and above in pieces ! 
        
        #HW: 2. : <Process4Xi(Process4Xi-1, initial)>
        #HW: x. : <class 'process4xi.Process4Xi'>
        #PX: {'authkey': b'\xd9\x1eHM\xe6v\xb0\x8a\x0eV\x8b\xee\x12ca\xc9\xb2n\xf4\x01H]\xab,j\xdb\x83\xc4C\xe0t7', 'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'], 'name': 'SyncManager4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'sys_argv': ['test_proxy_mp_xi4.py'], 'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py', 'log_to_stderr': True}; time: 16:32:57 04/06/10 FLE Daylight Time}
        
        
        
        t = type(obj)
        
        message2print = "{(Step 3/9a3) got" + "; type-got-by-save-of-ForkingPickler4Xi: " + str(t)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW 1. : class 'dict'; class 'str'; class 'list'; class 'bool'; class 'process4xi.Process4Xi';      
        #HW 2. : <class 'process4xi.Process4Xi'>
        #HW x. : <class 'type'>
        #PX: <class 'dict'>
                

                
        # -----------------------------------------------------------------------------------------    
        # Step 3/9b: depends on the previous step !!!
        # -----------------------------------------------------------------------------------------                  
    
        message2print = "{(Step 3/9b) got" + "; type-got-by-save-of-ForkingPickler4Xi: " + str(t)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW: 2. : class 'process4xi.Process4Xi'
        #HW: x. <class 'type'>
        #PX: <class 'dict'>
                
        # -----------------------------------------------------------------------------------------    
        # Step 3/9c get function OK
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 3/9c1) calling" + "; dispatch-get-calling-by-save-of-ForkingPickler4Xi: " + str(self.dispatch.get) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW 1. : <built-in method get of dict object at 0x02C150C0>
        #HW 2. : <built-in method get of dict object at 0x029F7C00>
        #HW x. : <built-in method get of dict object at 0x02CF4AE0>
        #PX:     <built-in method get of dict object at 0x02E72AE0>
        
        message2print = "{(Step 3/9c2) passing" + "; type-passing-to-dispatch-by-save-of-ForkingPickler4Xi: " + str(t) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 1. : class 'dict' ....       
        #HW: 2. : <class 'process4xi.Process4Xi'>    
        #HW: x. : <class 'type'>
        #PX: <class 'dict'>
        
        f = self.dispatch.get(t)
        
 
        message2print = "{(Step 3/9c3) got" + "; function-got-by-save-of-ForkingPickler4Xi: " + str(f)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW: 1. : <function save_str at 0x02C2C588>
        #     PX: <function save_dict at 0x02E96108>
            
        # -----------------------------------------------------------------------------------------    
        # (Step 3/9d)
        # -----------------------------------------------------------------------------------------                 
        
        if f:

            message2print = "{(Step 3/9d) calling" + "; function-calling-by-save-of-ForkingPickler4Xi: " + str(f) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)       
            #HW: 1. <function save_dict at 0x02D0CA08> 
            #PX:    <function save_dict at 0x02E96108>
            
            # -----------------------------------------------------------------------------------------    
            # OK
            # -----------------------------------------------------------------------------------------    
            try:
                f(self, obj) # Call unbound method with explicit self
                
                message2print = "{(Step 3/9e) performed" + "; function-performed-by-save-of-ForkingPickler4Xi: " + str(f) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #HW: <function save_global at 0x02B70348>:
                #        object-passed-to-save-global-of-ForkingPickler4Xi: <class 'process4xi.Process4Xi'> 
                #    <function save_tuple at 0x02B701E0>; ForkingPickler4Xi : 
                #        object-passed-to-save-tuple-of-ForkingPickler4Xi: () <-- looks empty !
                #        save_tuple ; (Step 5/23b) passing; EMPTY-TUPLE-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: b')'; 
                #   
            except:
            
                message2print = "{(Step 3/9f) hit" + "; function-hit-by-save-of-ForkingPickler4Xi: " + " *** " + str(f)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #HW: <function save_dict at 0x02B702B8>   
                            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
            message2print = "{(Step 3/9g) returning" + "; save-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
            # 13:54:51 02/11/10 FLE Standard Time
             
            return
        
        # -----------------------------------------------------------------------------------------    
        # Step 4/9: Check for a class with a custom metaclass; treat as regular class OK
        # -----------------------------------------------------------------------------------------                 
  
        message2print = "{(Step 4/9a) checking" + "; class-checking-by-save-of-ForkingPickler4Xi: " + str('a custom metaclass') + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)           
        #HW: 2. : a custom metaclass
        
        
        # -----------------------------------------------------------------------------------------    
        # Step 4/9b OK 
        # -----------------------------------------------------------------------------------------                 
        
        try:
    
            message2print = "{(Step 4/9b1) trying" + "; issubclass-trying-by-save-of-ForkingPickler4Xi: " + str(issubclass)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #HW: 2. :  <built-in function issubclass>
            
            message2print = "{(Step 4/9b2) passing" + "; type-passing-to-issubclass-by-save-of-ForkingPickler4Xi: " + str(t) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #HW: 2. :  <built-in function issubclass>   
            
            message2print = "{(Step 4/9b3) passing" + "; type-passing-to-issubclass-by-save-of-ForkingPickler4Xi: " + str(type) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #HW: 2. :  <class 'type'>;
            
            issc = issubclass(t, type)
            
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
            
        except TypeError: # t is not a class (old Boost; see SF #502085)
            issc = 0
            message2print = "{(Step 4/9b4) hit" + "; TypeError-hit-by-save-of-ForkingPickler4Xi: " + " *** " + str(TypeError)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
                        
        # -----------------------------------------------------------------------------------------    
        # Step 4/9c
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 4/9c1) got" + "; issc-got-by-save-of-ForkingPickler4Xi: " + str(issc)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW: 2. :  False
                            
        if issc:
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------            
            message2print = "{(Step 4/9c2) calling" + "; save-global-calling-by-save-of-ForkingPickler4Xi: " + str(self.save_global) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{(Step 4/9c3) passing" + "; object-passing-to-save-global-by-save-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------     
            self.save_global(obj)
            
            message2print = "{(Step 4/9c4) performed" + "; save-global-performed-by-save-of-ForkingPickler4Xi: " + str(self.save_global) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
            message2print = "{(Step 4/9c5) returning; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            
            return
            # -----------------------------------------------------------------------------------------    
            #
            # ----------------------------------------------------------------------------------------- 
                    
        # -----------------------------------------------------------------------------------------    
        # Step 5/9a: Check copyreg.dispatch_table - UNRESOLVED t multiprocessing.process.AuthenticationString !! <-- ERROR HERE !!!!
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 5/9a1) calling" + "; dispatch-table-get-calling-by-save-of-ForkingPickler4Xi: " + str(dispatch_table.get) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 2. :  <built-in method get of dict object at 0x025C14B0>
                       
        message2print = "{(Step 5/9a2) passing" + "; type-passing-to-dispatch-table-get-by-save-of-ForkingPickler4Xi: " + str(t) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 2. :  <class 'process4xi.Process4Xi'>
        
        # -----------------------------------------------------------------------------------------    
        # Step 5/9b passed 
        # -----------------------------------------------------------------------------------------                 
        
        reduce2xi = dispatch_table.get(t)
      
        message2print = "{(Step 5/9b) got" + "; reduce2xi-got-by-save-of-ForkingPickler4Xi: " + str(reduce2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW: 2. : None
        
        # -----------------------------------------------------------------------------------------    
        # 
        # -----------------------------------------------------------------------------------------                 
        
        if reduce2xi:
        
     
            message2print = "{(Step 5/9c1) calling" + "; reduce2xi-calling-by-save-of-ForkingPickler4Xi: " + str(reduce2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{(Step 5/9c2) passing" + "; object-passing-to-reduce2xi-by-save-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
             
            rv2xi = reduce2xi(obj)
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
        
            message2print = "{(Step 5/9d) got" + "; rv2xi-got-by-save-of-ForkingPickler4Xi: " + str(rv2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                    
            
        else:

            # -----------------------------------------------------------------------------------------    
            # Check for a __reduce_ex__ method, fall back to __reduce__
            # -----------------------------------------------------------------------------------------    
    
            message2print = "{(Step 5/9e) calling" + "; getattr-calling-by-save-of-ForkingPickler4Xi: " + str(getattr) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #HW: 2. : <built-in function getattr>
            
            reduce2xi = getattr(obj, "__reduce_ex__", None)
            
            message2print = "{(Step 5/9f) got" + "; reduce2xi-got-by-save-of-ForkingPickler4Xi: " + str(reduce2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HW: 2. : <built-in method __reduce_ex__ of Process4Xi object at 0x02B06670>
            
            
            if reduce2xi:
            
                
                # -------------------------------------------------------------------------------------    
                # to-be-solved: why multiprocessing.process.AuthenticationString instead of AuthenticationString4Xi ????? 
                # -------------------------------------------------------------------------------------           
                message2print = "{(Step 5/9g) calling" + "; reduce2xi-calling-by-save-of-ForkingPickler4Xi: " + str(reduce2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                 
                #HW: 2. : <built-in method __reduce_ex__ of Process4Xi object at 0x02B06670>
                
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------                 
                
                rv2xi = reduce2xi(self.proto)
                
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------                 

                message2print = "{(Step 5/9h) got" + "; rv2xi-got-by-save-of-ForkingPickler4Xi: " + str(rv2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
                #HW: 2. : (<function __newobj__ at 0x024E5BB8>, 
                #         (<class 'process4xi.Process4Xi'>,),
                #         {'_daemonic': False,
                #            '_target': <function f4xi at 0x02AF97C8>,
                #              '_args': (), 
                #           '_tempdir': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-g0ruto', 
                #              '_name': 'Process4Xi-1',
                #           '_authkey': 'b">y\\xc6=\\xe9\\xddB\\xd0\'\\xc5p)\\x10F\\xb0\\xb0\\xa1\\xc8\\xe7\\xa3\\xbf\\x95\\x18g\\xfeP\\x17\\xfbn\\xba\\x9a\\x02"', 
                #        '_parent_pid': 3716, 
                #            '_kwargs': {}, 
                #          '_identity': (1,),
                #             '_popen': None
                #         },
                #         None, None)

                #tp
                # { ForkingPickler4Xi : save ; Step 5/9h: got rv2xi: 
                #      (<function __newobj__ at 0x0000000002C9A6C8>, 
                #      (<class 'queue4xi.Queue4Xi'>,), (2147483647, <read-only PipeConnection, handle %Id>, <write-only PipeConnection, handle %Id>, 
                #       <Lock(owner=None)>, None, 
                #       <BoundedSemaphore4Xi(value4xi=2147483647, maxvalue=2147483647)>, 3728), None, None); time: 19:13:00 09/08/09 FLE Daylight Time}

                
            else:
                
  
                message2print = "{(Step 5/9i) calling" + "; getattr-calling-by-save-of-ForkingPickler4Xi: " + str(getattr) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                                
                                                
                reduce2xi = getattr(obj, "__reduce__", None)
                  
                message2print = "{(Step 5/9j) got" + "; reduce2xi-got-by-save-of-ForkingPickler4Xi: " + str(reduce2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------                 
                   
                if reduce2xi:
                
        
                    message2print = "{(Step 5/9k) calling" + "; reduce2xi-calling-by-save-of-ForkingPickler4Xi: " + str(reduce2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
                                    
                                    
                    # ---------------------------------------------------------------------------------    
                    #
                    # ---------------------------------------------------------------------------------                 
                   
                    rv2xi = reduce2xi()
                    
                    # ---------------------------------------------------------------------------------    
                    #
                    # ---------------------------------------------------------------------------------                 
       
                    message2print = "{(Step 5/9l) got" + "; rv2xi-got-by-save-of-ForkingPickler4Xi: " + str(rv2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                     
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------                 
                       
                else:


     
                    message2print = "{(Step 5/9m) yielded; t-name-yielded: " + str(t.__name__)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    message2print = "{(Step 5/9n) yielded" + "; t-name-yielded-by-save-of-ForkingPickler4Xi: " + str(t.__name__)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                    
                       
                    message2print = "{(Step 5/9o) yielded" + "; object-yielded-by-save-of-ForkingPickler4Xi: " + str(obj)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                                      
                                        
                    # ---------------------------------------------------------------------------------    
                    #
                    # ---------------------------------------------------------------------------------                 
                                          
        # -----------------------------------------------------------------------------------------    
        # Step 6/9: Check for string returned by reduce(), meaning "save as global" OK
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 6/9a) calling" + "; isinstance-calling-by-save-of-ForkingPickler4Xi: " + str(isinstance) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 2. : <built-in function isinstance>
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 

        if isinstance(rv2xi, str):
            message2print = "{(Step 6/9b1) calling" + "; save-global-calling-by-save-of-ForkingPickler4Xi: " + str(self.save_global) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{(Step 6/9b2) passing" + "; object-passing-to-save-global-by-save-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            
            message2print = "{(Step 6/9b3) passing" + "; rv2xi-passing-to-save-global-by-save-of-ForkingPickler4Xi: " + str(rv2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                                
                            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
         
            self.save_global(obj, rv2xi)
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
            message2print = "{(Step 6/9b4) performed" + "; save-global-performed-by-save-of-ForkingPickler4Xi: " + str(self.save_global) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                               
                             
            message2print = "{(Step 6/9c) returning; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            message2print = "{(Step 6/9c) returning" + "; save-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                    
                
            return
        
            
        # -----------------------------------------------------------------------------------------    
        # Step 7/9: Assert that reduce() returned string or tuple OK
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 7/9a1) calling" + "; isinstance-calling-by-save-of-ForkingPickler4Xi: " + str(isinstance) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 2. :  <built-in function isinstance>
        
        message2print = "{(Step 7/9a2) passing" + "; rv2xi-passing-to-isinstance-by-save-of-ForkingPickler4Xi: " + str(rv2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 2. : (<function __newobj__ at 0x024E5BB8>, 
        #         (<class 'process4xi.Process4Xi'>,),
        #         {'_daemonic': False,
        #            '_target': <function f4xi at 0x02AF97C8>,
        #              '_args': (), 
        #           '_tempdir': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-g0ruto', 
        #              '_name': 'Process4Xi-1', '_authkey': 'b">y\\xc6=\\xe9\\xddB\\xd0\'\\xc5p)\\x10F\\xb0\\xb0\\xa1\\xc8\\xe7\\xa3\\xbf\\x95\\x18g\\xfeP\\x17\\xfbn\\xba\\x9a\\x02"',
        #        '_parent_pid': 3716,
        #            '_kwargs': {},
        #          '_identity': (1,),
        #             '_popen': None},
        #          None, None)
        
        message2print = "{(Step 7/9a3) passing" + "; tuple-passing-to-isinstance-by-save-of-ForkingPickler4Xi: " + str(tuple) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 2. : <class 'tuple'>
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 

        if not isinstance(rv2xi, tuple):
        
            message2print = "{(Step 7/9b) yielded" + "; rv2xi-is-tuple-instance-yielded-by-save-of-ForkingPickler4Xi: " + " *** " + str(rv2xi)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                

        
        # -----------------------------------------------------------------------------------------    
        # Step 8/9: Assert that it returned an appropriately sized tuple
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 8/9a1) calling" + "; len-calling-by-save-of-ForkingPickler4Xi: " + str(len) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 2. : <built-in function len>
                        
        message2print = "{(Step 8/9a2) passing" + "; rv2xi-passing-to-len-by-save-of-ForkingPickler4Xi: " + str(rv2xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 2. :
        #1.       (<function __newobj__ at 0x024E5BB8>, 
        #2.       (<class 'process4xi.Process4Xi'>,), 
        #3.       {'_daemonic': False, 
        #            '_target': <function f4xi at 0x02AF97C8>,
        #              '_args': (), 
        #           '_tempdir': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-g0ruto', 
        #              '_name': 'Process4Xi-1', 
        #           '_authkey': 'b">y\\xc6=\\xe9\\xddB\\xd0\'\\xc5p)\\x10F\\xb0\\xb0\\xa1\\xc8\\xe7\\xa3\\xbf\\x95\\x18g\\xfeP\\x17\\xfbn\\xba\\x9a\\x02"', 
        #        '_parent_pid': 3716, 
        #            '_kwargs': {}, 
        #          '_identity': (1,),
        #             '_popen': None
        #         },
        #4.       None,
        #5.       None)
        
        # -----------------------------------------------------------------------------------------    
        # Step 8/9b OK
        # -----------------------------------------------------------------------------------------                 
        
        l = len(rv2xi)
        
            
        message2print = "{(Step 8/9b) got" + "; lenght-got-by-save-of-ForkingPickler4Xi: " + str(l)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #HW: 2. : 5
        
        # -----------------------------------------------------------------------------------------    
        # OK
        # -----------------------------------------------------------------------------------------                 
        
        if not (2 <= l <= 5):
            

            
            message2print = "{yielded" + "; lenght-yielded-by-save-of-ForkingPickler4Xi: " + " *** " + str(l)  + " ; should be either greater or equal to 2 or less or equal to 5 *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                

        
        # -----------------------------------------------------------------------------------------    
        # Step 9/9: Save the reduce() output and finally memoize the object
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 9/9a1) calling" + "; save-reduce-calling-by-save-of-ForkingPickler4Xi: " + str(self.save_reduce) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: 2. : <bound method ForkingPickler4Xi.save_reduce of <forkingpickler4xi.ForkingPickler4Xi object at 0x02BE9170>>
        
        
        message2print = "{(Step 9/9a2) passing" + "; object-passing-to-save-reduce-by-save-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 2. : <Process4Xi(Process4Xi-1, initial)> 
        

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        
        self.save_reduce(obj=obj, *rv2xi)        
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{(Step 9/9b) performed" + "; save-reduce-of-forkingpickler4xi-performed-by-save-of-ForkingPickler4Xi: " + str(self.save_reduce) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        #HW: <bound method ForkingPickler4Xi.save_reduce of <forkingpickler4xi.ForkingPickler4Xi object at 0x02BC6FD0>>
        
        message2print = "{(Step 9/9c) returning" + "; save-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                  
        # #17 obj = i
        # { ForkingPickler4Xi : save ; Step 9/9b: returning time: 11:22:39 07/03/09 FLE Daylight Time}
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "persistent-id-of-ForkingPickler4Xi" usage = "(self, obj)">
    #               this method exists so a subclass can override it
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def persistent_id(self, obj):
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
                         
        class2print = "ForkingPickler4Xi"
        method2print = "persistent-id"          
        
        
        message2print = "{started" + "; persistent-id-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        message2print = "{passed" + "; object-passed-to-persistent-id-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        #HW:
        # {     'authkey': "b'Bg\\xb5\\xf8\\xfc\\xe5\\x9d=\\x00\\xfe|\\xabK_\\xfa\\x9a\\xf7\\xc5\\xa3I\\xc7\\xd9\\xe2c\\x17t\\x91\\x92\\x143u\\xc3'", 
        #      'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'], 
        #          'name': 'Process4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 
        #      'sys_argv': ['hello_world4xi.py'], 'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 
        # 'log_to_stderr': True} ...and then above in pieces

        # { ForkingPickler4Xi : persistent_id ; obj: (<class 'syncmanager4xi.SyncManager4Xi'>, '_run_server'); time: 15:04:06 06/17/09 FLE Daylight Time}
        # { ForkingPickler4Xi : persistent_id ; obj: {'AsyncResult': (None, None, None, <function AutoProxy4Xi at 0x000000000381AF48>),
        #                                             'Condition4Xi': (<function Condition at 0x0000000003221248>, ('acquire', 'release', 'wait', 'notify', 'notify_all', 'sense', 'reason', 'ace'), None, <class 'conditionproxy4xi.ConditionProxy4Xi'>),
        #                                             'Event4Xi': (<function Event at 0x0000000003221EC8>, ('is_set', 'set', 'clear', 'wait', 'sense', 'reason', 'ace'), None, <class 'eventproxy4xi.EventProxy4Xi'>),
        #                                             'Iterator': (None, ('__next__', 'send', 'throw', 'close', 'sense', 'reason', 'ace'), None, <class 'iteratorproxy4xi.IteratorProxy4Xi'>),
        #                                             'BoundedSemaphore4Xi': (<function BoundedSemaphore at 0x0000000003221C48>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>),
        #                                             'Array4Xi': (<function Array4Xi at 0x0000000003812DC8>, ('__len__', '__getitem__', '__setitem__', '__sense__', '__reason__', '__ace__'), None, <class 'makeproxytype4xi.ArrayProxy4Xi'>), 
        #                                             'list': (<class 'list'>, ('__add__', '__contains__', '__delitem__', '__delslice__', '__getitem__', '__len__', '__mul__', '__reversed__', '__rmul__', '__setitem__', 'append', 'count', 'extend', 'index', 'insert', 'pop', 'remove', 'reverse', 'sort', '__imul__', 'sense', 'reason', 'ace'), None, <class 'listproxy4xi.ListProxy4Xi'>),
        #                                             'Queue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x000000000381AF48>),
        #                                             'Pool4Xi': (<class 'pool4xi.Pool4Xi'>, ('apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join', 'map', 'map_async', 'terminate', 'sense', 'reason', 'ace'), {'apply_async': 'AsyncResult', 'imap_unordered': 'Iterator', 'imap': 'Iterator', 'map_async': 'AsyncResult'}, <class 'makeproxytype4xi.PoolProxy4Xi'>),
        #                                             'Lock4Xi': (<built-in function allocate_lock>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>),
        #                                             'Semaphore4Xi': (<function Semaphore at 0x00000000032216C8>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>),
        #                                             'RLock4Xi': (<function RLock at 0x00000000032211C8>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>),
        #                                             'dict': (<class 'dict'>, ('__contains__', '__delitem__', '__getitem__', '__len__', '__setitem__', 'clear', 'copy', 'get', 'has_key', 'items', 'keys', 'pop', 'popitem', 'setdefault', 'update', 'valuessense', 'reason', 'ace'), None, <class 'makeproxytype4xi.DictProxy4Xi'>),
        #                                             'JoinableQueue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x000000000381AF48>),
        #                                             'Value4Xi': (<class 'value4xi.Value4Xi'>, ('get', 'set', 'sense', 'reason', 'ace'), None, <class 'valueproxy4xi.ValueProxy4Xi'>),
        #                                             'Namespace4Xi': (<class 'namespace4xi.Namespace4Xi'>, ('__getattribute__', '__setattr__', '__delattr__', 'sense', 'reason', 'ace'), None, <class 'namespaceproxy4xi.NamespaceProxy4Xi'>)}
        #                                             ; time: 15:04:06 06/17/09 FLE Daylight Time}
        # { ForkingPickler4Xi : persistent_id ; obj: (<function Condition at 0x0000000003221248>, ('acquire', 'release', 'wait', 'notify', 'notify_all', 'sense', 'reason', 'ace'), None, <class 'conditionproxy4xi.ConditionProxy4Xi'>); time: 15:04:06 06/17/09 FLE Daylight Time}
        # { ForkingPickler4Xi : persistent_id ; obj: <class 'makeproxytype4xi.ArrayProxy4Xi'>; time: 13:46:30 06/17/09 FLE Daylight Time}
        # { ForkingPickler4Xi : persistent_id ; obj: <Process4Xi(SyncManager4Xi-2, initial)>; time: 12:09:47 07/02/09 FLE Daylight Time}   
        # { ForkingPickler4Xi : persistent_id ; obj: {'_daemonic': False, '_target': <function modify3xi at 0x0000000003878AC8>, '_args': (Value4Xi('i', 8), Value4Xi(<class 'ctypes.c_double'>, 0.33333333333333331), <SynchronizedString4Xi wrapper for <rawarray4xi.c_char_Array_10 object at 0x000000000389E548>>, <SynchronizedArray4Xi wrapper for <rawarray4xi.ThreeDimension3Xi_Array_3 object at 0x000000000389E948>>), '_tempdir': 'tempdir4process', '_name': 'Process4Xi-5', '_authkey': "b'\\xf4n\\xebg\\n\\x1a\\x14\\xe2(\\xe0\\xa2\\x10\\xf4\\xda\\x8d\\xfcwtC\\xb2\\xe8\\x8b\\x01^\\xa8\\x9al_\\xcc\\xdcM\\xbf'", '_parent_pid': 280, '_kwargs': {}, '_identity': (5,), '_popen': None}; time: 11:22:39 07/03/09 FLE Daylight Time}


        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{returning" + "; None-returning-by-id-of-ForkingPickler4Xi: " + str(None) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        # None
        
        return None  
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-pers-of-ForkingPickler4Xi" usage = "(self, pid)">
    #   save a persistent id reference
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def save_pers(self, pid):
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
            
        class2print = "ForkingPickler4Xi"
        method2print = "save-pers"           
        
        
        message2print = "{started" + "; save-pers-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        message2print = "{passed" + "; pid-passed-to-save-pers-of-ForkingPickler4Xi: " + str(pid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{got" + "; binary-protocol-logical-value-got-by-save-pers-of-ForkingPickler4Xi: " + str(self.bin)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
                            
        if self.bin:                                                 #binary-protocol-logical-value
     
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------
              
            message2print = "{calling" + "; save-calling-by-save-pers-of-ForkingPickler4Xi: " + str(self.save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; pid-passing-to-save-by-save-pers-of-ForkingPickler4Xi: " + str(pid) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            message2print = "{passing" + "; save-persistent-id-passing-to-save-by-save-pers-of-ForkingPickler4Xi: " + str(save_persistent_id) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                   
     
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------
     
            self.save(pid, save_persistent_id=False)
            
            message2print = "{performed" + "; save-performed-by-save-pers-of-ForkingPickler4Xi: " + str(self.save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
            message2print = "{calling" + "; write-calling-by-save-pers-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
         
            message2print = "{passing" + "; BINPERSID-passing-to-write-by-save-pers-of-ForkingPickler4Xi: " + str(BINPERSID) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------
                     
                            
            self.write(BINPERSID)
            
            message2print = "{performed" + "; write-performed-by-save-pers-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
               
            
        else:
     
            # ------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------
             
            message2print = "{calling" + "; write-calling-by-save-pers-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
  
            message2print = "{passing" + "; PERSID-passing-to-write-by-save-pers-of-ForkingPickler4Xi: " + str(PERSID) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                   
            self.write(PERSID + str(pid).encode("ascii") + b'\n')        
            
            message2print = "{performed" + "; write-performed-by-save-pers-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
          
     
        # ----------------------------------------------------------------------------------------- 
        # 
        # -----------------------------------------------------------------------------------------
    
        message2print = "{returning" + "; save-pers-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
                        
                                      
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-reduce-of-ForkingPickler4Xi" usage = "(self, func, args, state=None, listitems=None, dictitems=None, obj=None)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def save_reduce(self, func, args, state=None, listitems=None, dictitems=None, obj=None):
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
    
  
        class2print = "ForkingPickler4Xi"
        method2print = "save-reduce"           
        
        message2print = "{started" + "; save-reduce-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  

        message2print = "{passed" + "; function-passed-to-save-reduce-of-ForkingPickler4Xi: " + str(func) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{passed" + "; args-passed-to-save-reduce-of-ForkingPickler4Xi: " + str(args) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{passed" + "; state-passed-to-save-reduce-of-ForkingPickler4Xi: " + str(state) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{passed" + "; listitems-passed-to-save-reduce-of-ForkingPickler4Xi: " + str(listitems) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{passed" + "; dictitems-passed-to-save-reduce-of-ForkingPickler4Xi: " + str(dictitems) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          

        message2print = "{passed" + "; object-passed-to-save-reduce-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
    
        # Assert that args is a tuple
        if not isinstance(args, tuple):
            message2print = "{yielded" + "; args-yielded-by-save-reduce-of-ForkingPickler4Xi: " + " *** args from save_reduce() should be a tuple *** " + str(args) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                

            

        message2print = "{asserted" + "; args-asserted-by-save-reduce-of-ForkingPickler4Xi: " + "ok, args is a tuple" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                        
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
            
        # Assert that func is callable
        if not hasattr(func, '__call__'):
            message2print = "{yielded" + "; function-yielded-by-save-reduce-of-ForkingPickler4Xi: " + str(func)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            

        
        message2print = "{got" + "; function-got-by-save-reduce-of-ForkingPickler4Xi: " + str(func)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
 
        message2print = "{asserted" + "; function-asserted-by-save-reduce-of-ForkingPickler4Xi: " + "ok, func is callable" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
    
            
        save = self.save
     
        message2print = "{set" + "; save-set-by-save-reduce-of-ForkingPickler4Xi to: " + str(save)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    

        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
            
        write = self.write
 
        message2print = "{set" + "; method-write-set-by-save-reduce-of-ForkingPickler4Xi to: " + str(write)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
 
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
            
        message2print = "{got" + "; proto-got-by-save-reduce-of-ForkingPickler4Xi: " + str(self.proto)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        attr2xi = getattr(func, "__name__", "")
        
        message2print = "{got" + "; attr2xi-got-by-save-reduce-of-ForkingPickler4Xi: " + str(attr2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
                                            
        if self.proto >= 2 and getattr(func, "__name__", "") == "__newobj__":
            # A __reduce__ implementation can direct protocol 2 to
            # use the more efficient NEWOBJ opcode, while still
            # allowing protocol 0 and 1 to work normally.  For this to
            # work, the function returned by __reduce__ should be
            # called __newobj__, and its first argument should be a
            # new-style class.  The implementation for __newobj__
            # should be as follows, although pickle has no way to
            # verify this:
            #
            # def __newobj__(cls, *args):
            #     return cls.__new__(cls, *args)
            #
            # Protocols 0 and 1 will pickle a reference to __newobj__,
            # while protocol 2 (and above) will pickle a reference to
            # cls, the remaining args tuple, and the NEWOBJ code,
            # which calls cls.__new__(cls, *args) at unpickling time
            # (see load_newobj below).  If __reduce__ returns a
            # three-tuple, the state from the third tuple item will be
            # pickled regardless of the protocol, calling __setstate__
            # at unpickling time (see load_build below).
            #
            # Note that no standard __newobj__ implementation exists;
            # you have to provide your own.  This is to enforce
            # compatibility with Python 2.2 (pickles written using
            # protocol 0 or 1 in Python 2.3 should be unpicklable by
            # Python 2.2).
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                
            message2print = "{decided" + "; newobject-decided-by-save-reduce-of-ForkingPickler4Xi: " + str(attr2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
            
            cls = args[0]

            message2print = "{got" + "; class-got-by-save-reduce-of-ForkingPickler4Xi: " + str(cls)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            # <class 'process4xi.Process4Xi'>; time: 11:22:39 07/03/09 FLE Daylight Time}
            # <class 'value4xi.Value4Xi'>; time: 11:22:39 07/03/09 FLE Daylight Time}
            # <class 'value4xi.Value4Xi'>; time: 11:22:39 07/03/09 FLE Daylight Time}
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                  
            if not hasattr(cls, "__new__"):

                
                message2print = "{yielded" + "; class-yielded-by-save-reduce-of-ForkingPickler4Xi: " + str(cls)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                      
                            

     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                 
            if obj is not None and cls is not obj.__class__:
                message2print = "{yielded" + "; args-yielded-by-save-reduce-of-ForkingPickler4Xi: " + " *** args[0] from __newobj__ args has the wrong class *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                message2print = "{obj" + "; object-yielded-by-save-reduce-of-ForkingPickler4Xi: " + str(obj)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
     
                message2print = "{cls" + "; class-yielded-by-save-reduce-of-ForkingPickler4Xi: " + str(cls)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                                
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                 
            args = args[1:]
            
            message2print = "{got" + "; args-got-by-save-reduce-of-ForkingPickler4Xi: " + str(args)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            # { ForkingPickler4Xi : save_reduce ; set args to: (); time: 11:22:39 07/03/09 FLE Daylight Time}
            #12 { ForkingPickler4Xi : save_reduce ; set args to: (); time: 11:22:39 07/03/09 FLE Daylight Time}
            
            message2print = "{calling" + "; save-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{passing" + "; class-passing-to-save-by-save-reduce-of-ForkingPickler4Xi: " + str(cls) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                     
            save(cls)
            
  
            message2print = "{performed" + "; save-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            message2print = "{calling" + "; save-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                          
            message2print = "{passing" + "; args-passing-to-save-by-save-reduce-of-ForkingPickler4Xi: " + str(args) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
     
            save(args)
            
            message2print = "{performed" + "; save-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            message2print = "{calling" + "; write-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{passing" + "; NEWOBJ-passing-to-write-by-save-reduce-of-ForkingPickler4Xi: " + str(NEWOBJ) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                                                
            write(NEWOBJ)
 
            message2print = "{performed" + "; write-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             

                        
        else:
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
             
            message2print = "{calling" + "; save-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{passing" + "; function-passing-to-save-by-save-reduce-of-ForkingPickler4Xi: " + str(func) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                
            save(func)
 
            message2print = "{performed" + "; save-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                
            message2print = "{calling" + "; save-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{passing" + "; args-passing-to-save-by-save-reduce-of-ForkingPickler4Xi: " + str(args) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                               
            save(args)
            
            message2print = "{performed" + "; save-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
      
            message2print = "{calling" + "; write-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{passing" + "; REDUCE-passing-to-write-by-save-reduce-of-ForkingPickler4Xi: " + str(REDUCE) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                                
            write(REDUCE)
   
            message2print = "{performed" + "; write-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
    
        if obj is not None:
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
             
            message2print = "{calling" + "; memoize-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                            
            message2print = "{passing" + "; object-passing-to-memoize-by-save-reduce-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                                                 
            self.memoize(obj)

            
            message2print = "{performed" + "; memoize-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                 
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
     
            
        # More new special cases (that work with older protocols as
        # well): when __reduce__ returns a tuple with 4 or 5 items,
        # the 4th and 5th item should be iterators that provide list
        # items and dict items (as (key, value) tuples), or None.

        if listitems is not None:
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                  
            message2print = "{calling" + "; batch-appends-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(self._batch_appends) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; listitems-passing-to-batch-appends-by-save-reduce-of-ForkingPickler4Xi: " + str(listitems) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                      
            self._batch_appends(listitems)
            
            message2print = "{performed" + "; batch-appends-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(self._batch_appends) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            

        if dictitems is not None: 
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                  
            message2print = "{calling" + "; batch-setitems-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(self._batch_setitems) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; dictitems-passing-to-batch-setitems-by-save-reduce-of-ForkingPickler4Xi: " + str(dictitems) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                        
            self._batch_setitems(dictitems)
            
            message2print = "{performed" + "; batch-setitems-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(self._batch_setitems) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
              


        if state is not None:
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                  
            message2print = "{calling" + "; save-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; state-passing-to-save-by-save-reduce-of-ForkingPickler4Xi: " + str(state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                        
            save(state)
            
            message2print = "{performed" + "; save-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             

            message2print = "{calling" + "; write-calling-by-save-reduce-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; BUILD-passing-to-write-by-save-reduce-of-ForkingPickler4Xi: " + str(BUILD) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
     
            # ----------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------
                        
            write(BUILD)
            
            message2print = "{performed" + "; write-performed-by-save-reduce-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             

     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
                 

        message2print = "{returning" + "; save-reduce-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    

    #</methodize4xi>
            
        
    # ---------------------------------------------------------------------------------------------        
    # Methods below this point are dispatched through the dispatch table 
    # ---------------------------------------------------------------------------------------------

  
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch = {}
    #"
    #/>
            
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-none-of-ForkingPickler4Xi" usage = "(self, obj)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def save_none(self, obj):
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
        
        class2print = "ForkingPickler4Xi"
        method2print = "save-none"           
        
        
        message2print = "{started" + "; save-none-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        message2print = "{passed" + "; object-passed-to-save-none-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        message2print = "{calling" + "; write-calling-by-save-none-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        message2print = "{passing" + "; NONE-passing-to-write-by-save-none-of-ForkingPickler4Xi: " + str(NONE) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
                    
                
                        
        self.write(NONE)
        
   
        message2print = "{performed" + "; write-performed-by-save-none-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                        
        # { ForkingPickler4Xi : save_none ; performed: self.write(NONE); time: 13:46:30 06/17/09 FLE Daylight Time}
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
            
        message2print = "{returning" + "; save-none-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
           
        # { ForkingPickler4Xi : save_none ; returning time: 13:46:30 06/17/09 FLE Daylight Time}        
    #</methodize4xi>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[type(None)] = save_none
    #"
    #/>
    
    class2print = "ForkingPickler4Xi"
    method2print = "share"          
    
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                      
                        
    # ---------------------------------------------------------------------------------------------    
    #<methodize4xi method = "save-bool-of-ForkingPickler4Xi" usage = "(self, obj">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------        
    def save_bool(self, obj):
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
        
        class2print = "ForkingPickler4Xi"
        method2print = "save-bool"          
       
        
        message2print = "{started" + "; save-bool-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # forkingpickler4xi.ForkingPickler4Xi object at 0x02AA4E10
             
        message2print = "{passed" + "; object-passed-to-save-bool-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: True
            
    
        if self.proto >= 2:
     
            # --------------------------------------------------------------------------------------------- 
            #
            # ---------------------------------------------------------------------------------------------
  
            message2print = "{calling" + "; write-of-forkingpickler2xi-calling-by-save-bool-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            # built-in method write of _io.BufferedWriter object at 0x02BB5CB0
            
            message2print = "{passing" + "; object-passing-to-write-of-forkingpickler2xi-by-save-bool-of-ForkingPickler4Xi: " + str(obj and NEWTRUE or NEWFALSE) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
            # b'\x88'
                                            
            self.write(obj and NEWTRUE or NEWFALSE)
            
            message2print = "{performed" + "; write-of-forkingpickler2xi-performed-by-save-bool-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # built-in method write of _io.BufferedWriter object at 0x02BB5CB0
                        
        else:
     
            # --------------------------------------------------------------------------------------------- 
            #
            # ---------------------------------------------------------------------------------------------
          
            message2print = "{calling" + "; self.write-calling-by-save-bool-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; object-passing-to-write-by-save-bool-of-ForkingPickler4Xi: " + str(obj and TRUE or FALSE) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
                  
            self.write(obj and TRUE or FALSE)
            
            message2print = "{performed" + "; write-performed-by-save-bool-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                  
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
        
        message2print = "{returning" + "; save-bool-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        #  14:30:20 02/11/10 FLE Standard Time
        
    #</methodize4xi>
            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[bool] = save_bool
    #"
    #/>
    
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                  
                            
    # ---------------------------------------------------------------------------------------------    
    #<methodize4xi method = "save-long-of-ForkingPickler4Xi" usage = "(self, obj, pack=struct.pack)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------           
    def save_long(self, obj, pack=struct.pack):
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
            
        class2print = "ForkingPickler4Xi"
        method2print = "save-long"        
        
        message2print = "{started" + "; save-long-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        message2print = "{passed" + "; object-passed-to-save-long-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        message2print = "{passed" + "; pack-passed-to-save-long-of-ForkingPickler4Xi: " + str(pack) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
          
        # { ForkingPickler4Xi : save_long ; obj: 8; pack: <built-in function pack>; time: 11:22:39 07/03/09 FLE Daylight Time}
        
        message2print = "{got" + "; binary-protocol-logical-value-got-by-save-long-of-ForkingPickler4Xi: " + str(self.bin)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
     
        # --------------------------------------------------------------------------------------------- 
        #
        # ---------------------------------------------------------------------------------------------
                                
        if self.bin:                                                     #binary-protocol-logical-value
            # If the int is small enough to fit in a signed 4-byte 2's-comp
            # format, we can store it more efficiently than the general
            # case.
            # First one- and two-byte unsigned ints:
            message2print = "{got" + "; object-got-by-save-long-of-ForkingPickler4Xi: " + str(obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            if obj >= 0:
                if obj <= 0xff:
                     
                    # --------------------------------------------------------------------------------------------- 
                    #
                    # ---------------------------------------------------------------------------------------------
    
                    message2print = "{(Step 1/7a) calling" + "; write-calling-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
                    
                    message2print = "{(Step 1/7b) passing" + "; BININT1-passing-to-write-by-save-long-of-ForkingPickler4Xi: " + str(BININT1 + bytes([obj])) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                
                     
                    # --------------------------------------------------------------------------------------------- 
                    #
                    # ---------------------------------------------------------------------------------------------
          
                    self.write(BININT1 + bytes([obj]))
                    

                    message2print = "{(Step 1/7d) performed" + "; write-performed-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    
                    message2print = "{(Step 1/7e) returning" + "; save-long-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)    
                                   
                    # { ForkingPickler4Xi : save_long ; 1/7 returning by first calling self.write(BININT1 + bytes([obj])) time: 11:22:39 07/03/09 FLE Daylight Time}
                    
                    return
                    
                if obj <= 0xffff:
                     
                    # --------------------------------------------------------------------------------------------- 
                    #
                    # ---------------------------------------------------------------------------------------------
                    
                    message2print = "{(Step 2/7a) calling" + "; write-calling-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    message2print = "{(Step 2/7b) passing" + "; BININT2-passing-to-write-by-save-long-of-ForkingPickler4Xi: " + str(BININT2 + bytes([obj&0xff, obj>>8])) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                       
                    self.write(BININT2 + bytes([obj&0xff, obj>>8]))
                    
                    message2print = "{(Step 2/7c) returning" + "; save-long-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print) 
                     
                    # --------------------------------------------------------------------------------------------- 
                    #
                    # ---------------------------------------------------------------------------------------------
                
                    return
                    
            # --------------------------------------------------------------------------------------------- 
            #
            # ---------------------------------------------------------------------------------------------
                        
            # Next check for 4-byte signed ints:
            high_bits = obj >> 31  # note that Python shift sign-extends
            
            message2print = "{set" + "; high-bits-set-by-save-long-of-ForkingPickler4Xi to: " + str(high_bits)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                    
            # --------------------------------------------------------------------------------------------- 
            #
            # ---------------------------------------------------------------------------------------------
                                 
            if high_bits == 0 or high_bits == -1:
                     
                # --------------------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------------------
                 
                # All high bits are copies of bit 2**31, so the value
                # fits in a 4-byte signed int.
                
                message2print = "{calling" + "; write-calling-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                
                message2print = "{passing" + "; BININT-passing-to-write-by-save-long-of-ForkingPickler4Xi: " + str(BININT + pack("<i", obj)) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                

                self.write(BININT + pack("<i", obj))
                
          
                message2print = "{(Step 3/7) returning" + "; save-long-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print) 

                return
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                     
        message2print = "{checking" + "; proto-checking-by-save-long-of-ForkingPickler4Xi: " + str(self.proto) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
                                
        if self.proto >= 2:
                     
            # ----------------------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------------------
            
            encoded = encode_long(obj)
            
            message2print = "{(Step 4/7) set" + "; encoded-set-by-save-long-of-ForkingPickler4Xi to: " + str(encoded)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
           
            n = len(encoded)
           
            message2print = "{(Step 5/7) set" + "; number-set-by-save-long-of-ForkingPickler4Xi to: " + str(n)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                     
            # ----------------------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------------------
                                
            if n < 256:
                     
                # ----------------------------------------------------------------------------------------------------- 
                #
                # -----------------------------------------------------------------------------------------------------
                 
                message2print = "{calling" + "; write-calling-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                                
                message2print = "{passing" + "; LONG1-passing-to-write-by-save-long-of-ForkingPickler4Xi: " + str(LONG1 + bytes([n]) + encoded) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                     
                # ----------------------------------------------------------------------------------------------------- 
                #
                # -----------------------------------------------------------------------------------------------------
    
                self.write(LONG1 + bytes([n]) + encoded)
                

                message2print = "{(Step 6/7) performed" + "; write-performed-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                                
            else:
                     
                # ----------------------------------------------------------------------------------------------------- 
                #
                # -----------------------------------------------------------------------------------------------------
                
                message2print = "{calling" + "; write-calling-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                
                message2print = "{passing" + "; LONG4-passing-to-write-by-save-long-of-ForkingPickler4Xi: " + str(LONG4 + pack("<i", n) + encoded) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                     
                # ----------------------------------------------------------------------------------------------------- 
                #
                # -----------------------------------------------------------------------------------------------------
                                   
                self.write(LONG4 + pack("<i", n) + encoded)
                

                message2print = "{(Step 7/7) performed" + "; write-performed-by-save-long-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                     
            # ----------------------------------------------------------------------------------------------------- 
            #
            # -----------------------------------------------------------------------------------------------------
        
            message2print = "{returning" + "; save-long-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                  
            return
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                
        message2print = "{returning; call: self.write(LONG + repr(obj).encode; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
                    
        self.write(LONG + repr(obj).encode("ascii") + b'\n') 
    #</methodize4xi>
  
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[int] = save_long
    #"
    #/>
    
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                      
                    
             
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-float-of-ForkingPickler4Xi" usage = "(self, obj, pack=struct.pack)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------      
    def save_float(self, obj, pack=struct.pack):
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
        
        class2print = "ForkingPickler4Xi"
        method2print = "save-float"   
        
        message2print = "{started" + "; save-float-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{passed" + "; object-passed-to-save-float-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        
        message2print = "{passed" + "; pack-passed-to-save-float-of-ForkingPickler4Xi: " + str(pack) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
              
        if self.bin:                                                             #binary-protocol-logical-value
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
            
            message2print = "{calling" + "; write-calling-by-save-float-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; BINFLOAT-passing-to-write-by-save-float-of-ForkingPickler4Xi: " + str(BINFLOAT + pack('>d', obj)) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
                    
            self.write(BINFLOAT + pack('>d', obj))
            
            message2print = "{performed" + "; write-performed-by-save-float-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
 
        else:
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
            
            message2print = "{calling" + "; write-calling-by-save-float-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            
            message2print = "{passing" + "; FLOAT-passing-to-write-by-save-float-of-ForkingPickler4Xi: " + str(FLOAT + repr(obj).encode("ascii") + b'\n') + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
           
            self.write(FLOAT + repr(obj).encode("ascii") + b'\n')
            
            
            message2print = "{performed" + "; write-performed-by-save-float-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
   
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
    
        message2print = "{returning" + "; save-float-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                
    #</methodize4xi>
  
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[float] = save_float
    #"
    #/>
           
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                        
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-bytes-of-ForkingPickler4Xi" usage = "(self, obj, pack=struct.pack)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------       
    def save_bytes(self, obj, pack=struct.pack):
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
       
        class2print = "ForkingPickler4Xi"
        method2print = "save-bytes"   
        
        message2print = "{started" + "; save-bytes-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
             
        message2print = "{passed" + "; object-passed-to-save-bytes-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
             
        message2print = "{passed" + "; pack-passed-to-save-bytes-of-ForkingPickler4Xi: " + str(pack) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        message2print = "{checking" + "; proto-checking-by-save-bytes-of-ForkingPickler4Xi: " + str(self.proto) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                              
        if self.proto < 3:
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
             
            message2print = "{calling" + "; save-reduce-calling-by-save-bytes-of-ForkingPickler4Xi: " + str(self.save_reduce) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            message2print = "{passing" + "; bytes-passing-to-save-reduce-by-save-bytes-of-ForkingPickler4Xi: " + str(bytes) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{passing" + "; object-passing-to-save-reduce-by-save-bytes-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                  
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
            
            self.save_reduce(bytes, (list(obj),), obj=obj)
            
            message2print = "{performed" + "; save-reduce-performed-by-save-bytes-of-ForkingPickler4Xi: " + str(self.save_reduce) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            message2print = "{returning" + "; save-bytes-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                   
            return
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                 
        n = len(obj)
        message2print = "{set" + "; number-set-by-save-bytes-of-ForkingPickler4Xi to: " + str(n)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                     
        if n < 256:
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
             
            message2print = "{calling" + "; write-calling-by-save-bytes-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                 
            message2print = "{passing" + "; SHORT-BINBYTES-passing-to-write-by-save-bytes-of-ForkingPickler4Xi: " + str(SHORT_BINBYTES + bytes([n]) + bytes(obj)) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
                       
            self.write(SHORT_BINBYTES + bytes([n]) + bytes(obj))
            
            message2print = "{performed" + "; write-performed-by-save-bytes-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
        else:
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
             
            message2print = "{calling" + "; write-calling-by-save-bytes-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                 
            message2print = "{passing" + "; BINBYTES-passing-to-write-by-save-bytes-of-ForkingPickler4Xi: " + str(BINBYTES + pack("<i", n) + bytes(obj)) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
              
            self.write(BINBYTES + pack("<i", n) + bytes(obj))
            
            message2print = "{performed" + "; write-performed-by-save-bytes-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
           
        message2print = "{calling" + "; memoize-calling-by-save-bytes-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                
        message2print = "{passing" + "; object-passing-to-memoize-by-save-bytes-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                                              
        self.memoize(obj) 
        
        message2print = "{performed" + "; memoize-performed-by-save-bytes-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                           
        message2print = "{returning" + "; save-bytes-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)         
 
    #</methodize4xi>
 
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[bytes] = save_bytes
    #"
    #/>
    
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                        
                          
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-str-of-ForkingPickler4Xi" usage = "(self, obj, pack=struct.pack)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def save_str(self, obj, pack=struct.pack):
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
       
        class2print = "ForkingPickler4Xi"
        method2print = "save-str"       
        
        message2print = "{started" + "; save-str-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # forkingpickler4xi.ForkingPickler4Xi object at 0x02B02E70
        
        message2print = "{passed" + "; object-passed-to-save-str-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # ['hello_world4xi.py']
        # authkey; b'=\x8b\xfa\xc3iK\xaa\xcagix\xbc5\x94\xb8\x1c@l\xedF\x00\xe1\x08\xc8\xfb\xf0\x91\x0cg\xc2' 
        # sys_path; e:\Sw4Xi\python32\Lib\multiprocessing4xi ...all in sys path to log_to_stderr
        
        
        message2print = "{passed" + "; pack-passed-to-save-str-of-ForkingPickler4Xi: " + str(pack) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        
        message2print = "{checking" + "; binary-protocol-logical-value-checking-by-save-str-of-ForkingPickler4Xi: " + str(self.bin) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                             
        if self.bin:                                                             #binary-protocol-logical-value
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
             
            encoded = obj.encode('utf-8')
            
            message2print = "{set" + "; encoded-set-by-save-str-of-ForkingPickler4Xi to: " + str(encoded)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # b'authkey' .....    
                            
            n = len(encoded)
            
            message2print = "{set" + "; number-set-by-save-str-of-ForkingPickler4Xi to: " + str(n)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 7 .....
            
            message2print = "{calling" + "; write-calling-by-save-str-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            # built-in method write of _io.BufferedWriter object at 0x02D25CB0
               
            message2print = "{passing" + "; BINUNICODE-passing-to-write-by-save-str-of-ForkingPickler4Xi: " + str(BINUNICODE + pack("<i", n) + encoded) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # b'X\x07\x00\x00\x00authkey' ....    
                                              
            self.write(BINUNICODE + pack("<i", n) + encoded)
            
            
        else:
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
             
            obj = obj.replace("\\", "\\u005c")
            obj = obj.replace("\n", "\\u000a")
            
            message2print = "{set" + "; object-set-by-save-str-of-ForkingPickler4Xi to: " + str(obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
              
            
            message2print = "{calling" + "; write-calling-by-save-str-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{passing" + "; UNICODE-passing-to-write-by-save-str-of-ForkingPickler4Xi: " + str(UNICODE + bytes(obj.encode('raw-unicode-escape')) + b'\n') + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                 
            self.write(UNICODE + bytes(obj.encode('raw-unicode-escape')) + b'\n')
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
           
        message2print = "{calling" + "; memoize-calling-by-save-str-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                
        message2print = "{passing" + "; object-passing-to-memoize-by-save-str-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # authkey ....        
                                                       
        self.memoize(obj) 
           
        message2print = "{performed" + "; memoize-performed-by-save-str-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # bound method ForkingPickler4Xi.memoize of forkingpickler4xi.ForkingPickler4Xi object at 0x02C6BE70
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                           
        message2print = "{returning" + "; save-str-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 14:41:51 02/11/10 FLE Standard Time               
                
                        
    #</methodize4xi>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[str] = save_str
    #"
    #/>
    
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                          
                    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-tuple-of-ForkingPickler4Xi" usage = "(self, obj">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------     
    def save_tuple(self, obj):
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 1/23)
        # -----------------------------------------------------------------------------------------------------
         
        class2print = "ForkingPickler4Xi"
        method2print = "save-tuple"       
        
        message2print = "{(Step 1/23a) started" + "; save-tuple-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: 1. 2. 3. = line 6256 and the third one goes to bananas *** to be solved *** 
               
        message2print = "{(Step 1/23b) passed" + "; object-passed-to-save-tuple-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        #HW: x. 1. and 2. : ()
        #HW: x. 3. : 1,)
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 2/23)
        # -----------------------------------------------------------------------------------------------------
         
        write = self.write
        
        message2print = "{(Step 2/23) set" + "; method-write-set-by-save-tuple-of-ForkingPickler4Xi to: " + str(write)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: x. 1. and 2.  : <built-in method write of _io.BufferedWriter object at 0x02C98DB0>        
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 3/23)
        # -----------------------------------------------------------------------------------------------------
     
        proto = self.proto
        
        message2print = "{(Step 3/23) set" + "; proto-set-by-save-tuple-of-ForkingPickler4Xi to: " + str(proto)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: x. 1. and 2.  : 3
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 4/23)
        # -----------------------------------------------------------------------------------------------------
       
        n = len(obj)
        
        message2print = "{(Step 4/23) set" + "; number-set-by-save-tuple-of-ForkingPickler4Xi to: " + str(n)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: x. 1. and 2.  : 0
        #HW: x. 3.         : 1  
                   
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
               
        if n == 0:
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
             
            if proto:
                     
                # --------------------------------------------------------------------------------------------- 
                # (Step 5/23)
                # ---------------------------------------------------------------------------------------------
                 
                message2print = "{(Step 5/23a) calling" + "; write-calling-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                #HW: x. 1. and 2.  : <built-in method write of _io.BufferedWriter object at 0x02C98DB0>
                
                message2print = "{(Step 5/23b) passing" + "; EMPTY-TUPLE-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(EMPTY_TUPLE) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                #HW: x. 1. and 2.  : b')'
                             
                write(EMPTY_TUPLE)
                
            else:
                     
                # --------------------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------------------
                 
                message2print = "{(Step 5/23c) calling" + "; write-calling-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                
                message2print = "{(Step 5/23d) passing" + "; MARK-TUPLE-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(MARK + TUPLE) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                              
                
                write(MARK + TUPLE)
            
            message2print = "{(Step 5/23e) returning" + "; save-tuple-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #HW: x. 1. and 2. : 14:28:08 03/16/10 FLE Standard Time                      
                     
        
            return
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 6/23)
        # -----------------------------------------------------------------------------------------------------
    
        save = self.save
        
        message2print = "{6/23) set" + "; save-set-by-save-tuple-of-ForkingPickler4Xi to: " + str(save)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: x. 3. : <bound method ForkingPickler4Xi.save of <forkingpickler4xi.ForkingPickler4Xi object at 0x02BCDAF0>>
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 7/23)
        # -----------------------------------------------------------------------------------------------------
                
        memo = self.memo
        
        message2print = "{(Step 7/23a) set" + "; memo-set-by-save-tuple-of-ForkingPickler4Xi to: " + str(memo)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: x. 3. : {39471776: (5, '_tempdir'),
        #             45317552: (1, {'_daemonic': False,
        #                              '_target': <function f4xi at 0x02CA0738>,
        #                                '_args': (),
        #                             '_tempdir': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-qxfjio',
        #                                '_name': 'Process4Xi-1',
        #                             '_authkey': "b'\\x1c\\xbc\\xf7\\xf8\\x9b91\\xde\\xdd\\xe6\\xadPqw8\\x0c+\\xab\\xaa(\\xaa\\x0f`\\x80\\x02\\xbe#$\\xa3\\xad\\x16!'",
        #                          '_parent_pid': 3560,
        #                              '_kwargs': {},
        #                            '_identity': (1,),
        #                               '_popen': None
        #                           }
        #                         ),
        #             39518112: (7, '_name'),
        #             39483232: (11, '_parent_pid'),
        #             39517344: (4, '_args'),
        #             46848352: (8, 'Process4Xi-1'),
        #             39428320: (14, '_identity'),
        #             39517760: (12, '_kwargs'),
        #             45324592: (13, {}),
        #             46833648: (0, <Process4Xi(Process4Xi-1, initial)>),
        #             39517536: (3, '_target'),
        #             46830112: (6, 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-qxfjio'),
        #             44885568: (10, "b'\\x1c\\xbc\\xf7\\xf8\\x9b91\\xde\\xdd\\xe6\\xadPqw8\\x0c+\\xab\\xaa(\\xaa\\x0f`\\x80\\x02\\xbe#$\\xa3\\xad\\x16!'"),
        #             39471712: (2, '_daemonic'),
        #             39461536: (9, '_authkey')
        #            }
        #

        
        message2print = "{(Step 7/23b) checking" + "; number-checking-by-save-tuple-of-ForkingPickler4Xi: " + str(n) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        #HW: x. 3. : 1
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
        #HW: x. 3. : n = 1 and proto = 3
                          
        if n <= 3 and proto >= 2:
        
            # ------------------------------------------------------------------------------------------------- 
            # (Step 8/23)
            # -------------------------------------------------------------------------------------------------        
            
            for element in obj:
            
                message2print = "{(Step 8/23a) passing" + "; element-passing-to-save-by-save-tuple-of-ForkingPickler4Xi: " + str(element) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)          
                #HW: 1
                        
                save(element)
                
                message2print = "{(Step 8/23b) performed" + "; save-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                #HW: <bound method ForkingPickler4Xi.save of <forkingpickler4xi.ForkingPickler4Xi object at 0x02D49250>>
                
                
            message2print = "{(Step 8/23c) did" + "; object-did-by-save-tuple-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
            #HW: (1,)
             
        
            # ------------------------------------------------------------------------------------------------- 
            # (Step 9/23)
            # -------------------------------------------------------------------------------------------------        
                                
            # Subtle.  Same as in the big comment below.
            if id(obj) in memo:
        
                # --------------------------------------------------------------------------------------------- 
                #
                # ---------------------------------------------------------------------------------------------        
                 
                message2print = "{(Step 9/23a) passing" + "; id-of-object-passing-to-get-of-forkingpickler2xi-by-save-tuple-of-ForkingPickler4Xi: " + str(id(obj)) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          

                get = self.get(memo[id(obj)][0])
                
                message2print = "{(Step 9/23b) got" + "; get-got-by-save-tuple-of-ForkingPickler4Xi: " + str(get)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
                message2print = "{(Step 9/23c) passing" + "; POP-n-get-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(POP * n + get) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
        
        
                write(POP * n + get)
                
                message2print = "{(Step 9/23d) performed" + "; write-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                        
        
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------        
                      
            else:
        
                # --------------------------------------------------------------------------------------------- 
                # (Step 10/23)
                # ---------------------------------------------------------------------------------------------        
                 
                message2print = "{(Step 10/23) passing" + "; tuplesize2code-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(_tuplesize2code[n]) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
                #HW: b'\x85'
                        

                write(_tuplesize2code[n])

        
                # --------------------------------------------------------------------------------------------- 
                # (Step 11/23)
                # ---------------------------------------------------------------------------------------------        
                     
                message2print = "{(Step 11/23a) performed" + "; write-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                #HW: <built-in method write of _io.BufferedWriter object at 0x02CCEE30>
                
                 
                message2print = "{(Step 11/23b) passing" + "; object-passing-to-memoize-of-forkingpickler2xi-by-save-tuple-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
                #HW: (1,)
                        
                                     
                self.memoize(obj)

                     
                message2print = "{(Step 11/23c) performed" + "; memoize-of-forkingpickler2xi-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                              
                # --------------------------------------------------------------------------------------------- 
                # (Step 12/23)
                # ---------------------------------------------------------------------------------------------        
                     
                message2print = "{(Step 12/23) performed" + "; memoize-of-forkingpickler2xi-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                #HW: <bound method ForkingPickler4Xi.memoize of <forkingpickler4xi.ForkingPickler4Xi object at 0x02D49250>>
                
                             
            # ------------------------------------------------------------------------------------------------- 
            # (Step 13/23)
            # -------------------------------------------------------------------------------------------------        
            message2print = "{(Step 13/23) returning" + "; save-tuple-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)      
            #HW: 16:17:33 03/16/10 FLE Standard Time
                              
            return
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 14/23)
        # -----------------------------------------------------------------------------------------------------
        # proto 0 or proto 1 and tuple isn't empty, or proto > 1 and tuple
        # has more than 3 elements.
        
        message2print = "{(Step 14/23a) calling" + "; write-calling-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                        
        message2print = "{(Step 14/23b) passing" + "; MARK-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(MARK) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 15/23)
        # -----------------------------------------------------------------------------------------------------
                                            
        write(MARK)
        
        message2print = "{(Step 15/23) performed" + "; write-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                        
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 16/23)
        # -----------------------------------------------------------------------------------------------------
            
        for element in obj:
        
            message2print = "{(Step 16/23a) passing" + "; element-passing-to-save-by-save-tuple-of-ForkingPickler4Xi: " + str(element) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            
            save(element)
            
            message2print = "{(Step 16/23b) performed" + "; save-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(save) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                   
        
        message2print = "{(Step 16/23c) did" + "; object-did-by-save-tuple-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                         
        
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
    
        if id(obj) in memo:
            # Subtle.  d was not in memo when we entered save_tuple(), so
            # the process of saving the tuple's elements must have saved
            # the tuple itself:  the tuple is recursive.  The proper action
            # now is to throw away everything we put on the stack, and
            # simply GET the tuple (it's already constructed).  This check
            # could have been done in the "for element" loop instead, but
            # recursive tuples are a rare thing.
        
            # ------------------------------------------------------------------------------------------------- 
            # (Step 17/23)
            # -------------------------------------------------------------------------------------------------
            message2print = "{(Step 17/23a) passing" + "; memo-id-object-passing-to-get-of-forkingpickler4xi-by-save-tuple-of-ForkingPickler4Xi: " + str(memo[id(obj)][0]) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
                    
                        
            
            get = self.get(memo[id(obj)][0])
            
            message2print = "{(Step 17/23b) got" + "; get-got-by-save-tuple-of-ForkingPickler4Xi: " + str(get)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                    
        
            # ------------------------------------------------------------------------------------------------- 
            # (Step 18/23)
            # -------------------------------------------------------------------------------------------------
                
            if proto:
                message2print = "{(Step 18/23a) passing" + "; POP-MARK-get-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(POP_MARK + get) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
                        
                        
            
                write(POP_MARK + get)
                
                message2print = "{(Step 18/23b) performed" + "; write-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
            
                        
                
            else:   # proto 0 -- POP_MARK not available
            
                message2print = "{(Step 18/23c) passing" + "; POP-MARK-get-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(POP * (n+1) + get) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                            
                write(POP * (n+1) + get)
                
                
                message2print = "{(Step 18/23d) performed" + "; write-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
              
            # ------------------------------------------------------------------------------------------------- 
            # (Step 19/23)
            # -------------------------------------------------------------------------------------------------
            message2print = "{(Step 19/23) returning" + "; save-tuple-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)      
                        
            return
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 20/23)
        # -----------------------------------------------------------------------------------------------------
    
        # No recursion.
        message2print = "{(Step 20/23a) calling" + "; write-calling-by-save-tuple-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                        
        message2print = "{(Step 20/23b) passing" + "; TUPLE-passing-to-write-by-save-tuple-of-ForkingPickler4Xi: " + str(TUPLE) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 21/23)
        # -----------------------------------------------------------------------------------------------------
                  
        self.write(TUPLE)
        
        message2print = "{(Step 21/23a) performed" + "; write-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(self.write) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
           
        message2print = "{(Step 21/23b) calling" + "; memoize-calling-by-save-tuple-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        message2print = "{(Step 21/23c) passing" + "; object-passing-to-memoize-by-save-tuple-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                
                      
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 22/23)
        # -----------------------------------------------------------------------------------------------------
            
        self.memoize(obj)
        
        message2print = "{(Step 22/23) performed" + "; memoize-performed-by-save-tuple-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                     
        # ----------------------------------------------------------------------------------------------------- 
        # (Step 23/23)
        # -----------------------------------------------------------------------------------------------------
               
        message2print = "{(Step 23/23) returning" + "; save-tuple-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
        return               
                                    
    #</methodize4xi>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[tuple] = save_tuple
    #"
    #/>
    
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                    
                    
                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-list-of-ForkingPickler4Xi" usage = "(self, obj)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def save_list(self, obj):
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
        
        class2print = "ForkingPickler4Xi"
        method2print = "save-list"       
        
        message2print = "{started" + "; save-list-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # forkingpickler4xi.ForkingPickler4Xi object at 0x02C6BE70
               
        message2print = "{passed" + "; object-passed-to-save-list-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages']
        # ['hello_world4xi.py']
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
        
        write = self.write
        
        message2print = "{set" + "; method-write-set-by-save-list-of-ForkingPickler4Xi to: " + str(write)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # built-in method write of _io.BufferedWriter object at 0x02D07CB0
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
             
        if self.bin:                                                             #binary-protocol-logical-value
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
            
            message2print = "{calling" + "; write-calling-by-save-list-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{passing" + "; EMPTY-LIST-passing-to-write-by-save-list-of-ForkingPickler4Xi: " + str(EMPTY_LIST) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # EMPTY-LIST-passing-to-write-by-save-list-of-ForkingPickler4Xi    
                                        
            write(EMPTY_LIST)
            
            message2print = "{performed" + "; write-performed-by-save-list-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # built-in method write of _io.BufferedWriter object at 0x02D07CB0               
            
        else:
                     
            # ------------------------------------------------------------------------------------------------- 
            #
            # -------------------------------------------------------------------------------------------------
                
            # proto 0 -- can't use EMPTY_LIST
            message2print = "{calling" + "; write-calling-by-save-list-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
            message2print = "{passing" + "; MARK-LIST-passing-to-write-by-save-list-of-ForkingPickler4Xi: " + str(MARK + LIST) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                        
            write(MARK + LIST)
            
            message2print = "{performed" + "; write-performed-by-save-list-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
            
        message2print = "{calling" + "; memoize-calling-by-save-list-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        message2print = "{passing" + "; object-passing-to-memoize-by-save-list-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages']</rdfs:comment>
        # ['hello_world4xi.py']
                
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                             
        self.memoize(obj)
        
        message2print = "{performed" + "; memoize-performed-by-save-list-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                        
        message2print = "{calling" + "; batch-appends-calling-by-save-list-of-ForkingPickler4Xi: " + str(self._batch_appends) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        message2print = "{passing" + "; object-passing-to-batch-appends-by-save-list-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages']</rdfs:comment>
        # ['hello_world4xi.py']                
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
                   
        self._batch_appends(obj)  
        
        message2print = "{performed" + "; batch-appends-performed-by-save-list-of-ForkingPickler4Xi: " + str(self._batch_appends) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # bound method ForkingPickler4Xi._batch_appends of forkingpickler4xi.ForkingPickler4Xi object at 0x02C4DE70 
                     
        # ----------------------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------------------
               
        message2print = "{returning" + "; save-list-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #  14:54:37 02/11/10 FLE Standard Time                
                
                                     
    #</methodize4xi>    

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[list] = save_list

    _BATCHSIZE = 1000   
    #"
    #/>
         
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                          
                    
    message2print = "{sharing" + "; BATCHSIZE-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(_BATCHSIZE)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                      
                        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "batch-appends-of-ForkingPickler4Xi" usage = "(self, items)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _batch_appends(self, items):
        # Helper to batch up APPENDS sequences
                     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
               
        class2print = "ForkingPickler4Xi"
        method2print = "batch-appends"       
        
        message2print = "{started" + "; batch-appends-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # forkingpickler4xi.ForkingPickler4Xi object at 0x02C4DE70
        
        message2print = "{passed" + "; items-passed-to-batch-appends-of-ForkingPickler4Xi: " + str(items) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages']
        # ['hello_world4xi.py']
        # dict_items([('authkey', 'b"g~\\xd2\\xfc\\xcc\\xe6v_\\xc2\\x1d\\xa3\\xb6\\x08O\\xda\\xdaj\\x83T\\xe4\\xea\\x8fr]CG}w\\xa5\\xb6\\xc6\'"'),
        #             ('sys_path', ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages']),
        #             ('name', 'Process4Xi-1'),
        #             ('orig_dir', 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi'),
        #             ('sys_argv', ['hello_world4xi.py']),
        #             ('main_path', 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py'),
        #             ('log_to_stderr', True)])
            
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
           
        save = self.save
        write = self.write

        if not self.bin:                                             #binary-protocol-logical-value
            for x in items:
                save(x)
                write(APPEND)
            return

        items = iter(items)
        r = range(self._BATCHSIZE)
        while items is not None:
            tmp = []
            for i in r:
                try:
                    x = next(items)
                    tmp.append(x)
                except StopIteration:
                    items = None
                    break
            n = len(tmp)
            if n > 1:
                write(MARK)
                for x in tmp:
                    save(x)
                write(APPENDS)
            elif n:
                save(tmp[0])
                write(APPEND)
            # else tmp is empty, and we're done    
                     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
           
        message2print = "{returning" + "; batch-appends-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 14:54:37 02/11/10 FLE Standard Time                        
                        
                                              
    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-dict-of-ForkingPickler4Xi" usage = "(self, obj)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def save_dict(self, obj):
                     
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
       
        class2print = "ForkingPickler4Xi"
        method2print = "save-dict"       
        
        message2print = "{started" + "; save-dict-of-ForkingPickler4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
        #HW2: forkingpickler4xi.ForkingPickler4Xi object at 0x02C4DE70
             
        message2print = "{passed" + "; object-passed-to-save-dict-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                      
        #HW1: { 'authkey': 'b"g~\\xd2\\xfc\\xcc\\xe6v_\\xc2\\x1d\\xa3\\xb6\\x08O\\xda\\xdaj\\x83T\\xe4\\xea\\x8fr]CG}w\\xa5\\xb6\\xc6\                        '"', 
        #      'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'],
        #          'name': 'Process4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi',
        #      'sys_argv': ['hello_world4xi.py'], 'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py',
        # 'log_to_stderr': True}
         
        #PX:   {  'authkey': b'\xd9\x1eHM\xe6v\xb0\x8a\x0eV\x8b\xee\x12ca\xc9\xb2n\xf4\x01H]\xab,j\xdb\x83\xc4C\xe0t7', 
        #        'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'], 
        #            'name': 'SyncManager4Xi-1',
        #        'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 
        #        'sys_argv': ['test_proxy_mp_xi4.py'], 
        #       'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\test_proxy_mp_xi4.py', 
        #   'log_to_stderr': True}
                             
        # ----------------------------------------------------------------------------------------- 
        # OK
        # -----------------------------------------------------------------------------------------
        
        write = self.write
        
        message2print = "{set" + "; method-write-set-by-save-dict-of-ForkingPickler4Xi to: " + str(write)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW, #PX: <built-in method write of _io.BufferedWriter object at 0x02B7A5B0>
               
        message2print = "{checking" + "; bin-checking-by-save-dict-of-ForkingPickler4Xi: " + str(self.bin) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        #HW, PX: True
                     
        # ----------------------------------------------------------------------------------------- 
        # OK
        # -----------------------------------------------------------------------------------------
                                    
        if self.bin:
            message2print = "{calling" + "; write-calling-by-save-dict-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #HW, #PX: <built-in method write of _io.BufferedWriter object at 0x02B7A5B0>
            
            message2print = "{passing" + "; EMPTY-DICT-passing-to-write-by-save-dict-of-ForkingPickler4Xi: " + str(EMPTY_DICT) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #HW, #PX:  b'}    
                
            ack4emptydictwrite = False
            nack4emptydictwrite = True
                    
            try:
                write(EMPTY_DICT)
            
                nack4emptydictwrite = False
                ack4emptydictwrite = True
                
                message2print = "{acknowledged" + "; ack4emptydictwrite-acknowledged-by-save-dict-of-ForkingPickler4Xi: " + str(ack4emptydictwrite) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)

                                
            except:
                message2print = "{yielded" + "; nack4emptydictwrite-yielded-by-save-dict-of-ForkingPickler4Xi: " + str(nack4emptydictwrite)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                            
                            
        else:   # proto 0 -- can't use EMPTY_DICT
            message2print = "{calling" + "; write-calling-by-save-dict-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            
            message2print = "{passing" + "; MARK-DICT-passing-to-write-by-save-dict-of-ForkingPickler4Xi: " + str(MARK + DICT) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                       
            write(MARK + DICT)
            
            message2print = "{performed" + "; write-performed-by-save-dict-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                     
        # ----------------------------------------------------------------------------------------- 
        # OK
        # -----------------------------------------------------------------------------------------
                
        ack4memoize = False
        nack4memoize = True
        
        message2print = "{calling" + "; memoize-calling-by-save-dict-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        #HW: <bound method ForkingPickler4Xi.memoize of <forkingpickler4xi.ForkingPickler4Xi object at 0x02BAD570>>
            
                            
        message2print = "{passing" + "; object-passing-to-memoize-by-save-dict-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: {'authkey': b'\xfd\xf1\x0ej\xc4\xa4^\xa9\x80/\x12\xd5\xd9\x87?\x8e\xcf\x91\x02\x80$v\xe5}HZ\xe0E\xf0z\x90\xcd', 'sys_path': ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages'], 'name': 'Process4Xi-1', 'orig_dir': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'sys_argv': ['hello_world4xi.py'], 'main_path': 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py', 'log_to_stderr': True}; time: 12:49:00 04/07/10 FLE Daylight Time}

        # ----------------------------------------------------------------------------------------- 
        # OK
        # -----------------------------------------------------------------------------------------
 
        try:
           
            self.memoize(obj)
            
         
            nack4memoize = False
            ack4memoize = True 
            
            message2print = "{acknowledged" + "; ack4memoize-acknowledged-by-save-dict-of-ForkingPickler4Xi: " + str(ack4memoize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
   
        except:
            message2print = "{yielded" + "; nack4memoize-3-yielded-by-save-dict-of-ForkingPickler4Xi: " + str(nack4memoize)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
                                     
        # ----------------------------------------------------------------------------------------- 
        # OK
        # -----------------------------------------------------------------------------------------
                
        ack4batch = False
        nack4batch = True
        
        try:            
            self._batch_setitems(obj.items())    
        
            nack4batch = False
            ack4batch = True
            
            message2print = "{acknowledged" + "; ack4batch-acknowledged-by-save-dict-of-ForkingPickler4Xi: " + str(ack4batch) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
               
            
        except:
            message2print = "{yielded" + "; nack4batch-yielded-by-save-dict-of-ForkingPickler4Xi: " + str(nack4batch)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                     
        # ----------------------------------------------------------------------------------------- 
        # OK
        # -----------------------------------------------------------------------------------------
              
        message2print = "{returning" + "; save-dict-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #HW, #PX: 18:07:07 04/07/10 FLE Daylight Time
 

    #</methodize4xi>                        

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[dict] = save_dict    
    if PyStringMap is not None:
        dispatch[PyStringMap] = save_dict
    #"
    #/>
    
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                   
       
                   
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "batch-setitems-of-ForkingPickler4Xi" usage = "(self, items)">
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def _batch_setitems(self, items):
        class2print = "ForkingPickler4Xi"
        method2print = "batch-setitems"       
        
        message2print = "{started" + "; batch-setitems-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: forkingpickler4xi.ForkingPickler4Xi object at 0x02C4DE70
        
        message2print = "{passed" + "; items-passed-to-batch-setitems-of-ForkingPickler4Xi: " + str(items) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #HW: dict_items([('authkey', b'\x9d\x9cq\xb8(#!S\x15O\xf7\xf7\x9e\xe77\xd8\x8aG\xf6/\x13\xcd\xf6\xee\xff\xeb\x84\x96 \xbcJ\xd8'),
        #                ('sys_path', ['e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi', 'c:\\sw\\python32', 'C:\\Windows\\system32\\python32.zip', 'c:\\sw\\python32\\DLLs', 'c:\\sw\\python32\\lib', 'c:\\sw\\python32\\lib\\plat-win', 'c:\\sw\\python32\\lib\\site-packages']),
        #                ('name', 'Process4Xi-1'),
        #                ('orig_dir', 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi'),
        #                ('sys_argv', ['hello_world4xi.py']),
        #                ('main_path', 'e:\\Sw4Xi\\python32\\Lib\\multiprocessing4xi\\hello_world4xi.py'),
        #                ('log_to_stderr', True)])
        
        
        
        # Helper to batch up SETITEMS sequences; proto >= 1 only
        save = self.save
        write = self.write

        if not self.bin:
            for k, v in items:
                save(k)
                save(v)
                write(SETITEM)
            return

        items = iter(items)
        r = range(self._BATCHSIZE)
        while items is not None:
            tmp = []
            for i in r:
                try:
                    tmp.append(next(items))
                except StopIteration:
                    items = None
                    break
            n = len(tmp)
            if n > 1:
                write(MARK)
                for k, v in tmp:
                    save(k)
                    save(v)
                write(SETITEMS)
            elif n:
                k, v = tmp[0]
                save(k)
                save(v)
                write(SETITEM)
            # else tmp is empty, and we're done 
               
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{returning" + "; batch-setitems-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 16:15:38 02/11/10 FLE Standard Time       
    #</methodize4xi>
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "save-global-of-ForkingPickler4Xi" usage = "(self, obj, name=None, pack=struct.pack)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------     
    def save_global(self, obj, name=None, pack=struct.pack):
               
        # ----------------------------------------------------------------------------------------- 
        #
        # -----------------------------------------------------------------------------------------
         
        class2print = "ForkingPickler4Xi"
        method2print = "save-global"       
        
        
        message2print = "{started" + "; save-global-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #HW: 13:44:12 03/16/10 FLE Standard Time
             
        message2print = "{passed" + "; object-passed-to-save-global-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: <class 'process4xi.Process4Xi'>, <function f4xi at 0x02C7F618>
        #PX: <class 'process4xi.Process4Xi'>
             
        message2print = "{passed" + "; name-passed-to-save-global-of-ForkingPickler4Xi: " + str(name) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: None
             
        message2print = "{passed" + "; pack-passed-to-save-global-of-ForkingPickler4Xi: " + str(pack) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: <built-in function pack>           
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        
        write = self.write

        message2print = "{set" + ";  method-write-set-by-save-global-of-ForkingPickler4Xi to: " + str( write)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: <built-in method write of _io.BufferedWriter object at 0x02A5ADB0>
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        
        memo = self.memo
        
        message2print = "{set" + "; memo-set-by-save-global-of-ForkingPickler4Xi to: " + str(memo)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: {}
        #    {45560328: (0, <class 'process4xi.Process4Xi'>), 
        #     46714928: (1, <Process4Xi(Process4Xi-1, initial)>), 
        #     42528224: (3, '_daemonic'), 
        #     46670448: (2, {'_daemonic': False,
        #    '_target': <function f4xi at 0x02C7F618>, 
        #      '_args': (),
        #   '_tempdir': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-xdzszc', 
        #      '_name': 'Process4Xi-1',
        #   '_authkey': b'L\x9eU\xaf\x1e\x81k\x98X|\x0e\x82\x87\xa2\xae\xe1+*f=\x8c\x12M\x90U\xb3zSi=\x00)', 
        #'_parent_pid': 4884, 
        #    '_kwargs': {},
        #  '_identity': (1,),
        #     '_popen': None}),
        #     42528544: (4, '_target')}

        #PX: {49123352: (0, <class 'process4xi.Process4Xi'>), 
        #     46550096: (1, <Process4Xi(SyncManager4Xi-1, initial)>),
        #     44261440: (4, '_target'),
        #     46764512: (2, {'_daemonic': False,
        #                      '_target': <bound method type._run_server of <class 'syncmanager4xi.SyncManager4Xi'>>,
        #                        '_args': ({'AsyncResult': (None, None, None, <function AutoProxy4Xi at 0x03692C48>), 'Condition4Xi': (<function Condition at 0x02957738>, ('wait', 'notify', 'notify_all', 'sense', 'reason', 'ace'), None, <class 'conditionproxy4xi.ConditionProxy4Xi'>), 'Event4Xi': (<function Event at 0x02957E40>, ('is_set', 'set', 'clear', 'wait', 'sense', 'reason', 'ace'), None, <class 'eventproxy4xi.EventProxy4Xi'>), 'Iterator': (None, ('send', 'throw', 'close', 'sense', 'reason', 'ace'), None, <class 'iteratorproxy4xi.IteratorProxy4Xi'>), 'BoundedSemaphore4Xi': (<function BoundedSemaphore at 0x02957CD8>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 'Array4Xi': (<function Array4Xi at 0x03692618>, ('__len__', '__getitem__', '__setitem__'), None, <class 'arrayproxy4xi.ArrayProxy4Xi'>), 'list': (<class 'list'>, ('sense', 'reason', 'ace'), None, <class 'listproxy4xi.ListProxy4Xi'>), 'Queue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x03692C48>), 
        #                                       'Pool4Xi': (<class 'pool4xi.Pool4Xi'>, ('apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join', 'map', 'map_async', 'terminate', 'sense', 'reason', 'ace'), {'apply_async': 'AsyncResult', 'imap_unordered': 'Iterator', 'imap': 'Iterator', 'map_async': 'AsyncResult'}, <class 'poolproxy4xi.PoolProxy4Xi'>), 'Lock4Xi': (<built-in function allocate_lock>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 'Semaphore4Xi': (<function Semaphore at 0x029579C0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 'RLock4Xi': (<function RLock at 0x029576F0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 'dict': (<class 'dict'>, ('__contains__', '__delitem__', '__getitem__', '__len__', '__setitem__', 'clear', 'copy', 'get', 'has_key', 'items', 'keys', 'pop', 'popitem', 'setdefault', 'update', 'values', 'sense', 'reason', 'ace'), None, <class 'dictproxy4xi.DictProxy4Xi'>), 
        #                              'JoinableQueue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x03692C48>), 'Value4Xi': (<class 'value4xi.Value4Xi'>, ('get', 'set', 'sense', 'reason', 'ace'), None, <class 'valueproxy4xi.ValueProxy4Xi'>), 'Namespace4Xi': (<class 'namespace4xi.Namespace4Xi'>, ('__getattribute__', '__setattr__', '__delattr__', 'sense', 'reason', 'ace'), None, <class 'namespaceproxy4xi.NamespaceProxy4Xi'>)
        #                   },
        #                   '\\\\.\\pipe\\pyc-1232-0-f70ixj', 
        #                   b"\x1a\x1f\x19\x1e\xae\x1b\x11\x81o'\xbb\xa5<\xc2\xcc\xd0N\x9d1\x15\xcd=Pc\x9e\xe7\xc1o\xc2>ha", 
        #                   'pickle',
        #                    <write-only PipeConnection, handle 420>),
        #                   '_tempdir': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-d97giq', 
        #                   '_name': 'SyncManager4Xi-1',
        #                   '_authkey': b"\xfc'\xbfC\xcf\x12\xfc*\x98\x8e\xda\x8b\x19{\x0f\xb6\xc0\xa5reO\x9f\xfe\xa2\xc72\x9fW\xc3\x18f\xc4", 
        #                   '_parent_pid': 1232, 
        #                   '_kwargs': {},
        #                   '_identity': (1,),
        #                   '_popen': None}),
        #     44261664: (3, '_daemonic')}

        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        message2print = "{checking" + "; name-checking-by-save-global-of-ForkingPickler4Xi: " + str(name) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        #HW, #PX: None
               
        if name is None:
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    

            name = obj.__name__
            
            message2print = "{set" + "; name-set-by-save-global-of-ForkingPickler4Xi to: " + str(name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #HW: Process4Xi, f4xi
            #PX: method
            
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 

        module = getattr(obj, "__module__", None)
        
        message2print = "{got" + "; module-got-by-save-global-of-ForkingPickler4Xi to: " + str(module)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        #HW: __main__
        #
        
        # -----------------------------------------------------------------------------------------    
        # 
        # -----------------------------------------------------------------------------------------                 
        
        if module is None:
            message2print = "{calling" + "; whichmodule-calling-by-save-global-of-ForkingPickler4Xi: " + str(whichmodule) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{passing" + "; object-passing-to-whichmodule-by-save-global-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                
            message2print = "{passing" + "; name-passing-to-whichmodule-by-save-global-of-ForkingPickler4Xi: " + str(name) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
  
            module = whichmodule(obj, name)
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
            message2print = "{set" + "; module-set-by-save-global-of-ForkingPickler4Xi to: " + str(module)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <module '__main__' from 'hello_world4xi.py'>
        
        # -----------------------------------------------------------------------------------------    
        # 
        # -----------------------------------------------------------------------------------------                 
        message2print = "{calling" + "; import-calling-by-save-global-of-ForkingPickler4Xi: " + str(__import__) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #HW: <built-in function __import__>
            
        message2print = "{passing" + "; module-passing-to-import-by-save-global-of-ForkingPickler4Xi: " + str(module) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: process4xi
            
        message2print = "{passing" + "; level-passing-to-import-by-save-global-of-ForkingPickler4Xi: " + str(0) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #HW: 0
                           
            
        try:
              

            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
                
            
            __import__(module, level=0)
            
            message2print = "{performed" + "; import-performed-by-save-global-of-ForkingPickler4Xi: " + str(__import__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #HW: x. : <built-in function __import__>
                            
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
                
            mod = sys.modules[module]
            
            message2print = "{set" + "; module-set-by-save-global-of-ForkingPickler4Xi to: " + str(mod)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #HW: <module '__main__' from 'hello_world4xi.py'>
            
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
                
            klass = getattr(mod, name)
            
            
            # -----------------------------------------------------------------------------------------    
            #
            # ----------------------------------------------------------------------------------------- 
               
            message2print = "{set" + "; klass-set-by-save-global-of-ForkingPickler4Xi to: " + str(klass)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #HW: <function f4xi at 0x02C7F618>
                

        except (ImportError, KeyError, AttributeError):

            # -----------------------------------------------------------------------------------------    
            # 
            # -----------------------------------------------------------------------------------------    
     
            message2print = "{yielded" + "; pickling-error-of-object-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(obj)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #TOKEN4XI: <class 'method'>
                    
            message2print = "{yielded" + "; pickling-error-of-module-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(module)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #TOKEN4XI: builtins
            
            message2print = "{yielded" + "; pickling-error-of-name-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(name)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #TOKEN4XI: method
            

        else:
            if klass is not obj:
                                  
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------    
                      
                message2print = "{yielded" + "; object-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(obj)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                    
                message2print = "{yielded" + "; module-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(module)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
 
                message2print = "{yielded" + "; name-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(name)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)           
                             
                
                message2print = "{yielded" + "; klass-yielded-by-save-global-of-ForkingPickler4Xi to: " + str(klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                                  

                    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
                    
                    
        if self.proto >= 2:
            
            # -----------------------------------------------------------------------------------------    
            #
            # ----------------------------------------------------------------------------------------- 
              
            code = _extension_registry.get((module, name))
            

            message2print = "{got" + "; code-got-by-save-global-of-ForkingPickler4Xi: " + str(code)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)

            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------    
                
            if code:

                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------    
                  
                assert code > 0

                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------    
                      
                if code <= 0xff:
            
                    # -----------------------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------------------- 
                      
                    message2print = "{calling" + "; write-calling-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)       
                                
                    message2print = "{passing" + "; EXT1-passing-to-write-by-save-global-of-ForkingPickler4Xi: " + str(EXT1 + bytes([code])) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                
                    write(EXT1 + bytes([code]))
                    
                    message2print = "{performed" + "; write-performed-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)

                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------    
                                               
                elif code <= 0xffff:
            
                    # -----------------------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------------------- 
                       
                    message2print = "{calling" + "; write-calling-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    message2print = "{passing" + "; EXT2-passing-to-write-by-save-global-of-ForkingPickler4Xi: " + str(EXT2 + bytes([code&0xff, code>>8])) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                   
                    write(EXT2 + bytes([code&0xff, code>>8]))
                    
                    message2print = "{performed" + "; write-performed-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)

                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------    
                                                 
                else:
            
                    # -----------------------------------------------------------------------------------------    
                    #
                    # ----------------------------------------------------------------------------------------- 
                       
                    message2print = "{calling" + "; write-calling-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    message2print = "{passing" + "; EXT4-passing-to-write-by-save-global-of-ForkingPickler4Xi: " + str(EXT4 + pack("<i", code)) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                       
                    write(EXT4 + pack("<i", code))
                    
                    message2print = "{performed" + "; write-performed-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             

                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------    
     
                message2print = "{returning" + "; save-global-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)                 
                                          
                return
                
        # -----------------------------------------------------------------------------------------    
        # Non-ASCII identifiers are supported only with protocols >= 3.
        # -----------------------------------------------------------------------------------------                 
        message2print = "{checking" + "; proto-checking-by-save-global-of-ForkingPickler4Xi: " + str(self.proto) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
                                
        
        if self.proto >= 3:
            
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------  
               
            message2print = "{calling" + "; write-calling-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            message2print = "{passing" + "; GLOBAL-passing-to-write-by-save-global-of-ForkingPickler4Xi: " + str(GLOBAL + bytes(module, "utf-8") + b'\n' + bytes(name, "utf-8") + b'\n') + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            ack2globalwrite = False
            nack2globalwrite = True
                
            try:
            
                # -----------------------------------------------------------------------------------------    
                #
                # -----------------------------------------------------------------------------------------  
                                             
                write(GLOBAL + bytes(module, "utf-8") + b'\n' + bytes(name, "utf-8") + b'\n')
         
                nack2globalwrite = False
                ack2globalwrite = True

                message2print = "{acknowledged" + "; ack2globalwrite-acknowledged-by-save-global-of-ForkingPickler4Xi: " + str(ack2globalwrite) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
            except:
                message2print = "{yielded" + "; nack2globalwrite-yielded-by-save-global-of-ForkingPickler4Xi: " + str(nack2globalwrite) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                            
        else:
            
            # -----------------------------------------------------------------------------------------    
            # self.proto < 3
            # -----------------------------------------------------------------------------------------  
             
            try:
            
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------  
                    
                message2print = "{calling" + "; write-calling-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                
                message2print = "{passing" + "; GLOBAL-passing-to-write-by-save-global-of-ForkingPickler4Xi: " + str(GLOBAL + bytes(module, "ascii") + b'\n' + bytes(name, "ascii") + b'\n') + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                                            
                write(GLOBAL + bytes(module, "ascii") + b'\n' + bytes(name, "ascii") + b'\n')
                
                message2print = "{performed" + "; write-performed-by-save-global-of-ForkingPickler4Xi: " + str(write) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                 
                
            except UnicodeEncodeError:
            
                # -------------------------------------------------------------------------------------    
                #
                # -------------------------------------------------------------------------------------  
                
            
                message2print = "{yielded" + "; module-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(module)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                message2print = "{yielded" + "; name-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(name)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                message2print = "{yielded" + "; proto-yielded-by-save-global-of-ForkingPickler4Xi: " + " *** " + str(self.proto)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                    
                                

                    
        # -----------------------------------------------------------------------------------------    
        # all protos
        # -----------------------------------------------------------------------------------------
                         
        ack2memoize = False
        nack2memoize = True
        
        message2print = "{calling" + "; memoize-calling-by-save-global-of-ForkingPickler4Xi: " + str(self.memoize) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                
        message2print = "{passing" + "; object-passing-to-memoize-by-save-global-of-ForkingPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
                       
        try:
            # -------------------------------------------------------------------------------------    
            #
            # -------------------------------------------------------------------------------------          
            self.memoize(obj)
                    
            nack2memoize = False
            ack2memoize = True
            
            message2print = "{acknowledged" + "; ack2memoize-acknowledged-by-save-global-of-ForkingPickler4Xi: " + str(ack2memoize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                        
        except:
            message2print = "{yielded" + "; nack2memoize-yielded-by-save-global-of-ForkingPickler4Xi: " + str(nack2memoize) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                           
                    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------
              
        message2print = "{returning" + "; save-global-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 

    #</methodize4xi>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    dispatch[FunctionType] = save_global
    dispatch[BuiltinFunctionType] = save_global
    dispatch[type] = save_global
    #"
    #/>
         
    message2print = "{sharing" + "; dispatch-sharing-by-forkingpickler2xi-of-ForkingPickler4Xi: " + str(dispatch)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                      
                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ForkingPickler4Xi" usage = "senses ForkingPickler4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses ForkingPickler4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        class2print = 'ForkingPickler4Xi'
        method2print = 'sense'
        
        message2print = "{started" + "; sense-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; ForkingPickler4Xi-sensing-by-sense-of-ForkingPickler4Xi: " + str(sense)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-ForkingPickler4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['dispatch-of-ForkingPickler4Xi'] = str(ForkingPickler4Xi.dispatch)
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-ForkingPickler4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
           
            
            
        message2print = "{sensed" + "; ForkingPickler4Xi-sensed-by-sense-of-ForkingPickler4Xi: " + str(sense)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-ForkingPickler4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                   
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ForkingPickler4Xi" usage = "reasons ForkingPickler4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons ForkingPickler4Xi class objects
        '''
                
        
        class2print = 'ForkingPickler4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; ForkingPickler4Xi-reasoning-by-reason-of-ForkingPickler4Xi: " + str(ForkingPickler4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-ForkingPickler4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        message2print = "{reasoned" + "; ForkingPickler4Xi-reasoned-by-reason-of-ForkingPickler4Xi: " + str(ForkingPickler4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-ForkingPickler4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                 
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ForkingPickler4Xi" usage = "aces ForkingPickler4Xi class objects ">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces ForkingPickler4Xi class objects 
        '''
        
        class2print = 'ForkingPickler4Xi'
        method2print = 'ace'
        
  
        message2print = "{started" + "; ace-of-ForkingPickler4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; ForkingPickler4Xi-acing-by-ace-of-ForkingPickler4Xi: " + str(ForkingPickler4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-ForkingPickler4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
        message2print = "{aced" + "; ForkingPickler4Xi-aced-by-ace-of-ForkingPickler4Xi: " + str(ForkingPickler4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-ForkingPickler4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                             
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
#</class4xi>

# -------------------------------------------------------------------------------------------------
# Pickling helpers
# -------------------------------------------------------------------------------------------------



# -------------------------------------------------------------------------------------------------
#<function4xi function = "keep-alive" usage = "
#   Keeps a reference to the object x in the memo
#"> 
# -------------------------------------------------------------------------------------------------
def _keep_alive(x, memo):
    """Keeps a reference to the object x in the memo.

    Because we remember objects by their id, we have
    to assure that possibly temporary objects are kept
    alive by referencing them.
    We store a reference at the id of the memo, which should
    normally not be used unless someone tries to deepcopy
    the memo itself...
    """
    
    # --------------------------------------------------------------------------------------------- 
    # (Step 1/3)
    # ---------------------------------------------------------------------------------------------           
    
    class2print = 'ForkingPickler4Xi'
    method2print = '_keep_alive'
    
    message2print = "{(Step 1/3) started" + "; keep-alive-of-ForkingPickler4Xi-started: " + str(_keep_alive)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
        
          
            
    # --------------------------------------------------------------------------------------------- 
    # (Step 2/3)
    # ---------------------------------------------------------------------------------------------           
               
    try:
        memo[id(memo)].append(x)
        
        message2print = "{(Step 2/3a) set" + "; memo-set-by-keep-alive-of-ForkingPickler4Xi to: " + str(memo)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                
    except KeyError:
        # this is the first one
        memo[id(memo)]=[x]
        
        message2print = "{(Step 2/3b) set" + "; memo-set-by-keep-alive-of-ForkingPickler4Xi to: " + str(memo)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
            
    # --------------------------------------------------------------------------------------------- 
    # (Step 2/2)
    # ---------------------------------------------------------------------------------------------           
    message2print = "{returning" + "; keep-alive-of-ForkingPickler4Xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print) 
                        
  
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<define4xi definition = "
# A cache for whichmodule(), mapping a function object to the name of
# the module in which the function was found.

classmap = {} # called classmap for backwards compatibility
#"
#/>


# -------------------------------------------------------------------------------------------------
#<function4xi function = "whichmodule" usage = "
#   Figure out the module in which a function occurs
#"> 
# -------------------------------------------------------------------------------------------------
def whichmodule(func, funcname):
    """Figure out the module in which a function occurs.

    Search sys.modules for the module.
    Cache in classmap.
    Return a module name.
    If the function cannot be found, return "__main__".
    """
    
    # --------------------------------------------------------------------------------------------- 
    # (Step 1/4)
    # ---------------------------------------------------------------------------------------------           
    
    class2print = 'ForkingPickler4Xi'
    method2print = 'whichmodule'
    
    message2print = "{(Step 1/4a) passed" + "; function-passed-to-whichmodule-of-ForkingPickler4Xi: " + str(func) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
    
    message2print = "{(Step 1/4b) passed" + "; funcname-passed-to-whichmodule-of-ForkingPickler4Xi: " + str(funcname) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
       
    
    # --------------------------------------------------------------------------------------------- 
    # (Step 2/4)
    # ---------------------------------------------------------------------------------------------           
    mod = None
    try:               
        # Python functions should always get an __module__ from their globals.
        mod = getattr(func, "__module__", None)
        
    
        if mod is not None:
            message2print = "{(Step 2/4a) returning" + "; module-returning-by-whichmodule-of-ForkingPickler4Xi: " + str(mod) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                
        
            return mod
    except:
        message2print = "{(Step 2/4b) yielded" + "; module-yielded-by-whichmodule-of-ForkingPickler4Xi: " + str(mod)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
    
    # --------------------------------------------------------------------------------------------- 
    # (Step 3/4)
    # ---------------------------------------------------------------------------------------------           
    try:        
        if func in classmap:
        
            message2print = "{(Step 3/4a) returning" + "; classmap-function-returning-by-whichmodule-of-ForkingPickler4Xi: " + str(classmap[func]) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)     
            
            return classmap[func]
    except:
        message2print = "{(Step 3/4b) yielded" + "; function-yielded-by-whichmodule-of-ForkingPickler4Xi: " + str(func)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
    # --------------------------------------------------------------------------------------------- 
    # (Step 4/4)
    # ---------------------------------------------------------------------------------------------           
    ack2name = False
    nack2name = True
    
    try:
        for name, module in list(sys.modules.items()):

            if module is None:
                continue # skip dummy package entries
            if name != '__main__' and getattr(module, funcname, None) is func:
                break
        else:
            name = '__main__'
            
        nack2name = False
        ack2name = True
        
        message2print = "{acknowledged" + "; ack2name-acknowledged-by-whichmodule-of-ForkingPickler4Xi: " + str(ack2name) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                        

        classmap[func] = name
        
    except:
        message2print = "{(Step 4/4a) yielded" + "; nack2name-yielded-by-whichmodule-of-ForkingPickler4Xi: " + str(nack2name)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)     

        name = None
        
    message2print = "{(Step 4/4b) returning" + "; name-returning-by-whichmodule-of-ForkingPickler4Xi: " + str(name) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)          
        
    return name       
#</function4xi>

# =================================================================================================
# UNPICKLING MACHINERY - HUGE ONE !
# =================================================================================================

# =================================================================================================
#<class4xi class = "Unpickler" base = "None" child = "None"
#   synopsis = "takes a binary file for reading a pickle data stream"
#   usage = "embedded inside forkingpickler4xi.py and called by load and loads functions"
#>
# =================================================================================================
class _Unpickler:

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Unpickler" usage = "initializes Unpickler">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------    
    def __init__(self, file, *, encoding="ASCII", errors="strict"):
        """This takes a binary file for reading a pickle data stream.

        The protocol version of the pickle is detected automatically, so no
        proto argument is needed.

        The file-like object must have two methods, a read() method
        that takes an integer argument, and a readline() method that
        requires no arguments.  Both methods should return bytes.
        Thus file-like object can be a binary file object opened for
        reading, a BytesIO object, or any other custom object that
        meets this interface.

        Optional keyword arguments are encoding and errors, which are
        used to decode 8-bit string instances pickled by Python 2.x.
        These default to 'ASCII' and 'strict', respectively.
        """
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/7) 
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'init'
     
        message2print = "{(Step 1/7a) started" + "; init-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                         
        message2print = "{(Step 1/7b) borning" + "; unpickler2xi-of-Unpickler-borning: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                
        message2print = "{(Step 1/7c) passed" + "; file-passed-to-init-of-Unpickler: " + str(file) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
                     
                
        message2print = "{(Step 1/7d) passed" + "; encoding-passed-to-init-of-Unpickler: " + str(encoding) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
   
                
        message2print = "{(Step 1/7e) passed" + "; errors-passed-to-init-of-Unpickler: " + str(errors) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
                                   
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------           
    
        self.readline = file.readline
     
        message2print = "{(Step 2/7) set" + "; readline-of-unpickler2xi-set-by-init-of-Unpickler to: " + str(self.readline)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    

        # ----------------------------------------------------------------------------------------- 
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------           
            
        self.read = file.read
     
        message2print = "{(Step 3/7) set" + "; read-of-unpickler2xi-set-by-init-of-Unpickler to: " + str(self.read)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------           
            
        self.memo = {}
     
        message2print = "{(Step 4/7) set" + "; memo-of-unpickler2xi-set-by-init-of-Unpickler to: " + str(self.memo)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------           
            
        self.encoding = encoding
     
        message2print = "{(Step 5/7) set" + "; encoding-of-unpickler2xi-set-by-init-of-Unpickler to: " + str(self.encoding)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------           
            
        self.errors = errors
     
        message2print = "{(Step 6/7) set" + "; errors-of-unpickler2xi-set-by-init-of-Unpickler to: " + str(self.errors)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------           
        message2print = "{ (Step 7/7) returning" + "; init-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
 
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-of-Unpickler" usage = "Read a pickled object representation from the open file">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # --------------------------------------------------------------------------------------------- 
    def load(self):
        """Read a pickled object representation from the open file.

        Return the reconstituted object hierarchy specified in the file.
        """
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/9)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load'
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/9)
        # ----------------------------------------------------------------------------------------- 
        ack2read = False
        nack2read = True 
                 
        if not hasattr(self, "read"):
        
            message2print = "{(Step 2/9) yielded" + "; nack2read-yielded-by-load-of-Unpickler: " + str(nack2read)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        else:
            nack2read = False
            ack2read = True 

            message2print = "{acknowledged" + "; ack2read-acknowledged-by-load-of-Unpickler: " + str(ack2read) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/9)
        # ----------------------------------------------------------------------------------------- 
                                                     
        self.mark = object() # any new unique object
        
        message2print = "{(Step 3/9) set" + "; mark-of-unpickler2xi-set-by-load-of-Unpickler to: " + str(self.mark)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                        
                        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/9)
        # ----------------------------------------------------------------------------------------- 
            
        self.stack = []
        
        message2print = "{(Step 4/9) set" + "; stack-of-unpickler2xi-set-by-load-of-Unpickler to: " + str(self.stack)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/9)
        # ----------------------------------------------------------------------------------------- 
            
        self.append = self.stack.append
        
        message2print = "{(Step 3/9) set" + "; append-of-unpickler2xi-set-by-load-of-Unpickler to: " + str(self.append)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/9)
        # ----------------------------------------------------------------------------------------- 
        if ack2read is True:
            
            read = self.read
            
            message2print = "{(Step 3/9a) set" + "; read-set-by-load-of-Unpickler to: " + str(read)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 3/9b) yielded" + "; nack2read-yielded-by-load-of-Unpickler: " + str(nack2read)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

        # ----------------------------------------------------------------------------------------- 
        # (Step 7/9)
        # ----------------------------------------------------------------------------------------- 
            
        dispatch = self.dispatch
        
        message2print = "{(Step 7/9) set" + "; dispatch-set-by-load-of-Unpickler to: " + str(dispatch)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        #
        # ----------------------------------------------------------------------------------------- 
        if ack2read is True:    
            try:
         
                # ------------------------------------------------------------------------------------- 
                #
                # ------------------------------------------------------------------------------------- 
                
                while 1:
                
                    # --------------------------------------------------------------------------------- 
                    # (Step 8/9)
                    # --------------------------------------------------------------------------------- 
                            
                    key = read(1)
                    
                    message2print = "{Step 8/9a) set" + "; key-set-by-load-of-Unpickler to: " + str(key)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                                    
                    
                    if not key:
                        # ----------------------------------------------------------------------------- 
                        # 
                        # ----------------------------------------------------------------------------- 
                             
                        raise EOFError
                    
                    if isinstance(key, bytes_types):
                    
                    
                        dispatch[key[0]](self)
                    
                        message2print = "{Step 8/9a) set" + "; dispatch-set-by-load-of-Unpickler to: " + str(dispatch)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                                        
                                        
            except _Stop as stopinst:
         
                # ------------------------------------------------------------------------------------- 
                #
                # ------------------------------------------------------------------------------------- 
               
                return stopinst.value

        # ----------------------------------------------------------------------------------------- 
        # (Step 9/9)
        # ----------------------------------------------------------------------------------------- 
        else:
            return None
                                     
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "marker-of-Unpickler" usage = "
    # Return largest index k such that self.stack[k] is self.mark.
    # If the stack doesn't contain a mark, eventually raises IndexError.
    # This could be sped by maintaining another stack, of indices at which
    # the mark appears.  For that matter, the latter stack would suffice,
    # and we wouldn't need to push mark objects on self.stack at all.
    # Doing so is probably a good thing, though, since if the pickle is
    # corrupt (or hostile) we may get a clue from finding self.mark embedded
    # in unpickled objects.
    #">
    # ---------------------------------------------------------------------------------------------     
    def marker(self):
    
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'marker'
        
        message2print = "{(Step 1/6) started" + "; marker-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                         
                                
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        
        stack = None
        
        try:  
            stack = self.stack
     
            message2print = "{(Step 2/6a) set" + "; stack-set-by-marker-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            nack2stack = False
            ack2stack = True  
            
            message2print = "{acknowledged" + "; ack2stack-acknowledged-by-marker-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                            
        except:
            message2print = "{(Step 2/6c) yielded" + "; nack2stack-yielded-by-marker-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
   
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------           
        ack2mark = False
        nack2mark = True
        
        mark = None
        
        try:      
            mark = self.mark
     
            message2print = "{(Step 3/6a) set" + "; mark-set-by-marker-of-Unpickler to: " + str(mark)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            nack2mark = False
            ack2mark = True
            
            message2print = "{acknowledged" + "; ack2mark-acknowledged-by-marker-of-Unpickler: " + str(ack2mark) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                  
        except:
            message2print = "{(Step 3/6c) yielded" + "; nack2mark-yielded-by-marker-of-Unpickler: " + str(nack2mark)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------           

        if ack2stack is True:      
            k = len(stack)-1
            
            message2print = "{(Step 4/6a) set" + "; k-set-by-marker-of-Unpickler to: " + str(k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        else:
            k = 0
            
            message2print = "{(Step 4/6b) set" + "; k-set-by-marker-of-Unpickler to: " + str(k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------           
        if ack2stack is True: 
        
            try:      
                while stack[k] is not mark:
                    k = k-1
            except:
                message2print = "{(Step 5/6a) yielded" + "; ack2stack-yielded-by-marker-of-Unpickler: " + str(ack2stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 5/6b) yielded" + "; ack2stack-yielded-by-marker-of-Unpickler: " + str(ack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 6/6) returning" + "; k-returning-by-marker-of-Unpickler: " + str(k) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                      
        return k
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "persistent-load-of-Unpickler" usage = "raises unsupported persistent id encountered">
    # --------------------------------------------------------------------------------------------- 
    def persistent_load(self, pid):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/2)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'persistent-load'
     
        message2print = "{(Step 1/2) passed" + "; pid-passed-to-persistent-load-of-Unpickler: " + str(pid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/2)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 2/2) yielded" + "; pid-yielded-by-persistent-load-of-Unpickler: " + str(pid)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 

    

    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch = {}
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-proto-of-Unpickler" usage = "loads proto">
    # --------------------------------------------------------------------------------------------- 
    def load_proto(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/5) 
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-proto'
     
        message2print = "{(Step 1/5) started" + "; load_proto-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                         
                                
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/5) 
        # -----------------------------------------------------------------------------------------
                   
        ack2proto = False
        nack2proto = True 
                  
        if not hasattr(self, "read"):
        

            message2print = "{(Step 2/5a) yielded" + "; nack2proto-yielded-by-load-of-Unpickler: " + str(nack2proto)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        else:
            nack2proto = False
            ack2proto = True 
            
            message2print = "{(Step 2/5b) acknowledged" + "; ack2proto-acknowledged-by-load-of-Unpickler: " + str(ack2proto) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                                
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/5) 
        # -----------------------------------------------------------------------------------------
        if ack2proto is True:                   
            try:
                proto = ord(self.read(1))

                message2print = "{(Step 3/5a) set" + "; proto-set-by-load-of-Unpickler to: " + str(proto)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                            
            except:
                nack2proto = True
                
                message2print = "{(Step 3/5b) yielded" + "; nack2proto-yielded-by-load-of-Unpickler: " + str(nack2proto)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
              
        else:
            message2print = "{(Step 3/5c) yielded" + "; nack2proto-yielded-by-load-of-Unpickler: " + str(nack2proto)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
        
                                
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/5) 
        # -----------------------------------------------------------------------------------------
             
        if not 0 <= proto <= HIGHEST_PROTOCOL:
        
            proto = HIGHEST_PROTOCOL

            message2print = "{(Step 4/5a) set" + "; proto-set-by-load-of-Unpickler to: " + str(proto)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            nack2proto = True
                
            message2print = "{(Step 4/5a) yielded" + "; nack2proto-yielded-by-load-of-Unpickler: " + str(nack2proto)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
                                
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/5) 
        # -----------------------------------------------------------------------------------------
        
        message2print = "{returning" + "; load-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[PROTO[0]] = load_proto
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-persid-of-Unpickler" usage = "loads persistent id">
    # --------------------------------------------------------------------------------------------- 
    def load_persid(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-persid'
        
        message2print = "{(Step 1/4) started" + "; load-persid-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                         
                                
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2pid = False
        nack2pid = True
        
        try:   
            pid = self.readline()[:-1].decode("ascii")

            message2print = "{(Step 2/4a) set" + "; pid-set-by-load-persid-of-Unpickler to: " + str(pid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            nack2pid = False
            ack2pid = True  
            
            message2print = "{acknowledged" + "; ack2pid-acknowledged-by-load-persid-of-Unpickler: " + str(ack2pid) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                            
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2pid-yielded-by-load-persid-of-Unpickler: " + str(nack2pid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2append= False
        nack2append = True
        
        if ack2pid is True:           
            try:
                self.append(self.persistent_load(pid))
            
                message2print = "{(Step 3/4a) performed" + "; append-of-unpickler2xi-performed-by-load-persid-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                nack2append= False
                ack2append = True                
                
                message2print = "{acknowledged" + "; ack2append-acknowledged-by-load-persid-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                 

            except:
                message2print = "{(Step 3/4b) yielded" + "; nack2pid-yielded-by-load-persid-of-Unpickler: " + str(nack2pid)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
        else:
            message2print = "{(Step 3/4c) yielded" + "; nack2pid-yielded-by-load-persid-of-Unpickler: " + str(nack2pid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
                   
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-persid-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                           
                   
                           
                
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[PERSID[0]] = load_persid
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binpersid-of-Unpickler" usage = "loads binpersid">
    # --------------------------------------------------------------------------------------------- 
    def load_binpersid(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binpersid'
     
        message2print = "{(Step 1/4) started" + "; load-binpersid-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                         
                                
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2pid = False
        nack2pid = True
        
        try:    
            pid = self.stack.pop()
            
            message2print = "{(Step 2/4a) set" + "; pid-set-by-load-binpersid-of-Unpickler to: " + str(pid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        
            nack2pid = False
            ack2pid = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2pid-acknowledged-by-load-binpersid-of-Unpickler: " + str(ack2pid) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2pid-yielded-by-load-binpersid-of-Unpickler: " + str(nack2pid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        if ack2pid is True:      
        
            try:
                self.append(self.persistent_load(pid))
        
                message2print = "{(Step 3/4a) performed" + "; append-of-unpickler2xi-performed-by-load-binpersid-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
            except:
                message2print = "{(Step 3/4b) yielded" + "; nack2pid-yielded-by-load-binpersid-of-Unpickler: " + str(nack2pid)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
        else:
            message2print = "{(Step 3/4c) yielded" + "; nack2pid-yielded-by-load-binpersid-of-Unpickler: " + str(nack2pid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
      
        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BINPERSID[0]] = load_binpersid
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-none-of-Unpickler" usage = "loads none">
    # --------------------------------------------------------------------------------------------- 
    def load_none(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-none'

        message2print = "{(Step 1/3) started" + "; load-none-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                         
                                     
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        try:       
            self.append(None)
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-none-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        
            nack2append = False
            ack2append = True            
        
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-none-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        except:
        
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-none-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                     
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        
        message2print = "{(Step 3/3) returning" + "; load-none-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[NONE[0]] = load_none
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-false-of-Unpickler" usage = "loads false">
    # --------------------------------------------------------------------------------------------- 
    def load_false(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3) 
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-false'
     
        message2print = "{(Step 1/3) started" + "; load-false-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
                         
                                
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        try:        
            self.append(False)
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-false-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
    
            nack2append = False
            ack2append = True
            
            message2print = "{acknowledged" + "; ack2append-acknowledged-by-load-false-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-false-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
          
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3 returning" + "; load-false-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[NEWFALSE[0]] = load_false
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-true-of-Unpickler" usage = "loads true">
    # --------------------------------------------------------------------------------------------- 
    def load_true(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-true'
     
        message2print = "{(Step 1/3) started" + "; load-true-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        try:        
            self.append(True)
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-true-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
 
            nack2append = False
            ack2append = True
            
            message2print = "{acknowledged" + "; ack2append-acknowledged-by-load-true-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
        except:
            message2print = "{(Step 2/3b) yielded" + "; nack2append-yielded-by-load-true-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
              
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[NEWTRUE[0]] = load_true
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-int-of-Unpickler" usage = "loads int">
    # --------------------------------------------------------------------------------------------- 
    def load_int(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-int'
     
        message2print = "{(Step 1/4) started" + "; load-int-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2data = False
        nack2data = True 
        
        try:         
            data = self.readline()
            
            message2print = "{(Step 2/4a) set" + "; data-set-by-load-int-of-Unpickler to: " + str(data)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    

            nack2data = False
            ack2data = True    
            
            message2print = "{acknowledged" + "; ack2data-acknowledged-by-load-int-of-Unpickler: " + str(ack2data) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                  
        except:
            message2print = "{(Step 2/4b) yielded" + "; nack2data-yielded-by-load-int-of-Unpickler: " + str(nack2data)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2val = False
        nack2val = True
        
        if ack2data is True: 
            try:    
                if data == FALSE[1:]:
                    val = False
                    
                    message2print = "{(Step 3/4a1) set" + "; val-set-by-load-int-of-Unpickler to: " + str(val)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    ack2val = True
                    
                elif data == TRUE[1:]:
                    val = True
                    
                    message2print = "{(Step 3/4a2) set" + "; val-set-by-load-int-of-Unpickler to: " + str(val)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                      
                    ack2val = True
                else:
                    try:
                        val = int(data, 0)
                        
                        message2print = "{(Step 3/4a3) set" + "; val-set-by-load-int-of-Unpickler to: " + str(val)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
  
                        ack2val = True
                    except ValueError:
                        val = int(data, 0)
                        
                        message2print = "{(Step 3/4a4) set" + "; val-set-by-load-int-of-Unpickler to: " + str(val)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                        
                        ack2val = True
            except:
                message2print = "{(Step 3/4b) yielded" + "; nack2val-yielded-by-load-int-of-Unpickler: " + str(nack2val)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                         
        else:
            message2print = "{(Step 3/4c) yielded" + "; nack2val-yielded-by-load-int-of-Unpickler: " + str(nack2val)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        
        if ack2val is True:
            message2print = "{acknowledged" + "; ack2val-acknowledged-by-load-int-of-Unpickler: " + str(ack2val) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
                             
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2val is True:             
            try:
                self.append(val)
                
                message2print = "{(Step 4/4a) performed" + "; append-of-unpickler2xi-performed-by-load-int-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             

                ack2append = True
                
                message2print = "{(Step 4/4b) acknowledged" + "; ack2append-acknowledged-by-load-int-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
                
            except:
                message2print = "{(Step 4/4d) yielded" + "; nack2append-yielded-by-load-int-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
        else:
            message2print = "{(Step 4/4e) yielded" + "; nack2append-yielded-by-load-int-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
        
        message2print = "{(Step 4/4d) returning" + "; load-int-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                                         
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[INT[0]] = load_int
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binint-of-Unpickler" usage = "loads binary int">
    # --------------------------------------------------------------------------------------------- 
    def load_binint(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binint'
     
        message2print = "{(Step 1/3) started" + "; load-binint-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True      
        
        try:
            self.append(mloads(b'i' + self.read(4)))
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-binint-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            
            ack2append = True 
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-binint-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/3c) yielded" + "; ack2append-yielded-by-load-binint-of-Unpickler: " + str(ack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-binint-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BININT[0]] = load_binint
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binint1-of-Unpickler" usage = "loads binary int1">
    # --------------------------------------------------------------------------------------------- 
    def load_binint1(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binint1'
     
        message2print = "{(Step 1/3) started" + "; load-binint1-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        try:         
            self.append(ord(self.read(1)))
       
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-binint1-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            ack2append = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-binint1-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
               
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-binint1-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-binint1-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                      
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BININT1[0]] = load_binint1
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binint2-of-Unpickler" usage = "loads binary int2">
    # --------------------------------------------------------------------------------------------- 
    def load_binint2(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binint2'
     
        message2print = "{(Step 1/3) started" + "; load-binint2-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True 
               
        try:
            self.append(mloads(b'i' + self.read(2) + b'\000\000'))
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-binint2-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            
            ack2append = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-binint2-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
                   
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-binint2-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
             
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BININT2[0]] = load_binint2
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-long-of-Unpickler" usage = "loads long">
    # --------------------------------------------------------------------------------------------- 
    def load_long(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-long'
     
        message2print = "{(Step 1/4) started" + "; load-long-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2val = False
        nack2val = True
        try:       
            val = self.readline()[:-1].decode("ascii")
            
            ack2val = True 
            
            message2print = "{acknowledged" + "; ack2val-acknowledged-by-load-long-of-Unpickler: " + str(ack2val) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
                  
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2val-yielded-by-load-long-of-Unpickler: " + str(nack2val)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2val is True:      
            try:
                self.append(int(val, 0))
                
                ack2append = True
                
                message2print = "{acknowledged" + "; ack2append-acknowledged-by-load-long-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
                
            except:
                message2print = "{(Step 3/4b) yielded" + "; ack2append-yielded-by-load-long-of-Unpickler: " + str(ack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/4c) yielded" + "; nack2append-yielded-by-load-long-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                       
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-long-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[LONG[0]] = load_long
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
      
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-long1-of-Unpickler" usage = "loads long1">
    # --------------------------------------------------------------------------------------------- 
    def load_long1(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/5)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-long1'
     
        message2print = "{(Step 1/5) started" + "; load-long1-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/5)
        # -----------------------------------------------------------------------------------------           
        ack2n = False
        nack2n = True 
                 
        try:
            n = ord(self.read(1))
            
            ack2n = True 
            
            message2print = "{(Step 2/5a) acknowledged" + "; ack2n-acknowledged-by-load-long1-of-Unpickler: " + str(ack2n) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
                  
        except:
            message2print = "{(Step 2/5b) yielded" + "; nack2n-yielded-by-load-long1-of-Unpickler: " + str(nack2n)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/5)
        # -----------------------------------------------------------------------------------------           
        ack2data = False
        if ack2n is True:         
            try:
                data = self.read(n)
                
                ack2data = True
                
                message2print = "{(Step 3/5a) acknowledged" + "; ack2data-acknowledged-by-load-long1-of-Unpickler: " + str(ack2data) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                       
            except:
                message2print = "{(Step 3/5b) yielded" + "; ack2data-yielded-by-load-long1-of-Unpickler: " + str(ack2data)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/5c) yielded" + "; ack2data-yielded-by-load-long1-of-Unpickler: " + str(ack2data)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/5)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2data is True:         
            try:
                self.append(decode_long(data))
               
                ack2append = True
                
                message2print = "{(Step 4/5a) acknowledged" + "; ack2append-acknowledged-by-load-long1-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                  
                
                
            except:
                message2print = "{(Step 4/5b) yielded" + "; nack2append-yielded-by-load-long1-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/5c) yielded" + "; nack2append-yielded-by-load-long1-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/5)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 5/5) returning" + "; load-long1-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>
    
    # --------------------------------------------------------------------------------------------- 
    #<define4xi definition = "
    dispatch[LONG1[0]] = load_long1
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-long4-of-Unpickler" usage = "loads long2">
    # --------------------------------------------------------------------------------------------- 
    def load_long4(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/5)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-long4'
     
        message2print = "{(Step 1/5) started" + "; load-long4-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/5)
        # -----------------------------------------------------------------------------------------           
        ack2n = False
        nack2n = True
        try:      
            n = mloads(b'i' + self.read(4))
            
            ack2n = True
            
            message2print = "{(Step 2/5a) acknowledged" + "; ack2n-acknowledged-by-load-long4-of-Unpickler: " + str(ack2n) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
        except:
            message2print = "{(Step 2/5b) yielded" + "; nack2n-yielded-by-load-long4-of-Unpickler: " + str(nack2n)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
           
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/5)
        # -----------------------------------------------------------------------------------------           
        ack2data = False
        nack2data = True
        
        if ack2n is True:      
            try:
                data = self.read(n)
                
                ack2data = True 
                
                message2print = "{(Step 3/5a) acknowledged" + "; ack2data-acknowledged-by-load-long4-of-Unpickler: " + str(ack2data) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                       
            except:
                message2print = "{(Step 3/5b) yielded" + "; nack2data-yielded-by-load-long4-of-Unpickler: " + str(nack2data)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
        else:
            message2print = "{(Step 3/5c) yielded" + "; nack2data-yielded-by-load-long4-of-Unpickler: " + str(nack2data)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/5)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2data is True:      
            try:
                self.append(decode_long(data))
                
                ack2append = True 
                
                message2print = "{(Step 4/5a) acknowledged" + "; ack2append-acknowledged-by-load-long4-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                       
            except:
                message2print = "{(Step 4/5b) yielded" + "; ack2append-yielded-by-load-long4-of-Unpickler: " + str(ack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
        else:
            message2print = "{(Step 4/5c) yielded" + "; nack2append-yielded-by-load-long4-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                           
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/5)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 5/5) returning" + "; load-long4-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[LONG4[0]] = load_long4
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-float-of-Unpickler" usage = "loads float">
    # --------------------------------------------------------------------------------------------- 
    def load_float(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-float'
     
        message2print = "{(Step 1/X) started" + "; load-float-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True 
        
        try:          
            self.append(float(self.readline()[:-1]))
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-float-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            ack2append = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-float-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                   
            
            
        
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-float-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-float-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[FLOAT[0]] = load_float
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binfloat-of-Unpickler" usage = "loads binary float">
    # --------------------------------------------------------------------------------------------- 
    def load_binfloat(self, unpack=struct.unpack):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binfloat'
     
        message2print = "{(Step 1/3) started" + "; load-binfloat-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        try:    
            self.append(unpack('>d', self.read(8))[0])
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-binfloat-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            ack2append = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-binfloat-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-binfloat-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-binfloat-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                 
         
                       
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BINFLOAT[0]] = load_binfloat
    #"
    #/>    
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-string-of-Unpickler" usage = "loads string">
    # --------------------------------------------------------------------------------------------- 
    def load_string(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-string'
     
        message2print = "{(Step 1/6) started" + "; load-string-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------           
        ack2orig = False
        nack2orig = True        
        try:
            orig = self.readline()
            
            message2print = "{(Step 2/6a) set" + "; orig-set-by-load-string-of-Unpickler to: " + str(orig)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                        
            ack2orig = True
            
            message2print = "{(Step 2/6b) acknowledged" + "; ack2orig-acknowledged-by-load-string-of-Unpickler: " + str(ack2orig) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
        except:
            message2print = "{(Step 2/6b) yielded" + "; nack2orig-yielded-by-load-string-of-Unpickler: " + str(nack2orig)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------           
        ack2rep = False
        nack2rep = True
        
        if ack2orig is True:      
            try:
                rep = orig[:-1]
                
                message2print = "{(Step 3/6a) set" + "; rep-set-by-load-string-of-Unpickler to: " + str(rep)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                    
                ack2rep = True
                
                message2print = "{(Step 3/6b) acknowledged" + "; ack2rep-acknowledged-by-load-string-of-Unpickler: " + str(ack2rep) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
                 
            except:
                message2print = "{(Step 3/6c) yielded" + "; nack2rep-yielded-by-load-string-of-Unpickler: " + str(nack2rep)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/6d) yielded" + "; nack2rep-yielded-by-load-string-of-Unpickler: " + str(nack2rep)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                 
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------           
        ack2q = False
        nack2q = True
        if ack2rep is True:
            try:      
                for q in (b'"', b"'"): # double or single quote
                    if rep.startswith(q):
                        if not rep.endswith(q):
                            message2print = "{(Step 4/6a) yielded" + "; ack2q-yielded-by-load-string-of-Unpickler: " + str(ack2q)  + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                        rep = rep[len(q):-len(q)]
                        
                        message2print = "{(Step 4/6b) set" + "; rep-set-by-load-string-of-Unpickler to: " + str(rep)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                        
                        
                        break
                        
                    ack2q = True
                    
                    message2print = "{acknowledged" + "; ack2q-acknowledged-by-load-string-of-Unpickler: " + str() + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                 
                    
                        
                else:
                    message2print = "{(Step 4/6c) yielded" + "; nack2q-yielded-by-load-string-of-Unpickler: " + str(nack2q)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            except:
                message2print = "{(Step 4/6d) yielded" + "; nack2q-yielded-by-load-string-of-Unpickler: " + str(nack2q)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/6e) yielded" + "; nack2q-yielded-by-load-string-of-Unpickler: " + str(nack2q)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                   
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2rep is True:
         
            try:          
                self.append(codecs.escape_decode(rep)[0]
                            .decode(self.encoding, self.errors))
                
                message2print = "{(Step 5/6a) performed" + "; append-of-unpickler2xi-performed-by-load-string-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                            
                ack2append = True
                
                message2print = "{acknowledged" + "; ack2append-acknowledged-by-load-string-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
                
            except:
                message2print = "{(Step 5/6b) yielded" + "; nack2append-yielded-by-load-string-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                     
        else:
            message2print = "{(Step 5/6c) yielded" + "; nack2append-yielded-by-load-string-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------           
                        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[STRING[0]] = load_string
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binstring-of-Unpickler" usage = "loads binstring">
    # --------------------------------------------------------------------------------------------- 
    def load_binstring(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binstring'
     
        message2print = "{(Step 1/6) started" + "; load-binstring-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------           
        ack2len = False
        nack2len = True
        try:      
            len = mloads(b'i' + self.read(4))
            
            ack2len = True
            
            message2print = "{(Step 2/6a) acknowledged" + "; ack2len-acknowledged-by-load-binstring-of-Unpickler: " + str(ack2len) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/6b) yielded" + "; nack2len-yielded-by-load-binstring-of-Unpickler: " + str(nack2len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------           
        ack2data = False
        nack2data = True 
        
        if ack2len is True:
              
            try:
                data = self.read(len)
                
                ack2data = True 
                
                message2print = "{(Step 3/6a) acknowledged" + "; ack2data-acknowledged-by-load-binstring-of-Unpickler: " + str(ack2data) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 3/6b) yielded" + "; nack2data-yielded-by-load-binstring-of-Unpickler: " + str(nack2data)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/6c) yielded" + "; nack2data-yielded-by-load-binstring-of-Unpickler: " + str(nack2data)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------           
        ack2value = False
        nack2value = True
        
        if ack2data is True:      
            try:
                value = str(data, self.encoding, self.errors)
                
                ack2value = True
                
                message2print = "{(Step 4/6a) acknowledged" + "; ack2value-acknowledged-by-load-binstring-of-Unpickler: " + str(ack2value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 4/6b) yielded" + "; nack2value-yielded-by-load-binstring-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
           message2print = "{(Step 4/6c) yielded" + "; nack2value-yielded-by-load-binstring-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
           print4xi(class2print, method2print, message2print)                         
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True 
        
        if ack2value is True:      
            try:
                self.append(value)
                
                ack2append = True
                
                message2print = "{(Step 5/6a) acknowledged" + "; ack2append-acknowledged-by-load-binstring-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                        
            except:
                message2print = "{(Step 5/6b) yielded" + "; nack2append-yielded-by-load-binstring-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 5/6c) yielded" + "; nack2append-yielded-by-load-binstring-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                         
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 6/6) returning" + "; load-binstring-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BINSTRING[0]] = load_binstring
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binbytes-of-Unpickler" usage = "loads binbytes">
    # --------------------------------------------------------------------------------------------- 
    def load_binbytes(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binbytes'
     
        message2print = "{(Step 1/X) started" + "; load-binbytes-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2len = False
        nack2len = True
        
        try:      
            len = mloads(b'i' + self.read(4))
            
            ack2len = True
            
            message2print = "{(Step 2/4a) acknowledged" + "; ack2len-acknowledged-by-load-binbytes-of-Unpickler: " + str(ack2len) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/4b) yielded" + "; nack2len-yielded-by-load-binbytes-of-Unpickler: " + str(nack2len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                 
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2len is True:
            try:        
                self.append(self.read(len))
                
                ack2append = True
                
                message2print = "{(Step 3/4a) acknowledged" + "; ack2append-acknowledged-by-load-binbytes-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 3/4b) yielded" + "; nack2append-yielded-by-load-binbytes-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/4c) yielded" + "; nack2append-yielded-by-load-binbytes-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                    
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-binbytes-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BINBYTES[0]] = load_binbytes
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-unicode-of-Unpickler" usage = "loads unicode">
    # --------------------------------------------------------------------------------------------- 
    def load_unicode(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-unicode'
     
        message2print = "{(Step 1/3) started" + "; load-unicode-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        try:         
            self.append(str(self.readline()[:-1], 'raw-unicode-escape'))
       
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-unicode-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            ack2append = True            
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-unicode-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-unicode-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-unicode-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[UNICODE[0]] = load_unicode
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binunicode-of-Unpickler" usage = "loads binunicode">
    # --------------------------------------------------------------------------------------------- 
    def load_binunicode(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binunicode'
     
        message2print = "{(Step 1/4) started" + "; load-binunicode-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2len = False
        nack2len = True  
              
        try:
            len = mloads(b'i' + self.read(4))
            
            message2print = "{(Step 2/4a) set" + "; length-set-by-load-binunicode-of-Unpickler to: " + str(len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
    
            ack2len = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2len-acknowledged-by-load-binunicode-of-Unpickler: " + str(ack2len) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2len-yielded-by-load-binunicode-of-Unpickler: " + str(nack2len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
               
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2len is True:        
            try:
                self.append(str(self.read(len), 'utf-8'))
                
                message2print = "{(Step 3/4a) performed" + "; append-of-unpickler2xi-performed-by-load-binunicode-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                
                ack2append = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2append-acknowledged-by-load-binunicode-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
                
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2append-yielded-by-load-binunicode-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2append-yielded-by-load-binunicode-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-binunicode-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BINUNICODE[0]] = load_binunicode
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-short-binstring-of-Unpickler" usage = "loads short binstring">
    # --------------------------------------------------------------------------------------------- 
    def load_short_binstring(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-short-binstring'
     
        message2print = "{(Step 1/6) started" + "; load-short-binstring-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------           
        ack2len = False
        nack2len = True
        
        try:      
            len = ord(self.read(1))
            
            message2print = "{(Step 2/6a) set" + "; len-set-by-load-short-binstring-of-Unpickler to: " + str(len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
    
            ack2len = True
            
            message2print = "{(Step 2/6b) acknowledged" + "; ack2len-acknowledged-by-load-short-binstring-of-Unpickler: " + str(ack2len) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/6c) yielded" + "; nack2len-yielded-by-load-short-binstring-of-Unpickler: " + str(nack2len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------           
        ack2data = False
        nack2data = True
        
        if ack2len is True:
                 
            try:
                data = bytes(self.read(len))
            
                message2print = "{(Step 3/6a) set" + "; data-set-by-load-short-binstring-of-Unpickler to: " + str(data)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                    
                ack2data = True
                
                message2print = "{(Step 3/6b) acknowledged" + "; ack2data-acknowledged-by-load-short-binstring-of-Unpickler: " + str(ack2data) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
                
            except:
                message2print = "{(Step 3/6c) yielded" + "; nack2data-yielded-by-load-short-binstring-of-Unpickler: " + str(nack2data)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)          
        else:
            message2print = "{(Step 3/6d) yielded" + "; nack2data-yielded-by-load-short-binstring-of-Unpickler: " + str(nack2data)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------           
        ack2value = False
        nack2value = True
        if ack2data is True:         
            try:
                value = str(data, self.encoding, self.errors)
            
                message2print = "{(Step 4/6a) set" + "; value-set-by-load-short-binstring-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                    
                ack2value = True
                
                message2print = "{(Step 4/6b) acknowledged" + "; ack2value-acknowledged-by-load-short-binstring-of-Unpickler: " + str(ack2value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
                
            except:
                message2print = "{(Step 4/6c) yielded" + "; nack2value-yielded-by-load-short-binstring-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
        else:
            message2print = "{(Step 4/6d) yielded" + "; nack2value-yielded-by-load-short-binstring-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                      
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True 
        if ack2value is True:
                 
            try:
                self.append(value)
                
                message2print = "{(Step 5/6a) performed" + "; append-of-unpickler2xi-performed-by-load-short-binstring-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                
                ack2appendack2append= True 
                
                message2print = "{(Step 5/6b) acknowledged" + "; ack2appendack2append-acknowledged-by-load-short-binstring-of-Unpickler: " + str(ack2appendack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 5/6c) yielded" + "; nack2append-yielded-by-load-short-binstring-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
        else:
            message2print = "{(Step 5/6d) yielded" + "; nack2append-yielded-by-load-short-binstring-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                      
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 6/6) returning" + "; load-short-binstring-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[SHORT_BINSTRING[0]] = load_short_binstring
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-short-binbytes-of-Unpickler" usage = "loads short binbytes">
    # --------------------------------------------------------------------------------------------- 
    def load_short_binbytes(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-short-binbytes'
     
        message2print = "{(Step 1/4) started" + "; load-short-binbytes-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2len = False
        nack2len = True
        try:
              
            len = ord(self.read(1))
            
            message2print = "{(Step 2/4a) set" + "; len-set-by-load-short-binbytes-of-Unpickler to: " + str(len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    

            ack2len = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2len-acknowledged-by-load-short-binbytes-of-Unpickler: " + str(ack2len) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
            
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2len-yielded-by-load-short-binbytes-of-Unpickler: " + str(nack2len)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        if ack2len is True:
            try:        
                self.append(bytes(self.read(len)))
        
                message2print = "{(Step 3/4a) performed" + "; append-of-unpickler2xi-performed-by-load-short-binbytes-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                
                ack2append = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2append-acknowledged-by-load-short-binbytes-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 3/4b) yielded" + "; nack2append-yielded-by-load-short-binbytes-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 3/4c) yielded" + "; nack2append-yielded-by-load-short-binbytes-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                              
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-short-binbytes-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
        
    #</methodize4xi>
            

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[SHORT_BINBYTES[0]] = load_short_binbytes
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-tuple-of-Unpickler" usage = "loads tuple">
    # --------------------------------------------------------------------------------------------- 
    def load_tuple(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-tuple'
     
        message2print = "{(Step 1/4) started" + "; load-tuple-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2k = False
        nack2k = True
        try:     
            k = self.marker()
            
            message2print = "{(Step 2/4a) set" + "; k-set-by-load-tuple-of-Unpickler to: " + str(k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    

            ack2k = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2k-acknowledged-by-load-tuple-of-Unpickler: " + str(ack2k) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2k-yielded-by-load-tuple-of-Unpickler: " + str(nack2k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        if ack2k is True: 
            try:         
                self.stack[k:] = [tuple(self.stack[k+1:])]
                
                message2print = "{(Step 3/4a) set" + "; stack-k-set-by-load-tuple-of-Unpickler to: " + str(self.stack[k:])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
  
                ack2stack = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2stack-acknowledged-by-load-tuple-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 3/4b) yielded" + "; nack2stack-yielded-by-load-tuple-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/4c) yielded" + "; nack2stack-yielded-by-load-tuple-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                          
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-tuple-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                
            
    #</methodize4xi>
            

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[TUPLE[0]] = load_tuple
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-empty-tuple-of-Unpickler" usage = "loads empty tuple">
    # --------------------------------------------------------------------------------------------- 
    def load_empty_tuple(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-empty-tuple'
     
        message2print = "{(Step 1/3) started" + "; load-empty-tuple-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        try:         
            self.append(())
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-empty-tuple-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            
            ack2append = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-empty-tuple-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-empty-tuple-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-empty-tuple-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[EMPTY_TUPLE[0]] = load_empty_tuple
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-tuple1-of-Unpickler" usage = "loads tuple1">
    # --------------------------------------------------------------------------------------------- 
    def load_tuple1(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-tuple1'
     
        message2print = "{(Step 1/X) started" + "; load-tuple1-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        try:
        
            message2print = "{(Step 2/3a) got" + "; stack-got-by-load-tuple1-of-Unpickler: " + str((self.stack[-1],))  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                 
            self.stack[-1] = (self.stack[-1],)
        
            ack2stack = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2stack-acknowledged-by-load-tuple1-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                  
            
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2stack-yielded-by-load-tuple1-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-tuple1-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[TUPLE1[0]] = load_tuple1
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-tuple2-of-Unpickler" usage = "loads tuple2">
    # --------------------------------------------------------------------------------------------- 
    def load_tuple2(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-tuple2'
     
        message2print = "{(Step 1/3) started" + "; load-tuple2-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False   
        nack2stack = True
        try:
            message2print = "{(Step 2/3a) got" + "; stack-2-got-by-load-tuple2-of-Unpickler: " + str(self.stack[-2])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            message2print = "{(Step 2/3b) got" + "; stack-1-got-by-load-tuple2-of-Unpickler: " + str(self.stack[-1])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
            self.stack[-2:] = [(self.stack[-2], self.stack[-1])]
            
            message2print = "{(Step 2/3c) set" + "; stack-2-set-by-load-tuple2-of-Unpickler to: " + str(self.stack[-2:])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2stack = True
            
            message2print = "{(Step 2/3d) acknowledged" + "; ack2stack-acknowledged-by-load-tuple2-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        
        except:
            message2print = "{(Step 2/3e) yielded" + "; nack2stack-yielded-by-load-tuple2-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
           
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
            
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[TUPLE2[0]] = load_tuple2
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-tuple3-of-Unpickler" usage = "loads tuple3">
    # --------------------------------------------------------------------------------------------- 
    def load_tuple3(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-tuple3'
     
        message2print = "{(Step 1/3) started" + "; load-tuple3-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        try: 
            message2print = "{(Step 2/3a) got" + "; stack-3-got-by-load-tuple3-of-Unpickler: " + str(self.stack[-3])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
            message2print = "{(Step 2/3b) got" + "; stack-2-got-by-load-tuple3-of-Unpickler: " + str(self.stack[-2])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            message2print = "{(Step 2/3c) got" + "; stack-1-got-by-load-tuple3-of-Unpickler: " + str(self.stack[-1])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                      
            self.stack[-3:] = [(self.stack[-3], self.stack[-2], self.stack[-1])]
        
            message2print = "{(Step 2/3d) set" + "; stack-set-by-load-tuple3-of-Unpickler to: " + str(self.stack[-3:])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2stack = True
            
            message2print = "{(Step 2/3e) acknowledged" + "; ack2stack-acknowledged-by-load-tuple3-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
        except:
            message2print = "{(Step 2/3f) yielded" + "; nack2stack-yielded-by-load-tuple3-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
             
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[TUPLE3[0]] = load_tuple3
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-empty-list-of-Unpickler" usage = "loads empty list">
    # --------------------------------------------------------------------------------------------- 
    def load_empty_list(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-empty-list'
     
        message2print = "{(Step 1/3) started" + "; load-empty-list-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        try:        
            self.append([])
        
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-empty-list-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            ack2append = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-empty-list-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                         
                         
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-empty-list-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-empty-list-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                
            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[EMPTY_LIST[0]] = load_empty_list
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
         

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-empty-dictionary-of-Unpickler" usage = "loads empty dictionary">
    # --------------------------------------------------------------------------------------------- 
    def load_empty_dictionary(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-empty-dictionary'
     
        message2print = "{(Step 1/3) started" + "; load-empty-dictionary-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        try:         
            self.append({})
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-empty-dictionary: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            ack2append = True
            
            message2print = "{(Step 2/3a) acknowledged" + "; ack2append-acknowledged-by-load-empty-dictionary: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                       
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-empty-dictionary-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-empty-dictionary-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[EMPTY_DICT[0]] = load_empty_dictionary
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-list-of-Unpickler" usage = "loads list">
    # --------------------------------------------------------------------------------------------- 
    def load_list(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-list'
     
        message2print = "{(Step 1/4) started" + "; load-list-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2k = False
        nack2k = True
        try:     
            k = self.marker()
        
            message2print = "{(Step 2/4a) set" + "; k-set-by-load-list-of-Unpickler to: " + str(k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            
            ack2k = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2k-acknowledged-by-load-list-of-Unpickler: " + str(ack2k) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                         
        
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2k-yielded-by-load-list-of-Unpickler: " + str(nack2k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        if ack2k is True:
            try:     
                self.stack[k:] = [self.stack[k+1:]]
            
                message2print = "{(Step 3/4a) set" + "; stack-of-unpickler2xi-set-by-load-list-of-Unpickler to: " + str(self.stack[k:])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
               
                ack2stack = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2stack-acknowledged-by-load-list-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                            
            
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2stack-yielded-by-load-list-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2stack-yielded-by-load-list-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                      
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-list-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                
            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[LIST[0]] = load_list
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-dict-of-Unpickler" usage = "loads dict">
    # --------------------------------------------------------------------------------------------- 
    def load_dict(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-dict'
     
        message2print = "{(Step 1/7) started" + "; load-dict-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------           
        ack2k = False
        nack2k = True
        try:
                               
            k = self.marker()
        
            ack2k = True
            
            message2print = "{(Step 2/7a) acknowledged" + "; ack2k-acknowledged-by-load-dict-of-Unpickler: " + str(ack2k) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                   
        except:    
            
            message2print = "{(Step 2/7b) yielded" + "; nack2k-yielded-by-load-dict-of-Unpickler: " + str(nack2k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------           
        ack2d = False
        nack2d = True
        try:
                             
            d = {}
        
            ack2d = True
            
            message2print = "{(Step 3/7a) acknowledged" + "; ack2d-acknowledged-by-load-dict-of-Unpickler: " + str(ack2d) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                   
        
        except:    
            
            message2print = "{(Step 3/7b) yielded" + "; nack2d-yielded-by-load-dict-of-Unpickler: " + str(nack2d)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
           
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------           
        ack2items = False
        nack2items = True
        if ack2k is True:
            try:
                                 
                items = self.stack[k+1:]
            
                ack2items = True
                
                message2print = "{(Step 4/7a) acknowledged" + "; ack2items-acknowledged-by-load-dict-of-Unpickler: " + str(ack2items) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                

            except:    
                
                message2print = "{(Step 4/7b) yielded" + "; nack2items-yielded-by-load-dict-of-Unpickler: " + str(nack2items)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/7c) yielded" + "; nack2items-yielded-by-load-dict-of-Unpickler: " + str(nack2items)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                           
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------           
        ack2key = False
        nack2key = True
        if ack2items is True:
        
            try:
                             
                for i in range(0, len(items), 2):
                    key = items[i]
                    value = items[i+1]
                    d[key] = value
                
                ack2key = True 
                
                message2print = "{(Step 5/7a) acknowledged" + "; ack2key-acknowledged-by-load-dict-of-Unpickler: " + str(ack2key) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                       
            except:    
                
                message2print = "{(Step 5/7b) yielded" + "; nack2key-yielded-by-load-dict-of-Unpickler: " + str(nack2key)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 5/7c) yielded" + "; nack2key-yielded-by-load-dict-of-Unpickler: " + str(nack2key)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                   
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        if ((ack2k is True) and (ack2d is True)):
            try:
                                 
                self.stack[k:] = [d]
            
                ack2stack = True
                
                message2print = "{(Step 6/7a) acknowledged" + "; ack2stack-acknowledged-by-load-dict-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                  
            except:    
                
                message2print = "{(Step 6/7b) yielded" + "; nack2stack-yielded-by-load-dict-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 6/7c) yielded" + "; nack2stack-yielded-by-load-dict-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                        
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 7/7) returning" + "; load-dict-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                
             
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[DICT[0]] = load_dict
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "instantiate-of-Unpickler" usage = "
    # INST and OBJ differ only in how they get a class object.  It's not
    # only sensible to do the rest in a common routine, the two routines
    # previously diverged and grew different bugs.
    # klass is the class to instantiate, and k points to the topmost mark
    # object, following which are the arguments for klass.__init__.
    #
    #">
    # ---------------------------------------------------------------------------------------------    
    def _instantiate(self, klass, k):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------           

        class2print = 'Unpickler'
        method2print = 'instantiate'
     
        message2print = "{(Step 1/7a) started" + "; instantiate-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 1/7b) passed" + "; klass-passed-to-instantiate-of-Unpickler: " + str(klass) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        message2print = "{(Step 1/7c) passed" + "; k-passed-to-instantiate-of-Unpickler: " + str(k) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
              
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------           
        ack2args = False
        nack2args = True
        try:          
            args = tuple(self.stack[k+1:])
            
            message2print = "{(Step 2/7a) set" + "; args-set-by-instantiate-of-Unpickler to: " + str(args)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
 
            ack2args = True
            
            message2print = "{(Step 2/7b) acknowledged" + "; ack2args-acknowledged-by-instantiate-of-Unpickler: " + str(ack2args) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                            
        
        except:    
            message2print = "{(Step 2/7b) yielded" + "; nack2args-yielded-by-instantiate-of-Unpickler: " + str(nack2args)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
              
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------           
        ack2del = False
        nack2del = True
        try:
        
            message2print = "{(Step 3/7a) deleting" + "; stack-deleting-by-instantiate-of-Unpickler: " + str(self.stack[k:])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                       
            
            del self.stack[k:]
           
            
            ack2del = True
            
            message2print = "{(Step 3/7b) acknowledged" + "; ack2del-acknowledged-by-instantiate-of-Unpickler: " + str(ack2del) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                     
            
        except:    
            message2print = "{(Step 3/7c) yielded" + "; nack2del-yielded-by-instantiate-of-Unpickler: " + str(nack2del)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
          
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------           
        ack2instantiated = False
        nack2instantiated = True           
          
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------           
        if ack2args is True:
           
            if (not args and
                    isinstance(klass, type) and
                    not hasattr(klass, "__getinitargs__")):
                try:
                            
                    value = _EmptyClass()
                    
                    message2print = "{(Step 3/7a1) set" + ";  value-set-by-instantiate-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
      
                    value.__class__ = klass
                    
                    message2print = "{(Step 3/7a2) set" + ";  value-class-set-by-instantiate-of-Unpickler to: " + str(value.__class__)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)      

                    ack2instantiated = True
                    
                    message2print = "{(Step 3/7a3) acknowledged" + "; ack2instantiated-acknowledged-by-instantiate-of-Unpickler: " + str(ack2instantiated) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                    
                except:    
                    message2print = "{(Step 3/7b) yielded" + "; nack2instantiated-yielded-by-instantiate-of-Unpickler: " + str(nack2instantiated)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                      
            else:
  
                message2print = "{(Step 5/7c) yielded" + "; nack2instantiated-yielded-by-instantiate-of-Unpickler: " + str(nack2instantiated)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
                        
                
        else:
  
            message2print = "{(Step 5/7d) yielded" + "; nack2instantiated-yielded-by-instantiate-of-Unpickler: " + str(nack2instantiated)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                    
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------           
        ack2value = False
        nack2value = True       
        if ((not instantiated) or (ack2instantiated is False)):
            try:
                value = klass(*args)
                
                message2print = "{(Step 6/7a) set" + "; value-set-by-instantiate-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    

                ack2value = True
                
                message2print = "{(Step 6/7b) acknowledged" + "; ack2value-acknowledged-by-instantiate-of-Unpickler: " + str(ack2value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                          
                
            except:
                message2print = "{(Step 6/7c) yielded" + "; nack2value-yielded-by-instantiate-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
 
          
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        if ack2value is True:     
            try:
                                           
                self.append(value)
                
                message2print = "{(Step 7/7a) performed" + "; append-of-unpickler2xi-performed-by-instantiate-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                ack2append = True
                
                message2print = "{(Step 7/7b) acknowledged" + "; ack2append-acknowledged-by-instantiate-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 7/7c) yielded" + "; nack2append-yielded-by-instantiate-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
        else:
            message2print = "{(Step 7/7d) yielded" + "; nack2append-yielded-by-instantiate-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                     
        message2print = "{(Step 7/7e) returning" + "; instantiate-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-inst-of-Unpickler" usage = "loads inst">
    # --------------------------------------------------------------------------------------------- 
    def load_inst(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/5)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-inst'
     
        message2print = "{(Step 1/5) started" + "; load-inst-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/5)
        # -----------------------------------------------------------------------------------------           
        ack2module = False
        nack2module = True
        try:      
            module = self.readline()[:-1].decode("ascii")
            
            message2print = "{(Step 2/5a) set" + "; module-set-by-load-inst-of-Unpickler to: " + str(module)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            
            ack2module = True
            
            message2print = "{(Step 2/5b) acknowledged" + "; ack2module-acknowledged-by-load-inst-of-Unpickler: " + str(ack2module) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        except:
            message2print = "{(Step 2/5c) yielded" + "; nack2module-yielded-by-load-inst-of-Unpickler: " + str(nack2module)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
               
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/5)
        # -----------------------------------------------------------------------------------------           
        ack2name = False
        nack2name = True
        try:              
            name = self.readline()[:-1].decode("ascii")
            
            message2print = "{(Step 3/5a) set" + "; name-set-by-load-inst-of-Unpickler to: " + str(name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
            ack2name = True
            
            message2print = "{(Step 3/5b) used" + "; ack2name-used-by-load-inst-of-Unpickler: " + str(ack2name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
            
            
        except:
            message2print = "{(Step 3/5c) yielded" + "; nack2name-yielded-by-load-inst-of-Unpickler: " + str(nack2name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
          
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/5)
        # -----------------------------------------------------------------------------------------           
        ack2klass = False
        nack2klass = True
        if ((ack2module is True) and (ack2name is True)):
            try:              
                klass = self.find_class(module, name)
                
                message2print = "{(Step 4/5a) set" + "; klass-set-by-load-inst-of-Unpickler to: " + str(klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                    
                ack2klass = True
                
                message2print = "{(Step 4/5b) acknowledged" + "; ack2klass-acknowledged-by-load-inst-of-Unpickler: " + str(ack2klass) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 4/5c) yielded" + "; nack2klass-yielded-by-load-inst-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
           message2print = "{(Step 4/5d) yielded" + "; nack2klass-yielded-by-load-inst-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
           print4xi(class2print, method2print, message2print) 
                      
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/5)
        # -----------------------------------------------------------------------------------------           
        ack2instantiate = False
        nack2instantiate = True
        if ack2klass is True: 
            try:
           
                self._instantiate(klass, self.marker())
                
                message2print = "{(Step 5/5a) performed" + "; instantiate-of-unpickler2xi-performed-by-load-inst-of-Unpickler: " + str(self._instantiate) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                ack2instantiate = True
                
                message2print = "{(Step 5/5b) acknowledged" + "; ack2instantiate-acknowledged-by-load-inst-of-Unpickler: " + str(ack2instantiate) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 5/5c) yielded" + "; nack2instantiate-yielded-by-load-inst-of-Unpickler: " + str(nack2instantiate)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                             
        else:
           message2print = "{(Step 5/5d) yielded" + "; nack2instantiate-yielded-by-load-inst-of-Unpickler: " + str(nack2instantiate)  + "; time: " + time4xi() + "}"
           print4xi(class2print, method2print, message2print)             
        
        message2print = "{(Step 5/5d) returning" + "; load-inst-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        

          
                                   
               
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[INST[0]] = load_inst
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-object-of-Unpickler" usage = "loads obj">
    # --------------------------------------------------------------------------------------------- 
    def load_obj(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/X)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-obj'
     
        message2print = "{(Step 1/5) started" + "; load-object-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/5)
        # -----------------------------------------------------------------------------------------           
        ack2k = False
        nack2k = True        
        try:
        
        
            # Stack is ... markobject classobject arg1 arg2 ...
            k = self.marker()
        
            message2print = "{(Step 2/5a) set" + "; k-set-by-load-object-of-Unpickler to: " + str(k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            ack2k = True
            
            message2print = "{(Step 2/5b) acknowledged" + "; ack2k-acknowledged-by-load-object-of-Unpickler: " + str(ack2k) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                      
               
        except:
            message2print = "{(Step 2/5c) yielded" + "; nack2k-yielded-by-load-object-of-Unpickler: " + str(nack2k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/5)
        # -----------------------------------------------------------------------------------------           
        ack2klass = False            
        nack2klass = True
        if ack2k is True:        
            try:
              
                klass = self.stack.pop(k+1)
            
                message2print = "{(Step 3/5a) set" + "; klass-set-by-load-object-of-Unpickler to: " + str(klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2klass = True
                
                message2print = "{(Step 3/5b) acknowledged" + "; ack2klass-acknowledged-by-load-object-of-Unpickler: " + str(ack2klass) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
            except:
                message2print = "{(Step 3/5c) yielded" + "; nack2klass-yielded-by-load-object-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/5d) yielded" + "; nack2klass-yielded-by-load-object-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/5)
        # -----------------------------------------------------------------------------------------           
        ack2instantiate = False
        nack2instantiate = True
        if ((ack2k is True) and (ack2klass is True)):        
            try:
                  
                self._instantiate(klass, k)
                
                ack2instantiate = True
                
                message2print = "{(Step 4/5a) acknowledged" + "; ack2instantiate-acknowledged-by-load-object-of-Unpickler: " + str(ack2instantiate) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
            except:
                message2print = "{(Step 4/5c) yielded" + "; nack2instantiate-yielded-by-load-object-of-Unpickler: " + str(nack2instantiate)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/5d) yielded" + "; nack2instantiate-yielded-by-load-object-of-Unpickler: " + str(nack2instantiate)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/5)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 5/5) returning" + "; load-object-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                
            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[OBJ[0]] = load_obj
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-newobject-of-Unpickler" usage = "loads ">
    # --------------------------------------------------------------------------------------------- 
    def load_newobj(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-newobj'
     
        message2print = "{(Step 1/6) started" + "; load-newobject-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------           
        ack2args = False
        nack2args = True
        try:
                   
            args = self.stack.pop()
        
            message2print = "{(Step 2/6a) set" + "; args-set-by-load-newobject-of-Unpickler to: " + str(args)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            
            ack2args = True
            
            message2print = "{(Step 2/6b) acknowledged" + "; ack2args-acknowledged-by-load-newobject-of-Unpickler: " + str(ack2args) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                  
        except:
            message2print = "{(Step 2/6c) yielded" + "; nack2args-yielded-by-load-newobject-of-Unpickler: " + str(nack2args)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                 
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------           
        ack2cls = False
        nack2cls = True
        try:
                   
            cls = self.stack[-1]
            
            message2print = "{(Step 3/6a) set" + "; class-set-by-load-newobject-of-Unpickler to: " + str(cls)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            
            ack2cls = True
            
            message2print = "{(Step 3/6b) acknowledged" + "; ack2cls-acknowledged-by-load-newobject-of-Unpickler: " + str(ack2cls) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                  
        except:
            message2print = "{(Step 3/6c) yielded" + "; nack2cls-yielded-by-load-newobject-of-Unpickler: " + str(nack2cls)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------           
        ack2obj = False
        nack2obj = True
        if((ack2args is True) and (ack2cls is True)):
            try:
                       
                obj = cls.__new__(cls, *args)
                
                message2print = "{(Step 4/6a) set" + "; object-set-by-load-newobject-of-Unpickler to: " + str(obj)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
                ack2obj = True
                
                message2print = "{(Step 4/6b) acknowledged" + "; ack2obj-acknowledged-by-load-newobject-of-Unpickler: " + str(ack2obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                  
            except:
                message2print = "{(Step 4/6c) yielded" + "; nack2object-yielded-by-load-newobject-of-Unpickler: " + str(nack2obj)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/6d) yielded" + "; nack2object-yielded-by-load-newobject-of-Unpickler: " + str(nack2obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        if ack2obj is True:
        
            try:
                     
                self.stack[-1] = obj
                
                ack2stack = True
                
                message2print = "{(Step 5/6a) acknowledged" + "; ack2stack-acknowledged-by-load-newobject-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                       
            except:
                message2print = "{(Step 5/6b) yielded" + "; nack2stack-yielded-by-load-newobject-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 5/6c) yielded" + "; nack2stack-yielded-by-load-newobject-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 6/6) returning" + "; load-newobject-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                
             
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[NEWOBJ[0]] = load_newobj
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-global-of-Unpickler" usage = "loads global">
    # --------------------------------------------------------------------------------------------- 
    def load_global(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-global'
     
        message2print = "{(Step 1/6) started" + "; load-global-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------           
        ack2module = False
        nack2module = True
        try:      
            module = self.readline()[:-1].decode("utf-8")
        
            message2print = "{(Step 2/6a) set" + "; module-set-by-load-global-of-Unpickler to: " + str(module)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    

            ack2module = True
            
            message2print = "{(Step 2/6b) acknowledged" + "; ack2module-acknowledged-by-load-global-of-Unpickler: " + str(ack2module) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
        except:
            message2print = "{(Step 2/6c) yielded" + "; nack2module-yielded-by-load-global-of-Unpickler: " + str(nack2module)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------           
        ack2name = False     
        nack2name = True
        try:      
            name = self.readline()[:-1].decode("utf-8")
        
            message2print = "{(Step 3/6a) set" + "; name-set-by-load-global-of-Unpickler to: " + str(name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        
            ack2name = True
            
            message2print = "{(Step 3/6b) acknowledged" + "; ack2name-acknowledged-by-load-global-of-Unpickler: " + str(ack2name) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
        except:
            message2print = "{(Step 3/6c) yielded" + "; nack2name-yielded-by-load-global-of-Unpickler: " + str(nack2name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                            
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------           
        ack2klass = False 
        nack2klass = True
        if ((ack2module is True) and (ack2name is True)):
        
            try:               
                klass = self.find_class(module, name)
            
                message2print = "{(Step 4/6a) set" + "; klass-set-by-load-global-of-Unpickler to: " + str(klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                ack2klass = True
                
                message2print = "{(Step 4/6b) acknowledged" + "; ack2klass-acknowledged-by-load-global-of-Unpickler: " + str(ack2klass) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                    
            except:
                message2print = "{(Step 4/6c) yielded" + "; nack2klass-yielded-by-load-global-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/6d) yielded" + "; nack2klass-yielded-by-load-global-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                                    
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------           
        ack2append = False   
        nack2append = True  
        if ack2klass is True:
        
            try:  
                self.append(klass)
                
                message2print = "{(Step 5/6a) performed" + "; append-of-unpickler2xi-performed-by-load-global-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                
                ack2append = True
                
                message2print = "{(Step 5/6b) acknowledged" + "; ack2append-acknowledged-by-load-global-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 5/6c) yielded" + "; nack2append-yielded-by-load-global-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 5/6d) yielded" + "; nack2append-yielded-by-load-global-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                         
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 6/6) returning" + "; load-global-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[GLOBAL[0]] = load_global
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-ext1-of-Unpickler" usage = "loads ext1">
    # --------------------------------------------------------------------------------------------- 
    def load_ext1(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-ext1'
     
        message2print = "{(Step 1/4) started" + "; load-ext1-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2code = False
        nack2code = True
        try:        
            code = ord(self.read(1))
            
            message2print = "{(Step 2/4a) set" + "; code-set-by-load-ext1-of-Unpickler to: " + str(code)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    

  
            ack2code = True
            
            message2print = "{(Step 2/4a) acknowledged" + "; ack2code-acknowledged-by-load-ext1-of-Unpickler: " + str(ack2code) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2code-yielded-by-load-ext1-of-Unpickler: " + str(nack2code)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2extension = False
        nack2extension = True
        if ack2code is True:
            try:
            
                self.get_extension(code)
        
                message2print = "{(Step 3/4a) performed" + "; get-extension-of-unpickler2xi-performed-by-load-ext1-of-Unpickler: " + str(self.get_extension) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2extension = True 
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2extension-acknowledged-by-load-ext1-of-Unpickler: " + str(ack2extension) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2extension-yielded-by-load-ext1-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2extension-yielded-by-load-ext1-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                         
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-ext1-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>
 
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[EXT1[0]] = load_ext1
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-ext2-of-Unpickler" usage = "loads ext2">
    # --------------------------------------------------------------------------------------------- 
    def load_ext2(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-ext2'
     
        message2print = "{(Step 1/4) started" + "; load-ext2-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2code = False
        nack2code = True
        try:       
            code = mloads(b'i' + self.read(2) + b'\000\000')
        
            message2print = "{(Step 2/4a) set" + "; code-set-by-load-ext2-of-Unpickler to: " + str(code)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            
            ack2code = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2code-acknowledged-by-load-ext2-of-Unpickler: " + str(ack2code) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2code-yielded-by-load-ext2-of-Unpickler: " + str(nack2code)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

        
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2extension = False
        nack2extension = True
        if ack2code is True:   
            try:

                self.get_extension(code)
        
                message2print = "{(Step 3/4a) performed" + "; get-extension-performed-by-load-ext2-of-Unpickler: " + str(self.get_extension) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                ack2extension = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2extension-acknowledged-by-load-ext2-of-Unpickler: " + str(ack2extension) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                         
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2extension-yielded-by-load-ext2-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:        
            message2print = "{(Step 3/4d) yielded" + "; nack2extension-yielded-by-load-ext2-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                             
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-ext2-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                   
        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[EXT2[0]] = load_ext2
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-ext4-of-Unpickler" usage = "loads ext4">
    # --------------------------------------------------------------------------------------------- 
    def load_ext4(self):
     
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-ext4'
     
        message2print = "{(Step 1/4) started" + "; load-ext4-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2code = False
        nack2code = True
        try:     
            code = mloads(b'i' + self.read(4))
        
            message2print = "{(Step 2/4a) set" + "; code-set-by-load-ext4-of-Unpickler to: " + str(code)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2code = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2code-acknowledged-by-load-ext4-of-Unpickler: " + str(ack2code) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                   
            
        except:    
            message2print = "{(Step 2/4c) yielded" + "; nack2code-yielded-by-load-ext4-of-Unpickler: " + str(nack2code)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2extension = False
        nack2extension = True
        if ack2code is True:    
        
            try:
            
                self.get_extension(code)
        
                message2print = "{(Step 3/4a) performed" + "; get-extension-of-unpickler2xi-performed-by-load-ext4-of-Unpickler: " + str(self.get_extension) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                ack2extension = True 
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2extension-acknowledged-by-load-ext4-of-Unpickler: " + str(ack2extension) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                  
            except:    
                message2print = "{(Step 3/4c) yielded" + "; nack2extension-yielded-by-load-ext4-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2extension-yielded-by-load-ext4-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-ext4-of-Unpickler-returning: " + time4xi() + "}" 
        pint4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[EXT4[0]] = load_ext4
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-extension-of-Unpickler" usage = "gets extension">
    # --------------------------------------------------------------------------------------------- 
    def get_extension(self, code):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/10)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'get-extension'
     
        message2print = "{(Step 1/10a) started" + "; get-extension-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        message2print = "{(Step 1/10b) passed" + "; code-passed-to-get-extension-of-Unpickler: " + str(code) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/10)
        # -----------------------------------------------------------------------------------------           
              
        nil = []
        
        message2print = "{(Step 2/10) set" + "; nil-set-by-get-extension-of-Unpickler to: " + str(nil)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/10)
        # -----------------------------------------------------------------------------------------           
        ack2obj = False
        nack2obj = True
        try:
            
            obj = _extension_cache.get(code, nil)
            
            message2print = "{(Step 3/10a) set" + "; object-set-by-get-extension-of-Unpickler to: " + str(obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2obj = True
            
            message2print = "{(Step 3/10b) acknowledged" + "; ack2obj-acknowledged-by-get-extension-of-Unpickler: " + str(ack2obj) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
            
        except:    
            message2print = "{(Step 3/10c) yielded" + "; nack2object-yielded-by-get-extension-of-Unpickler: " + str(nack2obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/10)
        # -----------------------------------------------------------------------------------------           
        if ack2obj is True:
            try: 
               
                if obj is not nil:
                    self.append(obj)
                    
                    message2print = "{(Step 4/10a) performed" + "; append-of-unpickler2xi-performed-by-get-extension-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    
                    message2print = "{(Step 4/10b) returning" + "; get-extension-of-Unpickler-returning: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print) 
                            

                    return
                else:
                    message2print = "{(Step 4/10c) yielded" + "; object-yielded-by-get-extension-of-Unpickler: " + str(obj)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                     
                    
            except:    
                message2print = "{(Step 4/10d) yielded" + "; nack2object-yielded-by-get-extension-of-Unpickler: " + str(nack2obj)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                     
        else:
            message2print = "{(Step 4/10e) yielded" + "; nack2object-yielded-by-get-extension-of-Unpickler: " + str(nack2obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                     
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/10)
        # -----------------------------------------------------------------------------------------           
        ack2key = False
        nack2key = True
        try:        
            key = _inverted_registry.get(code)
            
            message2print = "{(Step 5/10a) set" + "; key-set-by-get-extension-of-Unpickler to: " + str(key)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
 
            ack2key = True
            
            message2print = "{(Step 5/10b) acknowledged" + "; ack2key-acknowledged-by-get-extension-of-Unpickler: " + str(ack2key) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        except:
            message2print = "{(Step 5/10c) yielded" + "; nack2key-yielded-by-get-extension-of-Unpickler: " + str(nack2key)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                         
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/10)
        # -----------------------------------------------------------------------------------------           
            
        if ack2key is False:
            message2print = "{(Step 6/10) yielded" + "; ack2key-yielded-by-get-extension-of-Unpickler: " + str(ack2key)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/10)
        # -----------------------------------------------------------------------------------------           
        ack2obj = False
        nack2obj = True
        if ack2key is True: 
        
            try:       
                obj = self.find_class(*key)
        
                message2print = "{(Step 7/10a) set" + "; object-set-by-get-extension-of-Unpickler to: " + str(obj)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2obj = True
                
                message2print = "{(Step 7/10b) acknowledged" + "; ack2obj-acknowledged-by-get-extension-of-Unpickler: " + str(ack2obj) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                          
            except:
                message2print = "{(Step 7/10c) yielded" + "; nack2object-yielded-by-get-extension-of-Unpickler: " + str(nack2obj)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 7/10d) yielded" + "; nack2object-yielded-by-get-extension-of-Unpickler: " + str(nack2obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 8/10)
        # -----------------------------------------------------------------------------------------           
        ack2extension = False
        nack2extension = True
        if ack2obj is True:
        
            try:    
                _extension_cache[code] = obj
                
                message2print = "{(Step 8/10a) performed" + "; extension-cache-performed-by-get-extension-of-Unpickler: " + str(_extension_cache) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2extension = True
                
                message2print = "{(Step 8/10b) acknowledged" + "; ack2extension-acknowledged-by-get-extension-of-Unpickler: " + str(ack2extension) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                 
            except:
                message2print = "{(Step 8/10c) yielded" + "; nack2extension-yielded-by-get-extension-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 8/10d) yielded" + "; nack2extension-yielded-by-get-extension-of-Unpickler: " + str(nack2extension)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 9/10)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        if ack2obj is True:
        
            try:               
                self.append(obj)
                
                message2print = "{(Step 9/10a) performed" + "; append-of-unpickler2xi-performed-by-get-extension-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2append = True
                
                message2print = "{(Step 9/10b) acknowledged" + "; ack2append-acknowledged-by-get-extension-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                   
            except:
                message2print = "{(Step 9/10c) yielded" + "; nack2append-yielded-by-get-extension-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 9/10d) yielded" + "; nack2append-yielded-by-get-extension-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                           
        # ----------------------------------------------------------------------------------------- 
        # (Step 10/10)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 10/10) returning" + "; get-extension-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "find-class-of-Unpickler" usage = "finds class">
    # --------------------------------------------------------------------------------------------- 
    def find_class(self, module, name):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/5)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'find-class'
     
        message2print = "{(Step 1/5a) started" + "; find-class-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        message2print = "{(Step 1/5b) passed" + "; module-passed-to-find-class-of-Unpickler: " + str(module) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        message2print = "{(Step 1/5c) passed" + "; name-passed-to-find-class-of-Unpickler: " + str(name) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
           
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/5)
        # -----------------------------------------------------------------------------------------           
        
        ack2import = False
        nack2import = True      
        try:
            # Subclasses may override this
            __import__(module, level=0)
            
            message2print = "{(Step 2/5a) performed" + "; import-performed-by-find-class-of-Unpickler: " + str(__import__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            ack2import = True
            
            message2print = "{(Step 2/5b) acknowledged" + "; ack2import-acknowledged-by-find-class-of-Unpickler: " + str(ack2import) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                 
        except:
            message2print = "{(Step 2/5c) yielded" + "; nack2import-yielded-by-find-class-of-Unpickler: " + str(nack2import)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/5)
        # -----------------------------------------------------------------------------------------           
        ack2mod = False
        nack2mod = True
        if ack2import is True:    
        
            try:
                mod = sys.modules[module]
                
                message2print = "{(Step 3/5a) set" + "; module-set-by-find-class-of-Unpickler to: " + str(mod)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                ack2mod = True
                
                message2print = "{(Step 3/5b) acknowledged" + "; ack2mod-acknowledged-by-find-class-of-Unpickler: " + str(ack2mod) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
                
            except:
                message2print = "{(Step 3/5c) yielded" + "; nack2mod-yielded-by-find-class-of-Unpickler: " + str(nack2mod)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
        else:
            message2print = "{(Step 3/5d) yielded" + "; nack2mod-yielded-by-find-class-of-Unpickler: " + str(nack2mod)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/5)
        # -----------------------------------------------------------------------------------------           
        ack2klass = False
        nack2klass = True
        if ack2mod is True:
        
            try:
            
                klass = getattr(mod, name)
                
                message2print = "{(Step 4/5a) set" + "; klass-set-by-find-class-of-Unpickler to: " + str(klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                ack2klass = True
                
                message2print = "{(Step 4/5b) acknowledged" + "; ack2klass-acknowledged-by-find-class-of-Unpickler: " + str(ack2klass) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 3/5c) yielded" + "; nack2klass-yielded-by-find-class-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
        else:
            message2print = "{(Step 3/5d) yielded" + "; nack2klass-yielded-by-find-class-of-Unpickler: " + str(nack2klass)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                
                 
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/5)
        # -----------------------------------------------------------------------------------------           
        if ack2klass is True:
            message2print = "{(Step 5/5a) returning" + "; klass-returning-by-find-class-of-Unpickler: " + str(klass) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                    
            return klass
        else:
            message2print = "{(Step 5/5b) returning" + "; None-returning-by-find-class-of-Unpickler: " + str(None) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            return None
            
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-reduce-of-Unpickler" usage = "loads reduce">
    # --------------------------------------------------------------------------------------------- 
    def load_reduce(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-reduce'
     
        message2print = "{(Step 1/7) started" + "; load-reduce-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        try:        
            stack = self.stack
       
            message2print = "{(Step 2/7a) set" + "; stack-set-by-load-reduce-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
 
            ack2stack = True
            
            message2print = "{(Step 2/7b) acknowledged" + "; ack2stack-acknowledged-by-load-reduce-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                 
        except:
            message2print = "{(Step 2/7c) yielded" + "; nack2stack-yielded-by-load-reduce-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------           
        ack2args = False
        nack2args = True
        if ack2stack is True:     
        
            try:    
        
                args = stack.pop()
       
                message2print = "{(Step 3/7a) set" + "; args-set-by-load-reduce-of-Unpickler to: " + str(args)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    

                ack2args = True
                
                message2print = "{(Step 3/7b) acknowledged" + "; ack2args-acknowledged-by-load-reduce-of-Unpickler: " + str(ack2args) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                  
            except:
                message2print = "{(Step 3/7c) yielded" + "; nack2args-yielded-by-load-reduce-of-Unpickler: " + str(nack2args)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 3/7d) yielded" + "; nack2args-yielded-by-load-reduce-of-Unpickler: " + str(nack2args)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------           
        ack2func = False
        nack2func = True
        if ack2stack is True:     
        
            try:    
        
                func = stack[-1]
       
                message2print = "{(Step 4/7a) set" + "; function-set-by-load-reduce-of-Unpickler to: " + str(func)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                ack2func = True
                
                message2print = "{(Step 4/7b) acknowledged" + "; ack2func-acknowledged-by-load-reduce-of-Unpickler: " + str(ack2func) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
            except:
                message2print = "{(Step 4/7c) yielded" + "; nack2func-yielded-by-load-reduce-of-Unpickler: " + str(nack2func)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 4/7d) yielded" + "; nack2func-yielded-by-load-reduce-of-Unpickler: " + str(nack2func)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------           
        ack2value = False
        nack2value = True
        if ((ack2func is True) and (ack2args is True)):
             
            try:
                value = func(*args)
                
                message2print = "{(Step 5/7a) set" + "; value-set-by-load-reduce-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                
                ack2value = True
                
                message2print = "{(Step 5/7b) acknowledged" + "; ack2value-acknowledged-by-load-reduce-of-Unpickler: " + str(ack2value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                        
            except:
                message2print = "{(Step 5/7c) yielded" + "; nack2value-yielded-by-load-reduce-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 5/7d) yielded" + "; nack2value-yielded-by-load-reduce-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  
                       
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------           
        if ((ack2stack is True) and (ack2value is True)):
             
            try:
                     
                stack[-1] = value #to-be-solved - why not self.stack - this is also in original one !?
        
                message2print = "{(Step 6/7a) set" + "; stack-set-by-load-reduce-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                
                
            except:
                message2print = "{(Step 6/7b) yielded" + "; nack2stack-yielded-by-load-reduce-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 6/7c) yielded" + "; nack2stack-yielded-by-load-reduce-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  
                                       
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------           
             
    #</methodize4xi>
  
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[REDUCE[0]] = load_reduce
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-pop-of-Unpickler" usage = "loads pop">
    # --------------------------------------------------------------------------------------------- 
    def load_pop(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-pop'
     
        message2print = "{(Step 1/3) started" + "; load-pop-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        try:     
            message2print = "{(Step 2/3a) deleting" + "; stack-deleting-by-load-pop-of-Unpickler: " + str(self.stack[-1])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            del self.stack[-1]
            
            message2print = "{(Step 2/3b) set" + "; stack-of-unpickler2xi-set-by-load-pop-of-Unpickler to: " + str(self.stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            ack2stack = True
            
            message2print = "{(Step 2/3c) acknowledged" + "; ack2stack-acknowledged-by-load-pop-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                     
        except:
            message2print = "{(Step 2/3d) yielded" + "; nack2stack-yielded-by-load-pop-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
                                     
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-pop-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                      
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[POP[0]] = load_pop
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-pop-mark-of-Unpickler" usage = "loads pop mark">
    # --------------------------------------------------------------------------------------------- 
    def load_pop_mark(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-pop-mark'
     
        message2print = "{(Step 1/4) started" + "; load-pop-mark-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2k = False
        nack2k = True
        try:         
            k = self.marker()
        
            message2print = "{(Step 2/4a) set" + "; k-set-by-load-pop-mark-of-Unpickler to: " + str(k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            ack2k = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; -acknowledged-by-load-pop-mark-of-Unpickler: " + str(ack2k) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                   
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2k-yielded-by-load-pop-mark-of-Unpickler: " + str(nack2k)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2del = False
        nack2del = True
        if ack2k is True:     
        
            try:
                message2print = "{(Step 3/4a) deleting" + "; stack-deleting-by-load-pop-mark-of-Unpickler: " + str(self.stack[k:])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                
            
                del self.stack[k:]
                
                message2print = "{(Step 3/4b) set" + "; stack-of-unpickler2xi-set-by-load-pop-mark-of-Unpickler to: " + str(self.stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                ack2del = True
                
                message2print = "{(Step 3/4c) acknowledged" + "; ack2del-acknowledged-by-load-pop-mark-of-Unpickler: " + str(ack2del) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                      
            except:
                message2print = "{(Step 3/4d) yielded" + "; nack2del-yielded-by-load-pop-mark-of-Unpickler: " + str(nack2del)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
        else:
            message2print = "{(Step 3/4e) yielded" + "; nack2del-yielded-by-load-pop-mark-of-Unpickler: " + str(nack2del)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                         
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-pop-mark-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                     
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[POP_MARK[0]] = load_pop_mark
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-dup-of-Unpickler" usage = "loads dup">
    # --------------------------------------------------------------------------------------------- 
    def load_dup(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-dup'
     
        message2print = "{(Step 1/3) started" + "; load-dup-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        try:        
            self.append(self.stack[-1])
            
            message2print = "{(Step 2/3a) performed" + "; append-of-unpickler2xi-performed-by-load-dup-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            ack2append = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2append-acknowledged-by-load-dup-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-dup-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{ (Step 3/3) returning" + "; load-dup-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
        
    #</methodize4xi>
            
    
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[DUP[0]] = load_dup
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-get-of-Unpickler" usage = "loads get">
    # --------------------------------------------------------------------------------------------- 
    def load_get(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-get'
     
        message2print = "{(Step 1/4) started" + "; load-get-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2index = False
        nack2index = True
        try:          
            i = int(self.readline()[:-1])
        
            message2print = "{(Step 2/4a) set" + "; index-set-by-load-get-of-Unpickler to: " + str(i)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            ack2index = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2index-acknowledged-by-load-get-of-Unpickler: " + str(ack2index) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                 
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2index-yielded-by-load-get-of-Unpickler: " + str(nack2index)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        if ack2index is True:    
            try:
                self.append(self.memo[i])
                
                message2print = "{(Step 3/4a) set" + "; memo-of-unpickler2xi-set-by-load-get-of-Unpickler to: " + str(self.memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                ack2append = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2append-acknowledged-by-load-get-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2append-yielded-by-load-get-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2append-yielded-by-load-get-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                          
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4)returning" + "; load-get-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[GET[0]] = load_get
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binget-of-Unpickler" usage = "loads binget">
    # --------------------------------------------------------------------------------------------- 
    def load_binget(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binget'
     
        message2print = "{(Step 1/4) started" + "; load-binget-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2index = False
        nack2index = True
        try:             
            i = self.read(1)[0]
        
            message2print = "{(Step 2/4a) set" + "; index-set-by-load-binget-of-Unpickler to: " + str(i)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2index = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2index-acknowledged-by-load-binget-of-Unpickler: " + str(ack2index) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                  
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2index-yielded-by-load-binget-of-Unpickler: " + str(nack2index)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2memo = False
        nack2memo = True
        if ack2index is True:      
        
            try:
        
                self.append(self.memo[i])
        
                message2print = "{(Step 3/4a) set" + "; memo-set-by-load-binget-of-Unpickler to: " + str(self.memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                ack2memo = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2memo-acknowledged-by-load-binget-of-Unpickler: " + str(ack2memo) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                      
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2memo-yielded-by-load-binget-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                 
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2memo-yielded-by-load-binget-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        
        message2print = "{(Step 4/4) returning" + "; load-binget-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                
              
    #</methodize4xi>
 
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BINGET[0]] = load_binget
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-long-binget-of-Unpickler" usage = "loads long binget">
    # --------------------------------------------------------------------------------------------- 
    def load_long_binget(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-long-binget'
     
        message2print = "{(Step 1/4) started" + "; load-long-binget-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2index = False
        nack2index = True
        try:        
            i = mloads(b'i' + self.read(4))
        
            message2print = "{(Step 2/4a) set" + "; index-set-by-load-long-binget-of-Unpickler to: " + str(i)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            ack2index = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2index-acknowledged-by-load-long-binget-of-Unpickler: " + str(ack2index) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                
        except:
            message2print = "{(Step 2/4b) yielded" + "; nack2index-yielded-by-load-long-binget-of-Unpickler: " + str(nack2index)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # ----------------------------------------------------------------------------------------- 
        ack2append = False
        nack2append = True
        if ack2index is True:
        
            try:              
                self.append(self.memo[i])
                
                message2print = "{(Step 3/4a) performed" + "; append-of-unpickler2xi-performed-by-load-long-binget-of-Unpickler: " + str() + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                
                ack2append = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2append-acknowledged-by-load-long-binget-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2append-yielded-by-load-long-binget-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2append-yielded-by-load-long-binget-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                         
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 4/4) returning" + "; load-long-binget-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                             
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[LONG_BINGET[0]] = load_long_binget
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-put-of-Unpickler" usage = "loads put">
    # --------------------------------------------------------------------------------------------- 
    def load_put(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-put'
     
        message2print = "{(Step 1/4) started" + "; load-put-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
         
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2index = False
        nack2index = True
        try:        
            i = int(self.readline()[:-1])
            
            message2print = "{(Step 2/4a) set" + "; index-set-by-load-put-of-Unpickler to: " + str(i)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2index = True 
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2index-acknowledged-by-load-put-of-Unpickler: " + str(ack2index) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                              
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2index-yielded-by-load-put-of-Unpickler: " + str(nack2index)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        ack2memo = False
        nack2memo = True
        if ack2index is True:    
        
            try:
                self.memo[i] = self.stack[-1]
                
                message2print = "{(Step 3/4a) set" + "; memo-of-unpickler2xi-set-by-load-put-of-Unpickler to: " + str(self.memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
       
                ack2memo = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2memo-acknowledged-by-load-put-of-Unpickler: " + str(ack2memo) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2memo-yielded-by-load-put-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2memo-yielded-by-load-put-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                     
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-put-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                
        
                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[PUT[0]] = load_put
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-binput-of-Unpickler" usage = "loads binput">
    # --------------------------------------------------------------------------------------------- 
    def load_binput(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-binput'
     
        message2print = "{(Step 1/4) started" + "; load-binput-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2index = False
        nack2index = True
        try:         
            i = self.read(1)[0]
        
            message2print = "{(Step 2/4a) set" + "; index-set-by-load-binput-of-Unpickler to: " + str(i)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            ack2index = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2index-acknowledged-by-load-binput-of-Unpickler: " + str(ack2index) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                   
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2index-yielded-by-load-binput-of-Unpickler: " + str(nack2index)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
                            
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        
        ack2memo = False
        nack2memo = True
        if ack2index is True:
        
            try:   
                self.memo[i] = self.stack[-1]
                
                message2print = "{(Step 3/4a) set" + "; memo-of-unpickler2xi-set-by-load-binput-of-Unpickler to: " + str(self.memo[i])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                ack2memo = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2memo-acknowledged-by-load-binput-of-Unpickler: " + str(ack2memo) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2memo-yielded-by-load-binput-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
        else:              
            message2print = "{(Step 3/4d) yielded" + "; nack2memo-yielded-by-load-binput-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                   
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-binput-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        

          
                                   
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BINPUT[0]] = load_binput
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-long-binput-of-Unpickler" usage = "loads long binput">
    # --------------------------------------------------------------------------------------------- 
    def load_long_binput(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-long-binput'
     
        message2print = "{(Step 1/4) started" + "; load-long-binput-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/4)
        # -----------------------------------------------------------------------------------------           
        ack2index = False
        nack2index = True
        try:
               
            i = mloads(b'i' + self.read(4))
        
            message2print = "{(Step 2/4a) set" + "; index-set-by-load-long-binput-of-Unpickler to: " + str(i)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            ack2index = True
            
            message2print = "{(Step 2/4b) acknowledged" + "; ack2index-acknowledged-by-long-binput-of-Unpickler: " + str(ack2index) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                  
        except:
            message2print = "{(Step 2/4c) yielded" + "; nack2index-yielded-by-load-long-binput-of-Unpickler: " + str(nack2index)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                            
                            
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/4)
        # -----------------------------------------------------------------------------------------           
        
        ack2memo = False
        nack2memo = True
        if ack2index is True:    
        
            try:
                self.memo[i] = self.stack[-1]
        
                message2print = "{(Step 3/4a) set" + "; memo-of-unpickler2xi-set-by-load-long-binput-of-Unpickler to: " + str(self.memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                
                                
                ack2memo = True
                
                message2print = "{(Step 3/4b) acknowledged" + "; ack2memo-acknowledged-by-load-long-binput-of-Unpickler: " + str(ack2memo) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                    
            except:
                message2print = "{(Step 3/4c) yielded" + "; nack2memo-yielded-by-load-long-binput-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
        else:
            message2print = "{(Step 3/4d) yielded" + "; nack2memo-yielded-by-load-long-binput-of-Unpickler: " + str(nack2memo)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/4)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 4/4) returning" + "; load-long-binput-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                         
                 
                                    
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[LONG_BINPUT[0]] = load_long_binput
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-append-of-Unpickler" usage = "loads append">
    # --------------------------------------------------------------------------------------------- 
    def load_append(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-append'
     
        message2print = "{(Step 1/6) started" + "; load-append-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/6)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        try:         
            stack = self.stack
    
            message2print = "{(Step 2/6a) set" + "; stack-set-by-load-append-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            ack2stack = True
            
            message2print = "{(Step 2/6b) acknowledged" + "; ack2stack-acknowledged-by-load-append-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                
        except:
            message2print = "{(Step 2/6c) yielded" + "; nack2stack-yielded-by-load-append-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/6)
        # -----------------------------------------------------------------------------------------           
        ack2value = False
        nack2value = True
        if ack2stack is True:    
        
            try:
                value = stack.pop()
            
                message2print = "{(Step 3/6a) set" + "; value-set-by-load-append-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2value = True
                
                message2print = "{(Step 3/6b) acknowledged" + "; ack2value-acknowledged-by-load-append-of-Unpickler: " + str(ack2value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                            
            except:
                message2print = "{(Step 3/6c) yielded" + "; nack2value-yielded-by-load-append-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        
        else:
            message2print = "{(Step 3/6d) yielded" + "; nack2value-yielded-by-load-append-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
                            
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/6)
        # -----------------------------------------------------------------------------------------           
        ack2list = False
        nack2list = True
        if ack2stack is True:    
        
            try:            
                list = stack[-1]
                
                message2print = "{(Step 4/6a) set" + "; value-set-by-load-append-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2list = True
                
                message2print = "{(Step 4/6b) acknowledged" + "; ack2list-acknowledged-by-load-append-of-Unpickler: " + str(ack2list) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                      
            except:
                message2print = "{(Step 4/6c) yielded" + "; nack2list-yielded-by-load-append-of-Unpickler: " + str(nack2list)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        
        else:
            message2print = "{(Step 4/6d) yielded" + "; nack2list-yielded-by-load-append-of-Unpickler: " + str(nack2list)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/6)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        if ((ack2list is True) and (ack2value is True)):    
        
            try: 
                        
                list.append(value)
                
                message2print = "{(Step 5/6a) set" + "; list-set-by-load-append-of-Unpickler to: " + str(list)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2append = True
                
                message2print = "{(Step 5/6b) acknowledged" + "; ack2append-acknowledged-by-load-append-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                   
            except:
                message2print = "{(Step 5/6c) yielded" + "; nack2append-yielded-by-load-append-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        
        else:
            message2print = "{(Step 5/6d) yielded" + "; nack2append-yielded-by-load-append-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/6)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 6/6) returning" + "; load-append-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                
            
    #</methodize4xi>
 
    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[APPEND[0]] = load_append
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-appends-of-Unpickler" usage = "loads appends">
    # --------------------------------------------------------------------------------------------- 
    def load_appends(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-appends'
     
        message2print = "{(Step 1/7) started" + "; load-appends-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True 
        try:     
            stack = self.stack
            
            message2print = "{(Step 2/7a) set" + "; stack-set-by-load-appends-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2stack = True 
            
            message2print = "{(Step 2/7b) acknowledged" + "; ack2stack-acknowledged-by-load-appends-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                               
        except:
            message2print = "{(Step 2/7c) yielded" + "; nack2stack-yielded-by-load-appends-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------           
        ack2mark = False
        nack2mark = True
        try:   
            mark = self.marker()
            
            message2print = "{(Step 3/7a) set" + "; mark-set-by-load-appends-of-Unpickler to: " + str(mark)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2mark = True 
            
            message2print = "{(Step 3/7b) acknowledged" + "; ack2mark-acknowledged-by-load-appends-of-Unpickler: " + str(ack2mark) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                               
        except:
            message2print = "{(Step 3/7c) yielded" + "; nack2mark-yielded-by-load-appends-of-Unpickler: " + str(nack2mark)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                    
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------           
        ack2list = False
        nack2list = True
        if ack2mark is True:
        
            try:
                list = stack[mark - 1]
                
                message2print = "{(Step 4/7a) set" + "; list-set-by-load-appends-of-Unpickler to: " + str(list)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
                ack2list = True
                
                message2print = "{(Step 4/7b) acknowledged" + "; ack2list-acknowledged-by-load-appends-of-Unpickler: " + str(ack2list) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                      
            except:
                message2print = "{(Step 4/7c) yielded" + "; nack2list-yielded-by-load-appends-of-Unpickler: " + str(nack2list)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                         
        else:
            message2print = "{(Step 4/7d) yielded" + "; nack2list-yielded-by-load-appends-of-Unpickler: " + str(nack2list)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                     
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------           
        ack2extend = False
        nack2extend = True
        if ack2list is True:
        
            try:        
                list.extend(stack[mark + 1:])
                
                message2print = "{(Step 5/7a) set" + "; list-set-by-load-appends-of-Unpickler to: " + str(list)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
                ack2extend = True 
                
                message2print = "{(Step 5/7b) acknowledged" + "; ack2extend-acknowledged-by-load-appends-of-Unpickler: " + str(ack2extend) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                     
            except:
                message2print = "{(Step 5/7c) yielded" + "; nack2extend-yielded-by-load-appends-of-Unpickler: " + str(nack2extend)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                         
        else:
            message2print = "{(Step 5/7d) yielded" + "; nack2extend-yielded-by-load-appends-of-Unpickler: " + str(nack2extend)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------           
        ack2del = False
        nack2del = True
        if ((ack2mark is True) and (ack2stack is True)):
        
            try:
                message2print = "{(Step 6/7a) deleting" + "; stack-deleting-by-load-appends-of-Unpickler: " + str(stack[mark:])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)              
            
                del stack[mark:]
                
                ack2del = True
                
                message2print = "{(Step 6/7b) acknowledged" + "; ack2del-acknowledged-by-load-appends-of-Unpickler: " + str(ack2del) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                      
            except:
                message2print = "{(Step 6/7c) yielded" + "; nack2del-yielded-by-load-appends-of-Unpickler: " + str(nack2del)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                         
        else:
            message2print = "{(Step 6/7d) yielded" + "; nack2del-yielded-by-load-appends-of-Unpickler: " + str(nack2del)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 7/7) returning" + "; load-appends-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[APPENDS[0]] = load_appends
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-setitem-of-Unpickler" usage = "loads setitem">
    # --------------------------------------------------------------------------------------------- 
    def load_setitem(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-setitem'
     
        message2print = "{(Step 1/X) started" + "; load-setitem-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        try:       
            stack = self.stack
            
            message2print = "{(Step 2/7a) set" + "; stack-set-by-load-setitem-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            
            ack2stack = True 
            
            message2print = "{(Step 2/7b) acknowledged" + "; ack2stack-acknowledged-by-load-setitem-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                             
        except:
            message2print = "{(Step 2/7c) yielded" + "; nack2stack-yielded-by-load-setitem-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------           
        ack2value = False
        nack2value = True
        if ack2stack is True:    
        
            try:
                value = stack.pop()
        
                message2print = "{(Step 3/7a) set" + "; value-set-by-load-setitem-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2value = True
                
                message2print = "{(Step 3/7b) acknowledged" + "; ack2value-acknowledged-by-load-setitem-of-Unpickler: " + str(ack2value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                    
            except:
                message2print = "{(Step 3/7c) yielded" + "; nack2value-yielded-by-load-setitem-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/7d) yielded" + "; nack2stack-yielded-by-load-setitem-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                
                                
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------           
        ack2key = False
        nack2key = True
        if ack2stack is True:    
        
            try:            
                key = stack.pop()
                
                message2print = "{(Step 4/7a) set" + "; key-set-by-load-setitem-of-Unpickler to: " + str(key)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                 
                ack2key = True
                
                message2print = "{(Step 4/7b) acknowledged" + "; ack2key-acknowledged-by-load-setitem-of-Unpickler: " + str(ack2key) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                
            except:
                message2print = "{(Step 4/7c) yielded" + "; nack2key-yielded-by-load-setitem-of-Unpickler: " + str(nack2key)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 4/7d) yielded" + "; nack2key-yielded-by-load-setitem-of-Unpickler: " + str(nack2key)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------           
        ack2dict = False
        nack2dict = True
        if ack2stack is True:    
        
            try:                
                dict = stack[-1]
                
                message2print = "{(Step 5/7a) set" + "; dict-set-by-load-setitem-of-Unpickler to: " + str(dict)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                 
                ack2dict = True
                
                message2print = "{(Step 5/7b) acknowledged" + "; ack2dict-acknowledged-by-load-setitem-of-Unpickler: " + str(ack2dict) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                 
                
            except:
                message2print = "{(Step 5/7c) yielded" + "; nack2dict-yielded-by-load-setitem-of-Unpickler: " + str(nack2dict)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 5/7d) yielded" + "; nack2dict-yielded-by-load-setitem-of-Unpickler: " + str(nack2dict)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------           
        ack2dict = False
        nack2dict = True 
        if ((ack2dict is True) and (ack2value is True)):
        
            try:    
                dict[key] = value
                
                message2print = "{(Step 6/7a) set" + "; dict-key-set-by-load-setitem-of-Unpickler to: " + str(dict[key])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                
                ack2dict = True
                
                message2print = "{(Step 6/7b) acknowledged" + "; ack2dict-acknowledged-by-load-setitem-of-Unpickler: " + str(ack2dict) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 6/7c) yielded" + "; nack2dict-yielded-by-load-setitem-of-Unpickler: " + str(nack2dict)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 6/7d) yielded" + "; nack2dict-yielded-by-load-setitem-of-Unpickler: " + str(nack2dict)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 7/7) returning" + "; load-setitem-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)             
        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[SETITEM[0]] = load_setitem
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
                        
                
                            
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-setitems-of-Unpickler" usage = "loads setitems">
    # --------------------------------------------------------------------------------------------- 
    def load_setitems(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/7)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-setitems'
     
        message2print = "{(Step 1/7) started" + "; load-setitems-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/7)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True
        try:      
            stack = self.stack
            
            message2print = "{(Step 2/7a) set" + "; stack-set-by-load-setitems-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
           
            ack2stack = True
            
            message2print = "{(Step 2/7b) acknowledged" + "; ack2stack-acknowledged-by-load-setitems-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                  
        except:
            message2print = "{(Step 2/7c) yielded" + "; nack2stack-yielded-by-load-setitems-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

        # ----------------------------------------------------------------------------------------- 
        # (Step 3/7)
        # -----------------------------------------------------------------------------------------           
        ack2mark = False
        nack2mark = True
        try:      
            mark = self.marker()
            
            message2print = "{(Step 3/7a) set" + "; mark-set-by-load-setitems-of-Unpickler to: " + str(mark)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
           
            ack2mark = True
            
            message2print = "{(Step 3/7b) acknowledged" + "; ack2mark-acknowledged-by-load-setitems-of-Unpickler: " + str(ack2mark) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                 
        except:
            message2print = "{(Step 3/7b) yielded" + "; nack2mark-yielded-by-load-setitems-of-Unpickler: " + str(nack2mark)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/7)
        # -----------------------------------------------------------------------------------------           
        ack2dict = False
        nack2dict = True
        if ((ack2mark is True) and (ack2stack is True)):      
            
            try:
                dict = stack[mark - 1]
        
                message2print = "{(Step 4/7a) set" + "; dict-set-by-load-setitems-of-Unpickler to: " + str(dict)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
                ack2dict = True
                
                message2print = "{(Step 4/7b) acknowledged" + "; ack2dict-acknowledged-by-load-setitems-of-Unpickler: " + str(ack2dict) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 4/7c) yielded" + "; nack2mark-yielded-by-load-setitems-of-Unpickler: " + str(nack2mark)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                 
        else:
            message2print = "{(Step 4/7d) yielded" + "; nack2mark-yielded-by-load-setitems-of-Unpickler: " + str(nack2mark)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
            
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/7)
        # -----------------------------------------------------------------------------------------           
        if ack2dict is True:
        
            try:     
                for i in range(mark + 1, len(stack), 2):
                
                    message2print = "{(Step 5/7a1) set" + "; index-set-by-load-setitems-of-Unpickler to: " + str(i)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                    
                
                    dict[stack[i]] = stack[i + 1]
                    
                    message2print = "{(Step 5/7a2) got" + "; stack-element-got-by-load-setitems-of-Unpickler to: " + str(stack[i + 1])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                    
                    message2print = "{(Step 5/7a3) set" + "; dict-stack-set-by-load-setitems-of-Unpickler to: " + str(dict[stack[i]])  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                                    
            except:
                nack2dict = True
                            
                message2print = "{(Step 5/7b) yielded" + "; nack2dict-yielded-by-load-setitems-of-Unpickler: " + str(nack2dict)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                 
        else:
            message2print = "{(Step 5/7c) yielded" + "; nack2dict-yielded-by-load-setitems-of-Unpickler: " + str(nack2dict)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
                
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/7)
        # -----------------------------------------------------------------------------------------           
        ack2del = False
        nack2del = True
        
        if ((ack2stack is True) and (ack2mark is True)):        
        
            try:
                message2print = "{(Step 6/7a) deleting" + "; stack-deleting-by-load-setitems-of-Unpickler to: " + str(stack[mark:])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)              
                
                del stack[mark:]
                
                ack2del = True
                
                message2print = "{(Step 6/7b) acknowledged" + "; ack2del-acknowledged-by-load-setitems-of-Unpickler: " + str(ack2del) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:

                message2print = "{(Step 6/7c) yielded" + "; nack2del-yielded-by-load-setitems-of-Unpickler: " + str(nack2del)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                 
        else:
            message2print = "{(Step 6/7d) yielded" + "; nack2del-yielded-by-load-setitems-of-Unpickler: " + str(nack2del)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
                     
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/7)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 7/7)returning" + "; load-setitems-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                        
                
                                      
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[SETITEMS[0]] = load_setitems
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
         
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-build-of-Unpickler" usage = "loads build">
    # --------------------------------------------------------------------------------------------- 
    def load_build(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/11)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-build'
     
        message2print = "{(Step 1/11) started" + "; load-build-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/11)
        # -----------------------------------------------------------------------------------------           
        ack2stack = False
        nack2stack = True 
        try:                  
            stack = self.stack
            
            message2print = "{(Step 2/11a) set" + "; stack-set-by-load-build-of-Unpickler to: " + str(stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                        
            ack2stack = True 
            
            message2print = "{(Step 2/11b) acknowledged" + "; ack2stack-acknowledged-by-load-build-of-Unpickler: " + str(ack2stack) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                  
        except:
            message2print = "{(Step 2/11c) yielded" + "; nack2stack-yielded-by-load-build-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/11)
        # -----------------------------------------------------------------------------------------           
        ack2state = False
        nack2state = True
        if ack2stack is True:    
            
            try:
                state = stack.pop()
                
              
                message2print = "{(Step 3/11a) set" + "; state-set-by-load-build-of-Unpickler to: " + str(state)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                ack2state = True

                message2print = "{(Step 3/11b) acknowledged" + "; ack2state-acknowledged-by-load-build-of-Unpickler: " + str(ack2state) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                      
                    
            except:
                message2print = "{(Step 3/11c) yielded" + "; nack2state-yielded-by-load-build-of-Unpickler: " + str(nack2state)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
        else:
            message2print = "{(Step 3/11d) yielded" + "; nack2state-yielded-by-load-build-of-Unpickler: " + str(nack2state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
                            
        # ----------------------------------------------------------------------------------------- 
        # (Step 4/11)
        # -----------------------------------------------------------------------------------------           
        ack2inst = False
        nack2inst = True
        
        if ack2stack is True:    
            
            try:            
                inst = stack[-1]
                
                message2print = "{(Step 4/11a) set" + "; inst-set-by-load-build-of-Unpickler to: " + str(inst)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                ack2inst = True
                
                message2print = "{(Step 4/11b) acknowledged" + "; ack2inst-acknowledged-by-load-build-of-Unpickler: " + str(ack2inst) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            except:
                message2print = "{(Step 4/11c) yielded" + "; nack2stack-yielded-by-load-build-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
        else:
            message2print = "{(Step 4/11d) yielded" + "; nack2stack-yielded-by-load-build-of-Unpickler: " + str(nack2stack)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                        
        # ----------------------------------------------------------------------------------------- 
        # (Step 5/11)
        # -----------------------------------------------------------------------------------------           
        ack2setstate = False
        nack2setstate = True
        try:    
            setstate = getattr(inst, "__setstate__", None)
            
            message2print = "{(Step 5/11a) set" + "; setstate-set-by-load-build-of-Unpickler to: " + str(setstate)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
       
            ack2setstate = True
            
            message2print = "{(Step 5/11b) acknowledged" + "; ack2setstate-acknowledged-by-load-build-of-Unpickler: " + str(ack2setstate) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                    
        except:
            message2print = "{(Step 5/11c) yielded" + "; nack2setstate-yielded-by-load-build-of-Unpickler: " + str(nack2setstate)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
        # ----------------------------------------------------------------------------------------- 
        # (Step 6/11)
        # -----------------------------------------------------------------------------------------           
        ack2return = False
        nack2return = True
        
        if ((ack2setstate is True) and (ack2state is True)):    
 
            setstate(state)
            
            message2print = "{(Step 6/11a) performed" + "; setstate-performed-by-load-build-of-Unpickler: " + str(setstate) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            ack2return = True
            
            message2print = "{(Step 6/11b) acknowledged" + "; ack2return-acknowledged-by-yyyyyyyy-of-ZZZZZZZZ: " + str(ack2return) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                       
            message2print = "{(Step 6/11c) returning" + "; load-build-of-Unpickler-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                             
            
            return
        else:
        
            message2print = "{(Step 6/11d) yielded" + "; ack2return-yielded-by-load-build-of-Unpickler: " + str(ack2return)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)              
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 7/11)
        # -----------------------------------------------------------------------------------------           
                
        slotstate = None
        
           
        message2print = "{(Step 7/11) set" + "; slotstate-set-by-load-build-of-Unpickler to: " + str(slotstate)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        # ----------------------------------------------------------------------------------------- 
        # (Step 8/11)
        # -----------------------------------------------------------------------------------------           
        ack2slot = False
        nack2slot = True
        
        if ack2state is True:
        
            try:    
                if isinstance(state, tuple) and len(state) == 2:
                    
                    state, slotstate = state
                    
                 
                    message2print = "{(Step 8/11a) set" + "; state-set-by-load-build-of-Unpickler to: " + str(state)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
            
                    message2print = "{(Step 8/11b) set" + "; slotstate-set-by-load-build-of-Unpickler to: " + str(slotstate)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    
                    ack2slot = True
                    
                    message2print = "{(Step 8/11c) acknowledged" + "; ack2slot-acknowledged-by-load-build-of-Unpickler: " + str(ack2slot) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                   
  
                else:
                    pass
                    
            except:
                message2print = "{(Step 8/11d) yielded" + "; nack2state-yielded-by-load-build-of-Unpickler: " + str(nack2state)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                       
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 9/11)
        # -----------------------------------------------------------------------------------------           
        if ack2state is True:
        
            try:                   
                if state:
                    inst.__dict__.update(state)
                    
                    message2print = "{(Step 9/11a) performed" + "; inst-dict-update-performed-by-load-build-of-Unpickler: " + str(inst.__dict__.update) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    
                        
            except:
                message2print = "{(Step 9/11b) yielded" + "; nack2state-yielded-by-load-build-of-Unpickler: " + str(nack2state)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
        else:
            message2print = "{(Step 9/11d) yielded" + "; nack2state-yielded-by-load-build-of-Unpickler: " + str(nack2state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                            
        # ----------------------------------------------------------------------------------------- 
        # (Step 10/11)
        # -----------------------------------------------------------------------------------------           
        if ack2slot is True:
        
            try:                  
                if slotstate:
                    for k, v in slotstate.items():
                    
                        message2print = "{(Step 10/11a) passing" + "; inst-passing-to-setattr-by-load-build-of-Unpickler: " + str() + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
                                        
                        message2print = "{(Step 10/11b) passing" + "; k-passing-to-setattr-by-load-build-of-Unpickler: " + str(k) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                                            
                        
                        message2print = "{(Step 10/11c) passing" + "; v-passing-to-setattr-by-load-build-of-Unpickler: " + str(v) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                      
                        
                        setattr(inst, k, v)
                        
                        message2print = "{(Step 10/11d) performed" + "; setattr-performed-by-load-build-of-Unpickler: " + str(setattr) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                        
            except:
                message2print = "{(Step 10/11e) yielded" + "; nack2slot-yielded-by-load-build-of-Unpickler: " + str(nack2slot)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
        else:
            message2print = "{(Step 10/11f) yielded" + "; nack2slot-yielded-by-load-build-of-Unpickler: " + str(nack2slot)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
             
        # ----------------------------------------------------------------------------------------- 
        # (Step 11/11)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 11/11) returning" + "; load-build-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[BUILD[0]] = load_build
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
     
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-mark-of-Unpickler" usage = "loads mark">
    # --------------------------------------------------------------------------------------------- 
    def load_mark(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-mark'
     
        message2print = "{(Step 1/3) started" + "; load-mark-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2append = False
        nack2append = True
        
        try:
            message2print = "{(Step 2/3a) passing" + "; mark-of-unpickler2xi-passing-to-append-by-load-mark-of-Unpickler: " + str(self.mark) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
                              
            self.append(self.mark)
            
            message2print = "{(Step 2/3b) performed" + "; append-of-unpickler2xi-performed-by-load-mark-of-Unpickler: " + str(self.append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            ack2append = True
            
            message2print = "{(Step 2/3c) acknowledged" + "; ack2append-acknowledged-by-load-mark-of-Unpickler: " + str(ack2append) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2append-yielded-by-load-mark-of-Unpickler: " + str(nack2append)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
          
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        message2print = "{(Step 3/3) returning" + "; load-mark-of-Unpickler-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                                
                        
                                     
        
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<define4xi definition = "
    dispatch[MARK[0]] = load_mark
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "load-stop-of-Unpickler" usage = "loads stop">
    # --------------------------------------------------------------------------------------------- 
    def load_stop(self):
    
        # ----------------------------------------------------------------------------------------- 
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------           
    
        class2print = 'Unpickler'
        method2print = 'load-stop'
     
        message2print = "{(Step 1/3) started" + "; load-stop-of-Unpickler-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 2/3)
        # -----------------------------------------------------------------------------------------           
        ack2value = False
        nack2value = True
        try:
            value = self.stack.pop()
        
         
            message2print = "{(Step 2/3a) set" + "; value-set-by-load-stop-of-Unpickler to: " + str(value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            ack2value = True
            
            message2print = "{(Step 2/3b) acknowledged" + "; ack2value-acknowledged-by-load-stop-of-Unpickler: " + str(ack2value) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                           
        except:
            message2print = "{(Step 2/3c) yielded" + "; nack2value-yielded-by-load-stop-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
        # ----------------------------------------------------------------------------------------- 
        # (Step 3/3)
        # -----------------------------------------------------------------------------------------           
        if ack2value is True:  
            raise _Stop(value)
            
        else:
            message2print = "{(Step 3/3) yielded" + "; nack2value-yielded-by-load-stop-of-Unpickler: " + str(nack2value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)        
        
    #</methodize4xi>
  
    #<define4xi definition = "
    dispatch[STOP[0]] = load_stop
    #"
    #/>
    # --------------------------------------------------------------------------------------------- 
#</class4xi>

            
# -------------------------------------------------------------------------------------------------
#<class4xi class = "EmptyClass" base = "None" child = "None" synopsis = "Offer empty class"
#   usage = "Helper class for load_inst/load_obj"
#>
# -------------------------------------------------------------------------------------------------
# Helper class for load_inst/load_obj
class _EmptyClass:
    pass
#</class4xi>

 
    
    
# Encode/decode longs in linear time.



# -------------------------------------------------------------------------------------------------
#<function4xi function = "encode_long" usage = "
#   Encode a long to a two's complement little-endian binary string
#"> 
# -------------------------------------------------------------------------------------------------
def encode_long(x):
    r"""Encode a long to a two's complement little-endian binary string.
    Note that 0 is a special case, returning an empty string, to save a
    byte in the LONG1 pickling context.

    >>> encode_long(0)
    b''
    >>> encode_long(255)
    b'\xff\x00'
    >>> encode_long(32767)
    b'\xff\x7f'
    >>> encode_long(-256)
    b'\x00\xff'
    >>> encode_long(-32768)
    b'\x00\x80'
    >>> encode_long(-128)
    b'\x80'
    >>> encode_long(127)
    b'\x7f'
    >>>
    """
    
    # ----------------------------------------------------------------------------------------- 
    #(Step 1/2) 
    # -----------------------------------------------------------------------------------------           
    
    class2print = 'ForkinPickler4Xi'
    method2print = 'encode_long'
     
    message2print = "{(Step 1/2a) started" + "; encode-long-of-ForkinPickler4Xi-started: " + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    
    message2print = "{(Step 1/2b) passed" + "; x-passed-to-encode-long-of-ForkinPickler4Xi: " + str(x) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
        
    if x == 0:
        # ------------------------------------------------------------------------------------- 
        # (Step 2/2a) 
        # -------------------------------------------------------------------------------------       
        message2print = "{(Step 2/2a) returning" + "; encode-long-of-ForkinPickler4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        
           
        return b''
    if x > 0:
        ashex = hex(x)
        assert ashex.startswith("0x")
        njunkchars = 2 + ashex.endswith('L')
        nibbles = len(ashex) - njunkchars
        if nibbles & 1:
            # need an even # of nibbles for unhexlify
            ashex = "0x0" + ashex[2:]
        elif int(ashex[2], 16) >= 8:
            # "looks negative", so need a byte of sign bits
            ashex = "0x00" + ashex[2:]
    else:
        # Build the 256's-complement:  (1L << nbytes) + x.  The trick is
        # to find the number of bytes in linear time (although that should
        # really be a constant-time task).
        ashex = hex(-x)
        assert ashex.startswith("0x")
        njunkchars = 2 + ashex.endswith('L')
        nibbles = len(ashex) - njunkchars
        if nibbles & 1:
            # Extend to a full byte.
            nibbles += 1
        nbits = nibbles * 4
        x += 1 << nbits
        assert x > 0
        ashex = hex(x)
        njunkchars = 2 + ashex.endswith('L')
        newnibbles = len(ashex) - njunkchars
        if newnibbles < nibbles:
            ashex = "0x" + "0" * (nibbles - newnibbles) + ashex[2:]
        if int(ashex[2], 16) < 8:
            # "looks positive", so need a byte of sign bits
            ashex = "0xff" + ashex[2:]

    if ashex.endswith('L'):
        ashex = ashex[2:-1]
    else:
        ashex = ashex[2:]
    assert len(ashex) & 1 == 0, (x, ashex)
    binary = _binascii.unhexlify(ashex)
    
    # ------------------------------------------------------------------------------------- 
    # (Step 2/2b) 
    # -------------------------------------------------------------------------------------       
    message2print = "{(Step 2/2b) returning" + "; encode-long-of-ForkinPickler4Xi-returning: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    
    return bytes(binary[::-1])
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "decode-long" usage = "
#   Decode a long from a two's complement little-endian binary string
#"> 
# -------------------------------------------------------------------------------------------------
def decode_long(data):
    r"""Decode a long from a two's complement little-endian binary string.

    >>> decode_long(b'')
    0
    >>> decode_long(b"\xff\x00")
    255
    >>> decode_long(b"\xff\x7f")
    32767
    >>> decode_long(b"\x00\xff")
    -256
    >>> decode_long(b"\x00\x80")
    -32768
    >>> decode_long(b"\x80")
    -128
    >>> decode_long(b"\x7f")
    127
    """
    
    # ----------------------------------------------------------------------------------------- 
    #(Step 1/2) 
    # -----------------------------------------------------------------------------------------           
    
    class2print = 'ForkinPickler4Xi'
    method2print = 'decode_long'
     
    message2print = "{(Step 1/2a) started" + "; decode-long-of-ForkinPickler4Xi-started: " + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    message2print = "{(Step 1/2b) passed" + "; data-passed-to-decode-long-of-ForkinPickler4Xi: " + str(data) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
        
    nbytes = len(data)
    if nbytes == 0:
        return 0
    ashex = _binascii.hexlify(data[::-1])
    n = int(ashex, 16) # quadratic time before Python 2.3; linear now
    if data[-1] >= 0x80:
        n -= 1 << (nbytes * 8)
    
    # ----------------------------------------------------------------------------------------- 
    #(Step 2/2) 
    # -----------------------------------------------------------------------------------------           
    message2print = "{(Step 2/2) returning" + "; n-returning-by-decode-long-of-ForkinPickler4X: " + str(n) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
                                    
        
    return n
#</function4xi>





# -------------------------------------------------------------------------------------------------
#<function4xi function = "dump" usage = "
#   embedded inside forkingpickler4xi.py
#"> 
# -------------------------------------------------------------------------------------------------
def dump(obj, file, protocol=None):
    '''
    dump embedded
    '''
    
    # ----------------------------------------------------------------------------------------- 
    # (Step 1/2) 
    # -----------------------------------------------------------------------------------------     
    
    class2print = 'ForkinPickler4Xi'
    method2print = 'dump'
     
    message2print = "{(Step 1/2a) started" + "; dump-embedded-of-ForkinPickler4Xi-started: " + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    message2print = "{(Step 1/2b) passed" + "; object-passed-to-dump-embedded-of-ForkinPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
    
    message2print = "{(Step 1/2c) passed" + "; file-passed-to-dump-embedded-of-ForkinPickler4Xi: " + str(file) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    
    message2print = "{(Step 1/2d) passed" + "; protocol-passed-to-dump-embedded-of-ForkinPickler4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            

    # ----------------------------------------------------------------------------------------- 
    # (Step 2/2)
    # -----------------------------------------------------------------------------------------     
    message2print = "{(Step 2/2a) passing" + "; file-passing-to-ForkingPickler4Xi-by-dump-embedded-of-ForkinPickler4Xi: " + str(file) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
                 
    message2print = "{(Step 2/2b) passing" + "; protocol-passing-to-ForkingPickler4Xi-by-dump-embedded-of-ForkinPickler4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    
    message2print = "{(Step 2/2c) passing" + "; object-passing-to-dump-of-ForkingPickler4Xi-by-dump-embedded-of-ForkinPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                            
    
    message2print = "{(Step 1/2d) returning" + "; dump-method-of-ForkingPickler4Xi-returning-by-dump-embedded-of-ForkinPickler4Xi: " + str(dump) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)  
                    
    
    ForkingPickler4Xi(file, protocol).dump(obj)
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "dumps" usage = "
#   embedded inside forkingpickler4xi.py
#"> 
# -------------------------------------------------------------------------------------------------
def dumps(obj, protocol=None):
    
    # ----------------------------------------------------------------------------------------- 
    # (Step 1/6) 
    # -----------------------------------------------------------------------------------------     
    
    class2print = 'ForkinPickler4Xi'
    method2print = 'dumps'
     
    message2print = "{(Step 1/6a) started" + "; dumps-embedded-of-ForkinPickler4Xi-started: " + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    message2print = "{(Step 1/6b) passed" + "; object-passed-to-dumps-embedded-of-ForkinPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
    
    message2print = "{(Step 1/6c) passed" + "; protocol-passed-to-dumps-embedded-of-ForkinPickler4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   

    # ----------------------------------------------------------------------------------------- 
    # (Step 2/6)
    # -----------------------------------------------------------------------------------------
         
    f = io.BytesIO()

    message2print = "{(Step 2/6) set" + "; file-set-by-dumps-embedded-of-ForkinPickler4Xi to: " + str(f)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    
    
    
    # ----------------------------------------------------------------------------------------- 
    # (Step 3/6)
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 3/6a) passing" + "; file-passing-to-ForkingPickler4Xi-by-dumps-embedded-of-ForkinPickler4Xi: " + str(f) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
                    
    message2print = "{(Step 3/6b) passing" + "; protocol-passing-to-ForkingPickler4Xi-by-dumps-embedded-of-ForkinPickler4Xi: " + str(protocol) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    
    message2print = "{(Step 3/6c) passing" + "; object-passing-to-dump-of-ForkingPickler4Xi-by-dumps-embedded-of-ForkinPickler4Xi: " + str(obj) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
               
    ForkingPickler4Xi(f, protocol).dump(obj) #dragonized by embedding Pickler to ForkingPickler4Xi

    
    # ----------------------------------------------------------------------------------------- 
    # (Step 4/6)
    # -----------------------------------------------------------------------------------------
    res = None
           
    res = f.getvalue()
    
    message2print = "{(Step 4/6) set" + "; res-set-by-dumps-embedded-of-ForkinPickler4Xi to: " + str(res)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    
    

    # ----------------------------------------------------------------------------------------- 
    # (Step 5/6)
    # -----------------------------------------------------------------------------------------
    ack2instance = False
    nack2instance = True
           
    if isinstance(res, bytes_types):
    
        ack2instance = True
            
        message2print = "{(Step 5/6a) acknowledged" + "; ack2instance-acknowledged-by-dumps-embedded-of-ForkinPickler4Xi: " + str(ack2instance) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
    else:
        message2print = "{(Step 5/6b) warned" + "; nack2instance-warned-by-dumps-embedded-of-ForkinPickler4Xi: " + " ** " + str(nack2instance)  + " ** " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        

    # ----------------------------------------------------------------------------------------- 
    # (Step 6/6)
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 6/6) returning" + "; res-returning-by-dumps-embedded-of-ForkinPickler4Xi: " + str(res) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
                                   
    return res
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "load" usage = "
#   embedded inside forkingpickler4xi.py
#"> 
# -------------------------------------------------------------------------------------------------
def load(file, *, encoding="ASCII", errors="strict"):

    # ----------------------------------------------------------------------------------------- 
    # (Step 1/2) 
    # -----------------------------------------------------------------------------------------     
    
    class2print = 'ForkinPickler4Xi'
    method2print = 'load'
     
    message2print = "{(Step 1/2a) started" + "; load-embedded-of-ForkinPickler4Xi-started: " + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    message2print = "{(Step 1/2b) passed" + "; file-passed-to-load-embedded-of-ForkinPickler4Xi: " + str(file) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
    
    message2print = "{(Step 1/2c) passed" + "; encoding-passed-to-load-embedded-of-ForkinPickler4Xi: " + str(encoding) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   

    message2print = "{(Step 1/2d) passed" + "; errors-passed-to-load-embedded-of-ForkinPickler4Xi: " + str(errors) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       

    # ----------------------------------------------------------------------------------------- 
    # (Step 2/2) 
    # -----------------------------------------------------------------------------------------     
    message2print = "{(Step 2/2a) passing" + "; file-passing-to-Unpickler-embedded-by-load-embedded-of-ForkinPickler4Xi: " + str(file) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    
    message2print = "{(Step 2/2b) passing" + "; encoding-passing-to-Unpickler-embedded-by-load-embedded-of-ForkinPickler4Xi: " + str(encoding) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                  
    
    message2print = "{(Step 2/2c) passing" + "; errors-passing-to-Unpickler-embedded-by-load-embedded-of-ForkinPickler4Xi: " + str(errors) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                          

    message2print = "{(Step 2/2d) returning" + "; Unpickler-returning-by-load-embedded-of-ForkinPickler4Xi: " + str(_Unpickler) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)  
                    
    
    return _Unpickler(file, encoding=encoding, errors=errors).load() #dragonized by embeddeding in module
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "loads" usage = "
#   embedded inside forkingpickler4xi.py
#"> 
# -------------------------------------------------------------------------------------------------
def loads(s, *, encoding="ASCII", errors="strict"):


    # ----------------------------------------------------------------------------------------- 
    # (Step 1/4) 
    # -----------------------------------------------------------------------------------------     
    
    class2print = 'ForkinPickler4Xi'
    method2print = 'loads'
     
    message2print = "{(Step 1/4a) started" + "; loads-embedded-of-ForkinPickler4Xi-started: " + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    message2print = "{(Step 1/4b) passed" + "; string-passed-to-loads-embedded-of-ForkinPickler4Xi: " + str(s) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)      
    
    message2print = "{(Step 1/4c) passed" + "; encoding-passed-to-loads-embedded-of-ForkinPickler4Xi: " + str(encoding) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   

    message2print = "{(Step 1/4d) passed" + "; errors-passed-to-loads-embedded-of-ForkinPickler4Xi: " + str(errors) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    

    # ----------------------------------------------------------------------------------------- 
    # (Step 2/4) 
    # -----------------------------------------------------------------------------------------     
            
    if isinstance(s, str):
 
        message2print = "{(Step 2/4) warned" + "; s-warned-by-loads-embedded-of-ForkinPickler4Xi: " + " ** " + str(s) + " - challenge to load pickle from unicode string " + " ** " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        

    # ----------------------------------------------------------------------------------------- 
    # (Step 3/4) 
    # -----------------------------------------------------------------------------------------     
                     
    file = io.BytesIO(s)
    
    message2print = "{(Step 3/4) set" + "; file-set-by-loads-embedded-of-ForkinPickler4Xi to: " + str(file)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    
    

    # ----------------------------------------------------------------------------------------- 
    # (Step 4/4) 
    # -----------------------------------------------------------------------------------------     
    message2print = "{(Step 4/4a) passing" + "; file-passing-to-load-of-Unpickler-embedded-by-loads-embedded-of-ForkinPickler4Xi: " + str(file) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    
    message2print = "{(Step 4/4b) passing" + "; encoding-passing-to-load-of-Unpickler-embedded-by-loads-embedded-of-ForkinPickler4Xi: " + str(encoding) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                  
    
    message2print = "{(Step 4/4c) passing" + "; errors-passing-to-load-of-Unpickler-embedded-by-loads-embedded-of-ForkinPickler4Xi: " + str(errors) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                          

    message2print = "{(Step 4/4d) returning" + "; load-of-Unpickler-returning-by-load-embedded-of-ForkinPickler4Xi: " + str(_Unpickler) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)  
                               
    
    return _Unpickler(file, encoding=encoding, errors=errors).load() #dragonized by embeddeding in module
#</function4xi>




#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#
#   hello_world4xi.xml.txt
#   sense-hello-world4xi.xml
#   reason-hello-world4xi.xml
#   ace-hello-world4xi.xml
#
#   test_token4xi.xml.txt
#   sense-token4xi.xml
#   reason-tokenxi.xml
#   ace-token4xi.xml

#"
#>



# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
      
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of forkingpickler4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"

